@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Basemenu.cc,v 1.21 2002/04/09 12:22:06 fluxgen Exp $
+// $Id: Basemenu.cc,v 1.28 2002/08/04 15:19:34 fluxgen Exp $
 
 //use GNU extensions
 #ifndef	 _GNU_SOURCE
@@ -58,9 +58,9 @@ static Basemenu *shown = (Basemenu *) 0;
 Basemenu::Basemenu(BScreen *screen):
 m_fluxbox(Fluxbox::instance()),
 m_screen(screen),
-m_display(m_fluxbox->getXDisplay()),
+m_display(Fluxbox::instance()->getXDisplay()),
 m_parent(0),
-m_image_ctrl(m_screen->getImageControl()),
+m_image_ctrl(screen->getImageControl()),
 m_alignment(ALIGNDONTCARE) {
 
 	title_vis =
@@ -95,12 +95,12 @@ m_alignment(ALIGNDONTCARE) {
 	
 	if (i18n->multibyte()) {
 		menu.width = menu.title_h = menu.item_w = menu.frame_h =
-			m_screen->getMenuStyle()->titlefont->getFontSetExtents()->max_ink_extent.height +
+			m_screen->getMenuStyle()->titlefont.fontSetExtents()->max_ink_extent.height +
 			(menu.bevel_w	* 2);
 	} else {
 		menu.width = menu.title_h = menu.item_w = menu.frame_h =
-			m_screen->getMenuStyle()->titlefont->getFontStruct()->ascent +
-			m_screen->getMenuStyle()->titlefont->getFontStruct()->descent +
+			m_screen->getMenuStyle()->titlefont.fontStruct()->ascent +
+			m_screen->getMenuStyle()->titlefont.fontStruct()->descent +
 			(menu.bevel_w * 2);
 	}
 	
@@ -109,11 +109,11 @@ m_alignment(ALIGNDONTCARE) {
 		menu.minsub = 0;
 	
 	if (i18n->multibyte()) {
-		menu.item_h = m_screen->getMenuStyle()->framefont->getFontSetExtents()->max_ink_extent.height +
+		menu.item_h = m_screen->getMenuStyle()->framefont.fontSetExtents()->max_ink_extent.height +
 			(menu.bevel_w);
 	} else {
-		menu.item_h = m_screen->getMenuStyle()->framefont->getFontStruct()->ascent +
-			m_screen->getMenuStyle()->framefont->getFontStruct()->descent +
+		menu.item_h = m_screen->getMenuStyle()->framefont.fontStruct()->ascent +
+			m_screen->getMenuStyle()->framefont.fontStruct()->descent +
 			(menu.bevel_w);
 	}
 	menu.height = menu.title_h + m_screen->getBorderWidth() + menu.frame_h;
@@ -124,8 +124,8 @@ m_alignment(ALIGNDONTCARE) {
 	XSetWindowAttributes attrib;
 	attrib.background_pixmap = None;
 	attrib.background_pixel = attrib.border_pixel =
-		m_screen->getBorderColor()->getPixel();
-	attrib.colormap = m_screen->getColormap();
+		m_screen->getBorderColor()->pixel();
+	attrib.colormap = m_screen->colormap();
 	attrib.override_redirect = True;
 	attrib.event_mask = ButtonPressMask | ButtonReleaseMask |
 		ButtonMotionMask | ExposureMask;
@@ -140,7 +140,7 @@ m_alignment(ALIGNDONTCARE) {
 	
 	//attibutes for title to menuwindow
 	attrib_mask = CWBackPixmap | CWBackPixel | CWBorderPixel | CWEventMask;
-	attrib.background_pixel = m_screen->getBorderColor()->getPixel();
+	attrib.background_pixel = m_screen->getBorderColor()->pixel();
 	attrib.event_mask |= EnterWindowMask | LeaveWindowMask;
 	//create menu title
 	menu.title =
@@ -260,16 +260,16 @@ void Basemenu::update(void) {
 	I18n *i18n = I18n::instance();
 	
 	if (i18n->multibyte()) {
-		menu.item_h = m_screen->getMenuStyle()->framefont->getFontSetExtents()->max_ink_extent.height +
+		menu.item_h = m_screen->getMenuStyle()->framefont.fontSetExtents()->max_ink_extent.height +
 			menu.bevel_w;
-		menu.title_h =	m_screen->getMenuStyle()->titlefont->getFontSetExtents()->max_ink_extent.height +
+		menu.title_h =	m_screen->getMenuStyle()->titlefont.fontSetExtents()->max_ink_extent.height +
 			(menu.bevel_w * 2);
 	} else {
-		menu.item_h = m_screen->getMenuStyle()->framefont->getFontStruct()->ascent +
-			m_screen->getMenuStyle()->framefont->getFontStruct()->descent +
+		menu.item_h = m_screen->getMenuStyle()->framefont.fontStruct()->ascent +
+			m_screen->getMenuStyle()->framefont.fontStruct()->descent +
 			menu.bevel_w;
-		menu.title_h =	m_screen->getMenuStyle()->titlefont->getFontStruct()->ascent +
-				m_screen->getMenuStyle()->titlefont->getFontStruct()->descent +
+		menu.title_h =	m_screen->getMenuStyle()->titlefont.fontStruct()->ascent +
+				m_screen->getMenuStyle()->titlefont.fontStruct()->descent +
 				(menu.bevel_w * 2);
 	}
 		
@@ -279,14 +279,8 @@ void Basemenu::update(void) {
 				 FBNLS::BasemenuSet, FBNLS::BasemenuBlackboxMenu,
 				 "fluxbox Menu");
 		int l = strlen(s);
-		
-
-		if (i18n->multibyte()) {
-			XRectangle ink, logical;
-			XmbTextExtents(m_screen->getMenuStyle()->titlefont->getFontSet(), s, l, &ink, &logical);
-			menu.item_w = logical.width;
-		} else
-			menu.item_w = XTextWidth(m_screen->getMenuStyle()->titlefont->getFontStruct(), s, l);
+	
+		menu.item_w = m_screen->getMenuStyle()->titlefont.textWidth(s, l);
 		
 		menu.item_w += (menu.bevel_w * 2);
 	}	else
@@ -301,12 +295,8 @@ void Basemenu::update(void) {
 		const char *s = itmp->label().c_str();
 		int l = itmp->label().size();
 
-		if (i18n->multibyte()) {
-			XRectangle ink, logical;
-			XmbTextExtents(m_screen->getMenuStyle()->framefont->getFontSet(), s, l, &ink, &logical);
-			ii = logical.width;
-		} else
-			ii = XTextWidth(m_screen->getMenuStyle()->framefont->getFontStruct(), s, l);
+		ii = screen()->getMenuStyle()->framefont.textWidth(s, l);
+			
 
 		ii += (menu.bevel_w * 2) + (menu.item_h * 2);
 
@@ -340,14 +330,14 @@ void Basemenu::update(void) {
 	if (menu.height < 1) menu.height = 1;
 
 	Pixmap tmp;
-	BTexture *texture;
+	FbTk::Texture *texture;
 	if (title_vis) {
 		tmp = menu.title_pixmap;
 		texture = &(m_screen->getMenuStyle()->title);
-		if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+		if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			menu.title_pixmap = None;
 			XSetWindowBackground(m_display, menu.title,
-				 texture->getColor()->getPixel());
+				 texture->color().pixel());
 		} else {
 			menu.title_pixmap =
 				m_image_ctrl->renderImage(menu.width, menu.title_h, texture);
@@ -359,10 +349,10 @@ void Basemenu::update(void) {
 
 	tmp = menu.frame_pixmap;
 	texture = &(m_screen->getMenuStyle()->frame);
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 		menu.frame_pixmap = None;
 		XSetWindowBackground(m_display, menu.frame,
-			 texture->getColor()->getPixel());
+			 texture->color().pixel());
 	} else {
 		menu.frame_pixmap =
 			m_image_ctrl->renderImage(menu.width, menu.frame_h, texture);
@@ -372,7 +362,7 @@ void Basemenu::update(void) {
 
 	tmp = menu.hilite_pixmap;
 	texture = &(m_screen->getMenuStyle()->hilite);
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID))
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID))
 		menu.hilite_pixmap = None;
 	else
 		menu.hilite_pixmap =
@@ -380,7 +370,7 @@ void Basemenu::update(void) {
 	if (tmp) m_image_ctrl->removeImage(tmp);
 
 	tmp = menu.sel_pixmap;
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID))
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID))
 		menu.sel_pixmap = None;
 	else {
 		int hw = menu.item_h / 2;
@@ -439,7 +429,8 @@ void Basemenu::hide(void) {
 	if ((! torn) && hide_tree && m_parent && m_parent->isVisible()) {
 		Basemenu *p = m_parent;
 
-		while (p->isVisible() && (! p->torn) && p->m_parent) p = p->m_parent;
+		while (p->isVisible() && (! p->torn) && p->m_parent)
+			p = p->m_parent;
 		p->internal_hide();
 	} else
 		internal_hide();
@@ -459,7 +450,7 @@ void Basemenu::internal_hide(void) {
 	} else if (shown && shown->menu.window == menu.window)
 		shown = (Basemenu *) 0;
 
-	torn = visible = False;
+	torn = visible = false;
 	which_sub = which_press = which_sub = -1;
 
 	XUnmapWindow(m_display, menu.window);
@@ -484,11 +475,10 @@ void Basemenu::redrawTitle(void) {
 	int dx = menu.bevel_w, len = strlen(text);
 	unsigned int l;
 
-	l = m_screen->getMenuStyle()->titlefont->getTextWidth(text, len);
+	l = m_screen->getMenuStyle()->titlefont.textWidth(text, len);
 
 	l += (menu.bevel_w * 2);
 
-	//titlefont.justify
 	switch (m_screen->getMenuStyle()->titlefont_justify) {
 	case DrawUtil::Font::RIGHT:
 		dx += menu.width - l;
@@ -502,13 +492,13 @@ void Basemenu::redrawTitle(void) {
 	}
 
 	if (i18n->multibyte())
-		XmbDrawString(m_display, menu.title, m_screen->getMenuStyle()->titlefont->getFontSet(),
+		XmbDrawString(m_display, menu.title, m_screen->getMenuStyle()->titlefont.fontSet(),
 			m_screen->getMenuStyle()->t_text_gc, dx, menu.bevel_w -
-			m_screen->getMenuStyle()->titlefont->getFontSetExtents()->max_ink_extent.y,
+			m_screen->getMenuStyle()->titlefont.fontSetExtents()->max_ink_extent.y,
 			text, len);
 	else
 		XDrawString(m_display, menu.title, m_screen->getMenuStyle()->t_text_gc, dx,
-		m_screen->getMenuStyle()->titlefont->getFontStruct()->ascent + menu.bevel_w,
+		m_screen->getMenuStyle()->titlefont.fontStruct()->ascent + menu.bevel_w,
 		text, len);
 }
 
@@ -635,7 +625,8 @@ bool Basemenu::hasSubmenu(unsigned int index) const {
 void Basemenu::drawItem(unsigned int index, bool highlight, bool clear,
 			 int x, int y, unsigned int w, unsigned int h)
 {
-	if (index >= menuitems.size()) return;
+	if (index >= menuitems.size() || menuitems.size() == 0)
+		return;
 
 	BasemenuItem *item = menuitems[index];
 	if (! item) return;
@@ -652,18 +643,17 @@ void Basemenu::drawItem(unsigned int index, bool highlight, bool clear,
 	I18n *i18n = I18n::instance();
 	
 	if (text) {		
-		text_w = m_screen->getMenuStyle()->framefont->getTextWidth(text, len);
+		text_w = m_screen->getMenuStyle()->framefont.textWidth(text, len);
 		
-		if (m_screen->getMenuStyle()->framefont->multibyte()) {
+		if (FbTk::Font::multibyte()) {
 			text_y = item_y + (menu.bevel_w / 2) -
-				m_screen->getMenuStyle()->framefont->getFontSetExtents()->max_ink_extent.y;
+				m_screen->getMenuStyle()->framefont.fontSetExtents()->max_ink_extent.y;
 		} else {
 			text_y = item_y +
-				m_screen->getMenuStyle()->framefont->getFontStruct()->ascent +
+				m_screen->getMenuStyle()->framefont.fontStruct()->ascent +
 				(menu.bevel_w / 2);
 		}
 
-		// framfont.justify
 		switch(m_screen->getMenuStyle()->framefont_justify) {
 		case DrawUtil::Font::LEFT:
 			text_x = item_x + menu.bevel_w + menu.item_h + 1;
@@ -736,10 +726,12 @@ void Basemenu::drawItem(unsigned int index, bool highlight, bool clear,
 				m_screen->getMenuStyle()->hilite_gc,
 				hilite_x, hilite_y, hilite_w, hilite_h);
 		}
-	} else if (dosel && item->isSelected() &&
-					(menu.sel_pixmap != ParentRelative)) {
+	} 
+	
+	if (dosel && item->isSelected() &&
+			(menu.sel_pixmap != ParentRelative)) {
 		if (menu.sel_pixmap) {
-			XCopyArea(m_display, menu.sel_pixmap, menu.frame,		
+			XCopyArea(m_display, highlight ? menu.frame_pixmap : menu.sel_pixmap, menu.frame,		
 				m_screen->getMenuStyle()->hilite_gc, 0, 0,
 				half_w, half_w, sel_x, sel_y);
 		} else {
@@ -751,7 +743,7 @@ void Basemenu::drawItem(unsigned int index, bool highlight, bool clear,
 	
 	if (dotext && text) {
 		if (i18n->multibyte()) {
-			XmbDrawString(m_display, menu.frame, m_screen->getMenuStyle()->framefont->getFontSet(),
+			XmbDrawString(m_display, menu.frame, m_screen->getMenuStyle()->framefont.fontSet(),
 				tgc, text_x, text_y, text, len);
 		} else
 			XDrawString(m_display, menu.frame, tgc, text_x, text_y, text, len);
@@ -806,9 +798,9 @@ void Basemenu::drawItem(unsigned int index, bool highlight, bool clear,
 }
 
 
-void Basemenu::setLabel(const char *l) {
-	//we don't want 0 to std::string
-	menu.label = l ? l : "";
+void Basemenu::setLabel(const char *labelstr) {
+	//make sure we don't send 0 to std::string
+	menu.label = (labelstr ? labelstr : "");
 }
 
 
@@ -880,7 +872,7 @@ void Basemenu::buttonPressEvent(XButtonEvent *be) {
 void Basemenu::buttonReleaseEvent(XButtonEvent *re) {
 	if (re->window == menu.title) {
 		if (moving) {
-			moving = False;
+			moving = false;
 			
 			if (which_sub >= 0)
 				drawSubmenu(which_sub);
@@ -1071,7 +1063,7 @@ void Basemenu::enterNotifyEvent(XCrossingEvent *ce) {
 			XMoveWindow(m_display, menu.window, menu.x_shift, menu.y_shift);
 		}
 
-		if (which_sub >= 0) {
+		if (which_sub >= 0 && static_cast<size_t>(which_sub) < menuitems.size()) {
 			BasemenuItem *tmp = menuitems[which_sub];
 			if (tmp->submenu()->isVisible()) {
 				int sbl = (ce->x / menu.item_w), i = (ce->y / menu.item_h),
@@ -1094,16 +1086,17 @@ void Basemenu::leaveNotifyEvent(XCrossingEvent *ce) {
 		if (which_press != -1 && which_sbl != -1 && menuitems.size() > 0) {
 			int p = (which_sbl * menu.persub) + which_press;
 
-			drawItem(p, (p == which_sub), True);
+			drawItem(p, (p == which_sub), true);
 
 			which_sbl = which_press = -1;
 		}
 
 		if (shifted) {
 			XMoveWindow(m_display, menu.window, menu.x, menu.y);
-			shifted = False;
+			shifted = false;
 
-			if (which_sub >= 0) drawSubmenu(which_sub);
+			if (which_sub >= 0)
+				drawSubmenu(which_sub);
 		}
 	}
 }
@@ -1111,9 +1104,9 @@ void Basemenu::leaveNotifyEvent(XCrossingEvent *ce) {
 
 void Basemenu::reconfigure(void) {
 	XSetWindowBackground(m_display, menu.window,
-					 m_screen->getBorderColor()->getPixel());
+					 m_screen->getBorderColor()->pixel());
 	XSetWindowBorder(m_display, menu.window,
-			 m_screen->getBorderColor()->getPixel());
+			 m_screen->getBorderColor()->pixel());
 	XSetWindowBorderWidth(m_display, menu.window, m_screen->getBorderWidth());
 
 	menu.bevel_w = m_screen->getBevelWidth();
