@@ -16,13 +16,13 @@
 //
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.	IN NO EVENT SHALL
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 // THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Configmenu.cc,v 1.13 2002/04/12 15:14:09 fluxgen Exp $
+// $Id: Configmenu.cc,v 1.16 2002/09/01 11:41:22 fluxgen Exp $
 
 // stupid macros needed to access some functions in version 2 of the GNU C
 // library
@@ -40,7 +40,7 @@
 #include "Window.hh"
 
 enum {CMENU_USE_TABS=9, CMENU_USE_ICONS, CMENU_SLOPPY_WIN_GROUP,
-		CMENU_WORKSPACE_WARPING, CMENU_TAB_ROTATE=21};
+	CMENU_DESKTOP_WHEELING, CMENU_WORKSPACE_WARPING, CMENU_TAB_ROTATE=21};
 
 
 Configmenu::Configmenu(BScreen *scr) : Basemenu(scr) {
@@ -84,7 +84,7 @@ Configmenu::Configmenu(BScreen *scr) : Basemenu(scr) {
 		"Focus Last Window on Workspace"), 5);
 	insert(i18n->getMessage(
 		ConfigmenuSet, ConfigmenuMaxOverSlit,
-		"Maxmize Over Slit"), 6);
+		"Maximize Over Slit"), 6);
     insert(i18n->getMessage(
 		ConfigmenuSet, ConfigmenuTabs,
 		"Use Tabs"), CMENU_USE_TABS);
@@ -97,6 +97,9 @@ Configmenu::Configmenu(BScreen *scr) : Basemenu(scr) {
 	insert(i18n->getMessage(
 		ConfigmenuSet, ConfigmenuWorkspaceWarping,
 		"Workspace Warping"), CMENU_WORKSPACE_WARPING);
+	insert(i18n->getMessage(
+		ConfigmenuSet, ConfigmenuDesktopWheeling,
+		"Desktop MouseWheel Switching"), CMENU_DESKTOP_WHEELING);
 
 	update();
 	setItemSelected(8, screen->doMaxOverSlit());
@@ -110,6 +113,7 @@ Configmenu::Configmenu(BScreen *scr) : Basemenu(scr) {
 	setItemSelected(CMENU_USE_ICONS, Fluxbox::instance()->useIconBar());
 	setItemSelected(CMENU_SLOPPY_WIN_GROUP, screen->isSloppyWindowGrouping());
 	setItemSelected(CMENU_WORKSPACE_WARPING, screen->isWorkspaceWarping());
+	setItemSelected(CMENU_DESKTOP_WHEELING, screen->isDesktopWheeling());
 }
 
 
@@ -193,8 +197,18 @@ void Configmenu::itemSelected(int button, unsigned int index) {
 					screen->reconfigure();
 				}
 				break;
+			case CMENU_DESKTOP_WHEELING:
+				{
+					screen->saveDesktopWheeling(!screen->isDesktopWheeling());
+					setItemSelected(index, screen->isDesktopWheeling());
+					screen->reconfigure();
+				}
+				break;
 
 			}
+			//save resources
+			Fluxbox::instance()->save_rc();
+			
 		}
 }
 
@@ -289,7 +303,10 @@ void Configmenu::Focusmenu::itemSelected(int button, unsigned int index) {
 
 			break;
 			}
-
+			//save resources
+			Fluxbox::instance()->save_rc();
+			
+			//toggle options
 			setItemSelected(0, !(configmenu->screen->isSloppyFocus() || 
 							configmenu->screen->isSemiSloppyFocus()));
 			setItemSelected(1, configmenu->screen->isSloppyFocus());
@@ -427,6 +444,8 @@ void Configmenu::Placementmenu::itemSelected(int button, unsigned int index) {
 
 			break;
 			}
+			//save resources
+			Fluxbox::instance()->save_rc();
 		}
 	}
 }
@@ -638,7 +657,8 @@ void Configmenu::Tabmenu::itemSelected(int button, unsigned int index) {
 					configmenu->screen->saveTabRotateVertical(True);
 				break;
 			}
-
+			//save resources
+			Fluxbox::instance()->save_rc();
 			Fluxbox::instance()->reconfigureTabs(); //TODO
 			setSelected();			
 	}
