@@ -22,10 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// stupid macros needed to access some functions in version 2 of the GNU C
-// library
-
-// $Id: Image.cc,v 1.8 2002/04/08 18:57:53 fluxgen Exp $
+// $Id: Image.cc,v 1.13 2002/08/11 20:44:48 fluxgen Exp $
 
 //use GNU extensions
 #ifndef _GNU_SOURCE
@@ -69,7 +66,16 @@ typedef unsigned int u_int32_t;
 #	include <ctype.h>
 #endif // HAVE_CTYPE_H
 
-static unsigned long bsqrt(unsigned long x) {
+#include <iostream>
+
+using namespace std;
+
+// lookup table for texture
+unsigned long *BImageControl::sqrt_table = 0;
+
+namespace { // anonymous
+
+unsigned long bsqrt(unsigned long x) {
 	if (x <= 0) return 0;
 	if (x == 1) return 1;
 
@@ -83,6 +89,7 @@ static unsigned long bsqrt(unsigned long x) {
 	}
 }
 
+};
 
 BImage::BImage(BImageControl *c, unsigned int w, unsigned int h) {
 	control = c;
@@ -96,14 +103,14 @@ BImage::BImage(BImageControl *c, unsigned int w, unsigned int h) {
 
 	xtable = ytable = (unsigned int *) 0;
 
-	cpc = control->getColorsPerChannel();
+	cpc = control->colorsPerChannel();
 	cpccpc = cpc * cpc;
 
-	control->getColorTables(&red_table, &green_table, &blue_table,
+	control->colorTables(&red_table, &green_table, &blue_table,
 		&red_offset, &green_offset, &blue_offset,
 		&red_bits, &green_bits, &blue_bits);
 
-	if (control->getVisual()->c_class != TrueColor)
+	if (control->visual()->c_class != TrueColor)
 		control->getXColorTable(&colors, &ncolors);
 }
 
@@ -115,22 +122,23 @@ BImage::~BImage(void) {
 }
 
 
-Pixmap BImage::render(BTexture *texture) {
-	if (texture->getTexture() & BImage::PARENTRELATIVE)
+Pixmap BImage::render(const FbTk::Texture *texture) {
+	using namespace FbTk;
+	if (texture->type() & Texture::PARENTRELATIVE)
 		return ParentRelative;
-	else if (texture->getTexture() & BImage::SOLID)
-		return render_solid(texture);
-	else if (texture->getTexture() & BImage::GRADIENT)
-		return render_gradient(texture);
+	else if (texture->type() & Texture::SOLID)
+		return renderSolid(texture);
+	else if (texture->type() & Texture::GRADIENT)
+		return renderGradient(texture);
 
 	return None;
 }
 
 
-Pixmap BImage::render_solid(BTexture *texture) {
-	Pixmap pixmap = XCreatePixmap(control->getBaseDisplay()->getXDisplay(),
-		control->getDrawable(), width,
-		height, control->getDepth());
+Pixmap BImage::renderSolid(const FbTk::Texture *texture) {
+	Pixmap pixmap = XCreatePixmap(control->baseDisplay()->getXDisplay(),
+		control->drawable(), width,
+		height, control->depth());
 	if (pixmap == None) {
 		fprintf(stderr,
 			I18n::instance()->getMessage(
@@ -142,130 +150,145 @@ Pixmap BImage::render_solid(BTexture *texture) {
 	XGCValues gcv;
 	GC gc, hgc, lgc;
 
-	gcv.foreground = texture->getColor()->getPixel();
+	gcv.foreground = texture->color().pixel();
 	gcv.fill_style = FillSolid;
-	gc = XCreateGC(control->getBaseDisplay()->getXDisplay(), pixmap,
+	gc = XCreateGC(control->baseDisplay()->getXDisplay(), pixmap,
 		GCForeground | GCFillStyle, &gcv);
 
-	gcv.foreground = texture->getHiColor()->getPixel();
-	hgc = XCreateGC(control->getBaseDisplay()->getXDisplay(), pixmap,
+	gcv.foreground = texture->hiColor().pixel();
+	hgc = XCreateGC(control->baseDisplay()->getXDisplay(), pixmap,
 		GCForeground, &gcv);
 
-	gcv.foreground = texture->getLoColor()->getPixel();
-	lgc = XCreateGC(control->getBaseDisplay()->getXDisplay(), pixmap,
+	gcv.foreground = texture->loColor().pixel();
+	lgc = XCreateGC(control->baseDisplay()->getXDisplay(), pixmap,
 			GCForeground, &gcv);
 
-	XFillRectangle(control->getBaseDisplay()->getXDisplay(), pixmap, gc, 0, 0,
+	XFillRectangle(control->baseDisplay()->getXDisplay(), pixmap, gc, 0, 0,
 		width, height);
 
-#ifdef		INTERLACE
-	if (texture->getTexture() & BImage::INTERLACED) {
-		gcv.foreground = texture->getColorTo()->getPixel();
-		GC igc = XCreateGC(control->getBaseDisplay()->getXDisplay(), pixmap,
+	using namespace FbTk;
+#ifdef INTERLACE
+	if (texture->type() & Texture::INTERLACED) {
+		gcv.foreground = texture->colorTo().pixel();
+		GC igc = XCreateGC(control->baseDisplay()->getXDisplay(), pixmap,
 			GCForeground, &gcv);
 
 		register unsigned int i = 0;
 		for (; i < height; i += 2)
-			XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, igc,
+			XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, igc,
 				0, i, width, i);
 
-		XFreeGC(control->getBaseDisplay()->getXDisplay(), igc);
+		XFreeGC(control->baseDisplay()->getXDisplay(), igc);
 	}
 #endif // INTERLACE
 
 
-	if (texture->getTexture() & BImage::BEVEL1) {
-		if (texture->getTexture() & BImage::RAISED) {
-			XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, lgc,
+	if (texture->type() & Texture::BEVEL1) {
+		if (texture->type() & Texture::RAISED) {
+			XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, lgc,
 								0, height - 1, width - 1, height - 1);
-			XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, lgc,
+			XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, lgc,
 							width - 1, height - 1, width - 1, 0);
 
-			XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, hgc,
+			XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, hgc,
 								0, 0, width - 1, 0);
-			XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, hgc,
+			XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, hgc,
 								0, height - 1, 0, 0);
-		} else if (texture->getTexture() & BImage::SUNKEN) {
-			XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, hgc,
+		} else if (texture->type() & Texture::SUNKEN) {
+			XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, hgc,
 								0, height - 1, width - 1, height - 1);
-			XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, hgc,
+			XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, hgc,
 								width - 1, height - 1, width - 1, 0);
 
-			XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, lgc,
+			XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, lgc,
 								0, 0, width - 1, 0);
-			XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, lgc,
+			XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, lgc,
 								0, height - 1, 0, 0);
 		}
-	} else if (texture->getTexture() & BImage::BEVEL2) {
-			if (texture->getTexture() & BImage::RAISED) {
-				XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, lgc,
+	} else if (texture->type() & Texture::BEVEL2) {
+			if (texture->type() & Texture::RAISED) {
+				XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, lgc,
 							1, height - 3, width - 3, height - 3);
-				XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, lgc,
+				XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, lgc,
 							width - 3, height - 3, width - 3, 1);
 
-				XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, hgc,
+				XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, hgc,
 							1, 1, width - 3, 1);
-				XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, hgc,
+				XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, hgc,
 							1, height - 3, 1, 1);
-			} else if (texture->getTexture() & BImage::SUNKEN) {
-				XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, hgc,
+			} else if (texture->type() & Texture::SUNKEN) {
+				XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, hgc,
 							1, height - 3, width - 3, height - 3);
-				XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, hgc,
+				XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, hgc,
 							width - 3, height - 3, width - 3, 1);
 
-				XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, lgc,
+				XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, lgc,
 							1, 1, width - 3, 1);
-				XDrawLine(control->getBaseDisplay()->getXDisplay(), pixmap, lgc,
+				XDrawLine(control->baseDisplay()->getXDisplay(), pixmap, lgc,
 							1, height - 3, 1, 1);
 		}
 	}
 
-	XFreeGC(control->getBaseDisplay()->getXDisplay(), gc);
-	XFreeGC(control->getBaseDisplay()->getXDisplay(), hgc);
-	XFreeGC(control->getBaseDisplay()->getXDisplay(), lgc);
+	XFreeGC(control->baseDisplay()->getXDisplay(), gc);
+	XFreeGC(control->baseDisplay()->getXDisplay(), hgc);
+	XFreeGC(control->baseDisplay()->getXDisplay(), lgc);
 
 	return pixmap;
 }
 
 
-Pixmap BImage::render_gradient(BTexture *texture) {
- int inverted = 0;
+Pixmap BImage::renderGradient(const FbTk::Texture *texture) {
 
-#ifdef		INTERLACE
-	interlaced = texture->getTexture() & BImage::INTERLACED;
+	bool inverted = false;
+
+	using namespace FbTk;
+
+#ifdef INTERLACE
+	interlaced = texture->type() & Texture::INTERLACED;
 #endif // INTERLACE
 
-	if (texture->getTexture() & BImage::SUNKEN) {
-		from = texture->getColorTo();
-		to = texture->getColor();
+	if (texture->type() & Texture::SUNKEN) {
+		from = &(texture->colorTo());
+		to = &(texture->color());
 
-		if (! (texture->getTexture() & BImage::INVERT)) inverted = 1;
+		if (! (texture->type() & Texture::INVERT)) 
+			inverted = true;
 	} else {
-		from = texture->getColor();
-		to = texture->getColorTo();
+		from = &(texture->color());
+		to = &(texture->colorTo());
 
-		if (texture->getTexture() & BImage::INVERT) inverted = 1;
+		if (texture->type() & Texture::INVERT) 
+			inverted = true;
 	}
 
 	control->getGradientBuffers(width, height, &xtable, &ytable);
 
-	if (texture->getTexture() & BImage::DIAGONAL) dgradient();
-	else if (texture->getTexture() & BImage::ELLIPTIC) egradient();
-	else if (texture->getTexture() & BImage::HORIZONTAL) hgradient();
-	else if (texture->getTexture() & BImage::PYRAMID) pgradient();
-	else if (texture->getTexture() & BImage::RECTANGLE) rgradient();
-	else if (texture->getTexture() & BImage::VERTICAL) vgradient();
-	else if (texture->getTexture() & BImage::CROSSDIAGONAL) cdgradient();
-	else if (texture->getTexture() & BImage::PIPECROSS) pcgradient();
+	if (texture->type() & Texture::DIAGONAL) 
+		dgradient();
+	else if (texture->type() & Texture::ELLIPTIC) 
+		egradient();
+	else if (texture->type() & Texture::HORIZONTAL) 
+		hgradient();
+	else if (texture->type() & Texture::PYRAMID) 
+		pgradient();
+	else if (texture->type() & Texture::RECTANGLE) 
+		rgradient();
+	else if (texture->type() & Texture::VERTICAL) 
+		vgradient();
+	else if (texture->type() & Texture::CROSSDIAGONAL) 
+		cdgradient();
+	else if (texture->type() & Texture::PIPECROSS) 
+		pcgradient();
 
-	if (texture->getTexture() & BImage::BEVEL1) bevel1();
-	else if (texture->getTexture() & BImage::BEVEL2) bevel2();
+	if (texture->type() & Texture::BEVEL1)
+		bevel1();
+	else if (texture->type() & Texture::BEVEL2)
+		bevel2();
 
-	if (inverted) invert();
+	if (inverted)
+		invert();
 
-	Pixmap pixmap = renderPixmap();
-
-	return pixmap;
+	return renderPixmap();
 
 }
 
@@ -273,8 +296,8 @@ Pixmap BImage::render_gradient(BTexture *texture) {
 XImage *BImage::renderXImage(void) {
 	I18n *i18n = I18n::instance();
 	XImage *image =
-		XCreateImage(control->getBaseDisplay()->getXDisplay(),
-			 control->getVisual(), control->getDepth(), ZPixmap, 0, 0,
+		XCreateImage(control->baseDisplay()->getXDisplay(),
+			 control->visual(), control->depth(), ZPixmap, 0, 0,
 			 width, height, 32, 0);
 
 	if (! image) {
@@ -304,7 +327,7 @@ XImage *BImage::renderXImage(void) {
 			{1, 5, 0, 4},
 			{7, 3, 6, 2} };
 
-#ifdef		ORDEREDPSEUDO
+#ifdef ORDEREDPSEUDO
 		unsigned char dither8[8][8] = { 
 			{ 0, 32,  8, 40, 2,	34, 10, 42 },
 			{ 48, 16, 56, 24, 50, 18, 58, 26 },
@@ -316,7 +339,7 @@ XImage *BImage::renderXImage(void) {
 			{ 63, 31, 55, 23, 61, 29, 53, 21 } };
 #endif // ORDEREDPSEUDO
 
-		switch (control->getVisual()->c_class) {
+		switch (control->visual()->c_class) {
 		case TrueColor:
 			// algorithm: ordered dithering... many many thanks to rasterman
 			// (raster@rasterman.com) for telling me about this... portions of this
@@ -403,7 +426,7 @@ XImage *BImage::renderXImage(void) {
 			*ngerr = new short[width + 2],
 			*nberr = new short[width + 2];
 			int rr, gg, bb, rer, ger, ber;
-			int dd = 255 / control->getColorsPerChannel();
+			int dd = 255 / control->colorsPerChannel();
 
 			for (x = 0; x < width; x++) {
 				*(rerr + x) = *(red + x);
@@ -564,17 +587,17 @@ XImage *BImage::renderXImage(void) {
 			return (XImage *) 0;
 		}
 	} else {
-		switch (control->getVisual()->c_class) {
+		switch (control->visual()->c_class) {
 		case StaticColor:
 		case PseudoColor:
 			for (y = 0, offset = 0; y < height; y++) {
 				for (x = 0; x < width; x++, offset++) {
-			r = red_table[red[offset]];
+					r = red_table[red[offset]];
 					g = green_table[green[offset]];
-		b = blue_table[blue[offset]];
+					b = blue_table[blue[offset]];
 
-		pixel = (r * cpccpc) + (g * cpc) + b;
-		*pixel_data++ = colors[pixel].pixel;
+					pixel = (r * cpccpc) + (g * cpc) + b;
+					*pixel_data++ = colors[pixel].pixel;
 				}
 
 				pixel_data = (ppixel_data += image->bytes_per_line);
@@ -676,8 +699,8 @@ XImage *BImage::renderXImage(void) {
 Pixmap BImage::renderPixmap(void) {
 	I18n *i18n = I18n::instance();
 	Pixmap pixmap =
-		XCreatePixmap(control->getBaseDisplay()->getXDisplay(),
-				control->getDrawable(), width, height, control->getDepth());
+		XCreatePixmap(control->baseDisplay()->getXDisplay(),
+				control->drawable(), width, height, control->depth());
 
 	if (pixmap == None) {
 		fprintf(stderr,
@@ -690,16 +713,16 @@ Pixmap BImage::renderPixmap(void) {
 	XImage *image = renderXImage();
 
 	if (! image) {
-		XFreePixmap(control->getBaseDisplay()->getXDisplay(), pixmap);
+		XFreePixmap(control->baseDisplay()->getXDisplay(), pixmap);
 		return None;
 	} else if (! image->data) {
 		XDestroyImage(image);
-		XFreePixmap(control->getBaseDisplay()->getXDisplay(), pixmap);
+		XFreePixmap(control->baseDisplay()->getXDisplay(), pixmap);
 		return None;
 	}
 
-	XPutImage(control->getBaseDisplay()->getXDisplay(), pixmap,
-			DefaultGC(control->getBaseDisplay()->getXDisplay(),
+	XPutImage(control->baseDisplay()->getXDisplay(), pixmap,
+			DefaultGC(control->baseDisplay()->getXDisplay(),
 					control->getScreenInfo()->getScreenNumber()),
 						image, 0, 0, 0, 0, width, height);
 
@@ -951,22 +974,22 @@ void BImage::invert(void) {
 }
 
 
-void BImage::dgradient(void) {
+void BImage::dgradient() {
 	// diagonal gradient code was written by Mike Cole <mike@mydot.com>
 	// modified for interlacing by Brad Hughes
 
 	float drx, dgx, dbx, dry, dgy, dby, yr = 0.0, yg = 0.0, yb = 0.0,
-		xr = (float) from->getRed(),
-		xg = (float) from->getGreen(),
-		xb = (float) from->getBlue();
+		xr = (float) from->red(),
+		xg = (float) from->green(),
+		xb = (float) from->blue();
 	unsigned char *pr = red, *pg = green, *pb = blue;
 	unsigned int w = width * 2, h = height * 2, *xt = xtable, *yt = ytable;
 
 	register unsigned int x, y;
 
-	dry = drx = (float) (to->getRed() - from->getRed());
-	dgy = dgx = (float) (to->getGreen() - from->getGreen());
-	dby = dbx = (float) (to->getBlue() - from->getBlue());
+	dry = drx = (float) (to->red() - from->red());
+	dgy = dgx = (float) (to->green() - from->green());
+	dby = dbx = (float) (to->blue() - from->blue());
 
 	// Create X table
 	drx /= w;
@@ -1061,16 +1084,16 @@ void BImage::dgradient(void) {
 
 void BImage::hgradient(void) {
 	float drx, dgx, dbx,
-		xr = (float) from->getRed(),
-		xg = (float) from->getGreen(),
-		xb = (float) from->getBlue();
+		xr = (float) from->red(),
+		xg = (float) from->green(),
+		xb = (float) from->blue();
 	unsigned char *pr = red, *pg = green, *pb = blue;
 
 	register unsigned int x, y;
 
-	drx = (float) (to->getRed() - from->getRed());
-	dgx = (float) (to->getGreen() - from->getGreen());
-	dbx = (float) (to->getBlue() - from->getBlue());
+	drx = (float) (to->red() - from->red());
+	dgx = (float) (to->green() - from->green());
+	dbx = (float) (to->blue() - from->blue());
 
 	drx /= width;
 	dgx /= width;
@@ -1160,16 +1183,16 @@ void BImage::hgradient(void) {
 
 void BImage::vgradient(void) {
 	float dry, dgy, dby,
-		yr = (float) from->getRed(),
-		yg = (float) from->getGreen(),
-		yb = (float) from->getBlue();
+		yr = (float) from->red(),
+		yg = (float) from->green(),
+		yb = (float) from->blue();
 	unsigned char *pr = red, *pg = green, *pb = blue;
 
 	register unsigned int y;
 
-	dry = (float) (to->getRed() - from->getRed());
-	dgy = (float) (to->getGreen() - from->getGreen());
-	dby = (float) (to->getBlue() - from->getBlue());
+	dry = (float) (to->red() - from->red());
+	dgy = (float) (to->green() - from->green());
+	dby = (float) (to->blue() - from->blue());
 
 	dry /= height;
 	dgy /= height;
@@ -1247,14 +1270,14 @@ void BImage::pgradient(void) {
 		xr, xg, xb;
 	int rsign, gsign, bsign;
 	unsigned char *pr = red, *pg = green, *pb = blue;
-	unsigned int tr = to->getRed(), tg = to->getGreen(), tb = to->getBlue(),
+	unsigned int tr = to->red(), tg = to->green(), tb = to->blue(),
 		*xt = xtable, *yt = ytable;
 
 	register unsigned int x, y;
 
-	dry = drx = (float) (to->getRed() - from->getRed());
-	dgy = dgx = (float) (to->getGreen() - from->getGreen());
-	dby = dbx = (float) (to->getBlue() - from->getBlue());
+	dry = drx = (float) (to->red() - from->red());
+	dgy = dgx = (float) (to->green() - from->green());
+	dby = dbx = (float) (to->blue() - from->blue());
 
 	rsign = (drx < 0) ? -1 : 1;
 	gsign = (dgx < 0) ? -1 : 1;
@@ -1363,14 +1386,14 @@ void BImage::rgradient(void) {
 	float drx, dgx, dbx, dry, dgy, dby, xr, xg, xb, yr, yg, yb;
 	int rsign, gsign, bsign;
 	unsigned char *pr = red, *pg = green, *pb = blue;
-	unsigned int tr = to->getRed(), tg = to->getGreen(), tb = to->getBlue(),
+	unsigned int tr = to->red(), tg = to->green(), tb = to->blue(),
 		*xt = xtable, *yt = ytable;
 
 	register unsigned int x, y;
 
-	dry = drx = (float) (to->getRed() - from->getRed());
-	dgy = dgx = (float) (to->getGreen() - from->getGreen());
-	dby = dbx = (float) (to->getBlue() - from->getBlue());
+	dry = drx = (float) (to->red() - from->red());
+	dgy = dgx = (float) (to->green() - from->green());
+	dby = dbx = (float) (to->blue() - from->blue());
 
 	rsign = (drx < 0) ? -2 : 2;
 	gsign = (dgx < 0) ? -2 : 2;
@@ -1480,15 +1503,15 @@ void BImage::egradient(void) {
 	int rsign, gsign, bsign;
 	unsigned char *pr = red, *pg = green, *pb = blue;
 	unsigned int *xt = xtable, *yt = ytable,
-		tr = (unsigned long) to->getRed(),
-		tg = (unsigned long) to->getGreen(),
-		tb = (unsigned long) to->getBlue();
+		tr = (unsigned long) to->red(),
+		tg = (unsigned long) to->green(),
+		tb = (unsigned long) to->blue();
 
 	register unsigned int x, y;
 
-	dry = drx = (float) (to->getRed() - from->getRed());
-	dgy = dgx = (float) (to->getGreen() - from->getGreen());
-	dby = dbx = (float) (to->getBlue() - from->getBlue());
+	dry = drx = (float) (to->red() - from->red());
+	dgy = dgx = (float) (to->green() - from->green());
+	dby = dbx = (float) (to->blue() - from->blue());
 
 	rsign = (drx < 0) ? -1 : 1;
 	gsign = (dgx < 0) ? -1 : 1;
@@ -1607,15 +1630,15 @@ void BImage::pcgradient(void) {
 	int rsign, gsign, bsign;
 	unsigned char *pr = red, *pg = green, *pb = blue;
 	unsigned int *xt = xtable, *yt = ytable,
-		tr = to->getRed(),
-		tg = to->getGreen(),
-		tb = to->getBlue();
+		tr = to->red(),
+		tg = to->green(),
+		tb = to->blue();
 
 	register unsigned int x, y;
 
-	dry = drx = (float) (to->getRed() - from->getRed());
-	dgy = dgx = (float) (to->getGreen() - from->getGreen());
-	dby = dbx = (float) (to->getBlue() - from->getBlue());
+	dry = drx = (float) (to->red() - from->red());
+	dgy = dgx = (float) (to->green() - from->green());
+	dby = dbx = (float) (to->blue() - from->blue());
 
 	rsign = (drx < 0) ? -2 : 2;
 	gsign = (dgx < 0) ? -2 : 2;
@@ -1722,17 +1745,17 @@ void BImage::cdgradient(void) {
 	// adapted from kde sources for Blackbox by Brad Hughes
 
 	float drx, dgx, dbx, dry, dgy, dby, yr = 0.0, yg = 0.0, yb = 0.0,
-		xr = (float) from->getRed(),
-		xg = (float) from->getGreen(),
-		xb = (float) from->getBlue();
+		xr = (float) from->red(),
+		xg = (float) from->green(),
+		xb = (float) from->blue();
 	unsigned char *pr = red, *pg = green, *pb = blue;
 	unsigned int w = width * 2, h = height * 2, *xt, *yt;
 
 	register unsigned int x, y;
 
-	dry = drx = (float) (to->getRed() - from->getRed());
-	dgy = dgx = (float) (to->getGreen() - from->getGreen());
-	dby = dbx = (float) (to->getBlue() - from->getBlue());
+	dry = drx = (float) (to->red() - from->red());
+	dgy = dgx = (float) (to->green() - from->green());
+	dby = dbx = (float) (to->blue() - from->blue());
 
 	// Create X table
 	drx /= w;
@@ -1825,11 +1848,11 @@ void BImage::cdgradient(void) {
 }
 
 
-BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
+BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, bool _dither,
 		int _cpc, unsigned long cache_timeout,
-	unsigned long cmax):
+	unsigned long cmax)
 #ifdef TIMEDCACHE
-	 timer(this)
+: timer(this)
 #endif //TIMEDCACHE
 {
 	I18n *i18n = I18n::instance();
@@ -1852,16 +1875,14 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 	grad_xbuffer = grad_ybuffer = (unsigned int *) 0;
 	grad_buffer_width = grad_buffer_height = 0;
 
-	sqrt_table = (unsigned long *) 0;
-
 	screen_depth = screeninfo->getDepth();
 	window = screeninfo->getRootWindow();
 	screen_number = screeninfo->getScreenNumber();
 
 	int count;
 	XPixmapFormatValues *pmv = XListPixmapFormats(basedisplay->getXDisplay(),
-																&count);
-	colormap = screeninfo->getColormap();
+		&count);
+	m_colormap = screeninfo->colormap();
 
 	if (pmv) {
 		bits_per_pixel = 0;
@@ -1882,15 +1903,15 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 
 	red_offset = green_offset = blue_offset = 0;
 
-	switch (getVisual()->c_class) {
+	switch (visual()->c_class) {
 	case TrueColor:
 	{
 		int i;
 
 		// compute color tables
-		unsigned long red_mask = getVisual()->red_mask,
-			green_mask = getVisual()->green_mask,
-			blue_mask = getVisual()->blue_mask;
+		unsigned long red_mask = visual()->red_mask,
+			green_mask = visual()->green_mask,
+			blue_mask = visual()->blue_mask;
 
 		while (! (red_mask & 1)) { red_offset++; red_mask >>= 1; }
 		while (! (green_mask & 1)) { green_offset++; green_mask >>= 1; }
@@ -1957,19 +1978,20 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 			red_color_table[i] = green_color_table[i] = blue_color_table[i] =
 					i / bits;
 
-		for (r = 0, i = 0; r < colors_per_channel; r++)
-			for (g = 0; g < colors_per_channel; g++)
+		for (r = 0, i = 0; r < colors_per_channel; r++) {
+			for (g = 0; g < colors_per_channel; g++) {
 				for (b = 0; b < colors_per_channel; b++, i++) {
 					colors[i].red = (r * 0xffff) / (colors_per_channel - 1);
 					colors[i].green = (g * 0xffff) / (colors_per_channel - 1);
 					colors[i].blue = (b * 0xffff) / (colors_per_channel - 1);;
 					colors[i].flags = DoRed|DoGreen|DoBlue;
 				}
-
+			}
+		}
 		basedisplay->grab();
 
 		for (i = 0; i < ncolors; i++) {
-			if (! XAllocColor(basedisplay->getXDisplay(), colormap, &colors[i])) {
+			if (! XAllocColor(basedisplay->getXDisplay(), colormap(), &colors[i])) {
 				fprintf(stderr,
 				i18n->getMessage(
 					FBNLS::ImageSet, FBNLS::ImageColorAllocFail,
@@ -1988,7 +2010,7 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 		for (i = 0; i < incolors; i++)
 			icolors[i].pixel = i;
 
-		XQueryColors(basedisplay->getXDisplay(), colormap, icolors, incolors);
+		XQueryColors(basedisplay->getXDisplay(), colormap(), icolors, incolors);
 		for (i = 0; i < ncolors; i++) {
 			if (! colors[i].flags) {
 				unsigned long chk = 0xffffffff, pixel, close = 0;
@@ -2010,7 +2032,7 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 						colors[i].green = icolors[close].green;
 						colors[i].blue = icolors[close].blue;
 
-						if (XAllocColor(basedisplay->getXDisplay(), colormap,
+						if (XAllocColor(basedisplay->getXDisplay(), colormap(),
 								&colors[i])) {
 							colors[i].flags = DoRed|DoGreen|DoBlue;
 							break;
@@ -2027,7 +2049,7 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 	case StaticGray:
 	{
 
-		if (getVisual()->c_class == StaticGray) {
+		if (visual()->c_class == StaticGray) {
 			ncolors = 1 << screen_depth;
 		} else {
 			ncolors = colors_per_channel * colors_per_channel * colors_per_channel;
@@ -2077,7 +2099,7 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 			colors[i].blue = (i * 0xffff) / (colors_per_channel - 1);;
 			colors[i].flags = DoRed|DoGreen|DoBlue;
 
-			if (! XAllocColor(basedisplay->getXDisplay(), colormap,
+			if (! XAllocColor(basedisplay->getXDisplay(), colormap(),
 					&colors[i])) {
 				fprintf(stderr,
 					i18n->
@@ -2099,7 +2121,7 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 		for (i = 0; i < incolors; i++)
 			icolors[i].pixel = i;
 
-		XQueryColors(basedisplay->getXDisplay(), colormap, icolors, incolors);
+		XQueryColors(basedisplay->getXDisplay(), colormap(), icolors, incolors);
 		for (i = 0; i < ncolors; i++) {
 			if (! colors[i].flags) {
 				unsigned long chk = 0xffffffff, pixel, close = 0;
@@ -2121,7 +2143,7 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 						colors[i].green = icolors[close].green;
 						colors[i].blue = icolors[close].blue;
 
-						if (XAllocColor(basedisplay->getXDisplay(), colormap,
+						if (XAllocColor(basedisplay->getXDisplay(), colormap(),
 								&colors[i])) {
 							colors[i].flags = DoRed|DoGreen|DoBlue;
 							break;
@@ -2140,7 +2162,7 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 			getMessage(
 				FBNLS::ImageSet, FBNLS::ImageUnsupVisual,
 				"BImageControl::BImageControl: unsupported visual %d\n"),
-				getVisual()->c_class);
+				visual()->c_class);
 		throw static_cast<int>(1); //throw error code 1
 	
 	}
@@ -2163,11 +2185,10 @@ BImageControl::~BImageControl(void) {
 	if (colors) {
 		unsigned long *pixels = new unsigned long [ncolors];
 
-		int i;
-		for (i = 0; i < ncolors; i++)
-			*(pixels + i) = (*(colors + i)).pixel;
+		for (int color = 0; color < ncolors; color++)
+			*(pixels + color) = (*(colors + color)).pixel;
 
-		XFreeColors(basedisplay->getXDisplay(), colormap, pixels, ncolors, 0);
+		XFreeColors(basedisplay->getXDisplay(), colormap(), pixels, ncolors, 0);
 
 		delete [] colors;
 	}
@@ -2192,17 +2213,17 @@ BImageControl::~BImageControl(void) {
 
 
 Pixmap BImageControl::searchCache(unsigned int width, unsigned int height,
-			unsigned long texture,
-			BColor *c1, BColor *c2) {
+			unsigned long texture_type,
+			const FbTk::Color &color, const FbTk::Color &color_to) {
 	CacheList::iterator it = cache.begin();
 	CacheList::iterator it_end = cache.end();
 	for (; it != it_end; ++it) {
 		if (((*it)->width == width) &&
 				((*it)->height == height) &&
-				((*it)->texture == texture) &&
-				((*it)->pixel1 == c1->getPixel())) {
-			if (texture & BImage::GRADIENT) {
-				if ((*it)->pixel2 == c2->getPixel()) {
+				((*it)->texture == texture_type) &&
+				((*it)->pixel1 == color.pixel())) {
+			if (texture_type & FbTk::Texture::GRADIENT) {
+				if ((*it)->pixel2 == color_to.pixel()) {
 					(*it)->count++;
 					return (*it)->pixmap;
 				}
@@ -2218,11 +2239,12 @@ Pixmap BImageControl::searchCache(unsigned int width, unsigned int height,
 
 
 Pixmap BImageControl::renderImage(unsigned int width, unsigned int height,
-			BTexture *texture) {
-	if (texture->getTexture() & BImage::PARENTRELATIVE) return ParentRelative;
+			const FbTk::Texture *texture) {
+	if (texture->type() & FbTk::Texture::PARENTRELATIVE)
+		return ParentRelative;
 
-	Pixmap pixmap = searchCache(width, height, texture->getTexture(),
-						texture->getColor(), texture->getColorTo());
+	Pixmap pixmap = searchCache(width, height, texture->type(),
+						texture->color(), texture->colorTo());
 	if (pixmap) return pixmap;
 
 	BImage image(this, width, height);
@@ -2235,11 +2257,11 @@ Pixmap BImageControl::renderImage(unsigned int width, unsigned int height,
 		tmp->width = width;
 		tmp->height = height;
 		tmp->count = 1;
-		tmp->texture = texture->getTexture();
-		tmp->pixel1 = texture->getColor()->getPixel();
+		tmp->texture = texture->type();
+		tmp->pixel1 = texture->color().pixel();
 
-		if (texture->getTexture() & BImage::GRADIENT)
-			tmp->pixel2 = texture->getColorTo()->getPixel();
+		if (texture->type() & FbTk::Texture::GRADIENT)
+			tmp->pixel2 = texture->colorTo().pixel();
 		else
 			tmp->pixel2 = 0l;
 
@@ -2287,18 +2309,18 @@ void BImageControl::removeImage(Pixmap pixmap) {
 }
 
 
-unsigned long BImageControl::getColor(const char *colorname,
+unsigned long BImageControl::color(const char *colorname,
 							unsigned char *r, unsigned char *g,
 							unsigned char *b)
 {
 	XColor color;
 	color.pixel = 0;
 
-	if (! XParseColor(basedisplay->getXDisplay(), colormap, colorname, &color))
-		fprintf(stderr, "BImageControl::getColor: color parse error: \"%s\"\n",
+	if (! XParseColor(basedisplay->getXDisplay(), colormap(), colorname, &color))
+		fprintf(stderr, "BImageControl::color: color parse error: \"%s\"\n",
 			colorname);
-	else if (! XAllocColor(basedisplay->getXDisplay(), colormap, &color))
-		fprintf(stderr, "BImageControl::getColor: color alloc error: \"%s\"\n",
+	else if (! XAllocColor(basedisplay->getXDisplay(), colormap(), &color))
+		fprintf(stderr, "BImageControl::color: color alloc error: \"%s\"\n",
 			colorname);
 
 	if (color.red == 65535) *r = 0xff;
@@ -2312,16 +2334,16 @@ unsigned long BImageControl::getColor(const char *colorname,
 }
 
 
-unsigned long BImageControl::getColor(const char *colorname) {
+unsigned long BImageControl::color(const char *colorname) {
 	XColor color;
 	color.pixel = 0;
 
-	if (! XParseColor(basedisplay->getXDisplay(), colormap, colorname, &color))
-		fprintf(stderr, "BImageControl::getColor: color parse error: \"%s\"\n",
+	if (! XParseColor(basedisplay->getXDisplay(), colormap(), colorname, &color))
+		fprintf(stderr, "BImageControl::color: color parse error: \"%s\"\n",
 			colorname);
 	else	{
-		if (! XAllocColor(basedisplay->getXDisplay(), colormap, &color))
-			fprintf(stderr, "BImageControl::getColor: color alloc error: \"%s\"\n",
+		if (! XAllocColor(basedisplay->getXDisplay(), colormap(), &color))
+			fprintf(stderr, "BImageControl::color: color alloc error: \"%s\"\n",
 				colorname);
 		else
 			fprintf(stderr, "%s(%d) Allocated color:%s", __FILE__, __LINE__, colorname);
@@ -2330,7 +2352,7 @@ unsigned long BImageControl::getColor(const char *colorname) {
 }
 
 
-void BImageControl::getColorTables(unsigned char **rmt, unsigned char **gmt,
+void BImageControl::colorTables(unsigned char **rmt, unsigned char **gmt,
 	unsigned char **bmt,
 	int *roff, int *goff, int *boff,
 	int *rbit, int *gbit, int *bbit) {
@@ -2395,11 +2417,11 @@ void BImageControl::installRootColormap(void) {
 
 	if (cmaps) {
 		for (i = 0; i < ncmap; i++)
-			if (*(cmaps + i) == colormap)
+			if (*(cmaps + i) == colormap())
 	install = False;
 
 		if (install)
-			XInstallColormap(basedisplay->getXDisplay(), colormap);
+			XInstallColormap(basedisplay->getXDisplay(), colormap());
 
 		XFree(cmaps);
 	}
@@ -2431,65 +2453,68 @@ unsigned long BImageControl::getSqrt(unsigned int x) {
 }
 
 
-void BImageControl::parseTexture(BTexture *texture, char *t) {
-	if ((! texture) || (! t)) return;
+void BImageControl::parseTexture(FbTk::Texture *texture, const char *texture_string) {
+	if ((! texture) || (! texture_string))
+		return;
 
-	int t_len = strlen(t) + 1, i;
+	int t_len = strlen(texture_string) + 1;
 	char *ts = new char[t_len];
 	if (! ts) return;
 
 	// convert to lower case
-	for (i = 0; i < t_len; i++)
-		*(ts + i) = tolower(*(t + i));
+	for (int byte_pos = 0; byte_pos < t_len; byte_pos++)
+		*(ts + byte_pos) = tolower(*(texture_string + byte_pos));
+
+	using namespace FbTk;
 
 	if (strstr(ts, "parentrelative")) {
-		texture->setTexture(BImage::PARENTRELATIVE);
+		texture->setType(Texture::PARENTRELATIVE);
 	} else {
-		texture->setTexture(0);
+		texture->setType(Texture::NONE);
 
 		if (strstr(ts, "solid"))
-			texture->addTexture(BImage::SOLID);
+			texture->addType(Texture::SOLID);
 		else if (strstr(ts, "gradient")) {
-			texture->addTexture(BImage::GRADIENT);
+			texture->addType(Texture::GRADIENT);
 			if (strstr(ts, "crossdiagonal"))
-				texture->addTexture(BImage::CROSSDIAGONAL);
+				texture->addType(Texture::CROSSDIAGONAL);
 			else if (strstr(ts, "rectangle"))
-				texture->addTexture(BImage::RECTANGLE);
+				texture->addType(Texture::RECTANGLE);
 			else if (strstr(ts, "pyramid"))
-				texture->addTexture(BImage::PYRAMID);
+				texture->addType(Texture::PYRAMID);
 			else if (strstr(ts, "pipecross"))
-				texture->addTexture(BImage::PIPECROSS);
+				texture->addType(Texture::PIPECROSS);
 			else if (strstr(ts, "elliptic"))
-				texture->addTexture(BImage::ELLIPTIC);
+				texture->addType(Texture::ELLIPTIC);
 			else if (strstr(ts, "diagonal"))
-				texture->addTexture(BImage::DIAGONAL);
+				texture->addType(Texture::DIAGONAL);
 			else if (strstr(ts, "horizontal"))
-				texture->addTexture(BImage::HORIZONTAL);
+				texture->addType(Texture::HORIZONTAL);
 			else if (strstr(ts, "vertical"))
-				texture->addTexture(BImage::VERTICAL);
+				texture->addType(Texture::VERTICAL);
 			else
-				texture->addTexture(BImage::DIAGONAL);
+				texture->addType(Texture::DIAGONAL);
 		} else
-			texture->addTexture(BImage::SOLID);
+			texture->addType(Texture::SOLID);
 
 		if (strstr(ts, "raised"))
-			texture->addTexture(BImage::RAISED);
+			texture->addType(Texture::RAISED);
 		else if (strstr(ts, "sunken"))
-			texture->addTexture(BImage::SUNKEN);
+			texture->addType(Texture::SUNKEN);
 		else if (strstr(ts, "flat"))
-			texture->addTexture(BImage::FLAT);
+			texture->addType(Texture::FLAT);
 		else
-			texture->addTexture(BImage::RAISED);
+			texture->addType(Texture::RAISED);
 
-		if (! (texture->getTexture() & BImage::FLAT))
+		if (! (texture->type() & Texture::FLAT))
 			if (strstr(ts, "bevel2"))
-				texture->addTexture(BImage::BEVEL2);
+				texture->addType(Texture::BEVEL2);
 			else
-				texture->addTexture(BImage::BEVEL1);
+				texture->addType(Texture::BEVEL1);
 
-#ifdef		INTERLACE
+#ifdef INTERLACE
 		if (strstr(ts, "interlaced"))
-			texture->addTexture(BImage::INTERLACED);
+			texture->addType(Texture::INTERLACED);
 #endif // INTERLACE
 	}
 
@@ -2497,30 +2522,30 @@ void BImageControl::parseTexture(BTexture *texture, char *t) {
 }
 
 
-void BImageControl::parseColor(BColor *color, char *c) {
-	if (! color) return;
+void BImageControl::parseColor(FbTk::Color *col, const char *color_string) {
+	if (!col) return;
 
-	if (color->isAllocated()) {
-		unsigned long pixel = color->getPixel();
+	if (col->isAllocated()) {
+		unsigned long pixel = col->pixel();
 
-		XFreeColors(basedisplay->getXDisplay(), colormap, &pixel, 1, 0);
+		XFreeColors(basedisplay->getXDisplay(), colormap(), &pixel, 1, 0);
 
-		color->setPixel(0l);
-		color->setRGB(0, 0, 0);
-		color->setAllocated(False);
+		col->setPixel(0l);
+		col->setRGB(0, 0, 0);
+		col->setAllocated(false);
 	}
 
-	if (c) {
+	if (color_string != 0) {
 		unsigned char r, g, b;
 
-		color->setPixel(getColor(c, &r, &g, &b));
-		color->setRGB(r, g, b);
-		color->setAllocated(True);
+		col->setPixel(color(color_string, &r, &g, &b));
+		col->setRGB(r, g, b);
+		col->setAllocated(true);
 	}
 }
 
 
-void BImageControl::timeout(void) {
+void BImageControl::timeout() {
 	CacheList::iterator it = cache.begin();
 	CacheList::iterator it_end = cache.end();
 	for (; it != it_end; ++it) {
