@@ -19,48 +19,41 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-//$Id: Keys.cc,v 1.15 2002/04/19 09:33:42 fluxgen Exp $
+//$Id: Keys.cc,v 1.19 2002/08/28 20:25:15 fluxgen Exp $
 
-#ifdef		HAVE_CONFIG_H
-#	 include "config.h"
-#endif
 
 #include "Keys.hh"
 
 #include "StringUtil.hh"
 
-#ifdef		HAVE_STDIO_H
-#	 include <stdio.h>
-#endif	// HAVE_STDIO_H
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif // HAVE_CONFIG_H
 
-#ifdef		HAVE_CTYPE_H
-#	 include <ctype.h>
+
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
 #endif	// HAVE_CTYPE_H
 
-#ifdef		STDC_HEADERS
-#	 include <stdlib.h>
-#	 include <string.h>
-#  include <errno.h>
-#endif	// STDC_HEADERS
+#include <cstdio>
+#include <cstdlib>
+#include <cerrno>
+#include <cstring>
 
-#if HAVE_STRINGS_H
-# include <strings.h>
-#endif
-
-#ifdef		HAVE_SYS_TYPES_H
-#	 include <sys/types.h>
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
 #endif	// HAVE_SYS_TYPES_H
 
-#ifdef		HAVE_SYS_WAIT_H
-#	 include <sys/wait.h>
+#ifdef HAVE_SYS_WAIT_H
+#include <sys/wait.h>
 #endif	// HAVE_SYS_WAIT_H
 
-#ifdef		HAVE_UNISTD_H
-#	 include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
 #endif	// HAVE_UNISTD_H
 
-#ifdef		HAVE_SYS_STAT_H
-#	 include <sys/stat.h>
+#ifdef	HAVE_SYS_STAT_H
+#include <sys/stat.h>
 #endif	// HAVE_SYS_STAT_H
 
 #include <X11/Xlib.h>
@@ -104,6 +97,10 @@ Keys::t_actionstr Keys::m_actionlist[] = {
 	{"PrevWindow", PREVWINDOW},
 	{"NextTab", NEXTTAB},
 	{"PrevTab", PREVTAB},
+	{"FirstTab", FIRSTTAB},
+	{"LastTab", LASTTAB},
+	{"MoveTabPrev", MOVETABPREV},
+	{"MoveTabNext", MOVETABNEXT},
 	{"ShadeWindow", SHADE},
 	{"MaximizeWindow", MAXIMIZE},
 	{"StickWindow", STICK},
@@ -128,12 +125,12 @@ Keys::t_actionstr Keys::m_actionlist[] = {
 	{0, LASTKEYGRAB}
 	};	
 
-Keys::Keys(Display *display, char *filename):
+Keys::Keys(Display *display, const char *filename):
 m_abortkey(0),
 m_display(display)
 {
 	assert(display);
-	if (filename)
+	if (filename != 0)
 		load(filename);
 }
 
@@ -147,7 +144,7 @@ Keys::~Keys() {
 //--------------------------------
 void Keys::deleteTree() {
 	while (!m_keylist.empty()) {
-		if (m_keylist.back())
+		if (m_keylist.back() && m_keylist.back() != 0)
 			delete m_keylist.back();		
 		m_keylist.pop_back();
 	}
@@ -172,40 +169,40 @@ void Keys::ungrabKeys() {
 // Returns true on success else false
 // TODO: error checking
 //------------------------------------
-bool Keys::load(char *filename) {
+bool Keys::load(const char *filename) {
 	if (!filename)
 		return false;
 	
 	//ungrab all keys
 	ungrabKeys();
+
 	//free memory of previous grabs
 	deleteTree();
-			
+
 	XSync(m_display, False);
 						
 	//open the file
 	ifstream infile(filename);
 	if (!infile)
-		return false;
-	
-	
-	auto_ptr<char> linebuffer(new char[1024]);
-	
+		return false; // faild to open file
+
 	int line=0;//current line, so we can tell the user where the fault is
 
 	while (!infile.eof()) {
-		
-		infile.getline(linebuffer.get(), 1024);		
+		string linebuffer;
+
+		getline(infile, linebuffer);
 
 		line++;
 		vector<string> val;
 		//Parse arguments
-		StringUtil::stringtok(val, linebuffer.get());
+		StringUtil::stringtok(val, linebuffer.c_str());
+
 		//must have at least 1 argument
-		if (val.size()<=0)
+		if (val.size() <= 0)
 			continue;
 			
-		if (val[0][0]=='#') //the line is commented
+		if (val[0][0] == '#') //the line is commented
 			continue;
 		
 		unsigned int key=0, mod=0;		
@@ -224,8 +221,13 @@ bool Keys::load(char *filename) {
 					int tmpmod=getModifier(val[argc].c_str());
 					if(tmpmod)
 						mod|=tmpmod; //If it's a modifier
-					else{ 
+					else { 
 						key = getKey(val[argc].c_str()); // else get the key
+						if (key == 0) {
+							cerr<<"["<<filename<<"]: Invalid key/modifier on line("<<
+								line<<"): "<<linebuffer<<endl;
+							break; // get next line
+						}
 						if (!current_key) {
 							current_key = new t_key(key, mod);
 							last_key = current_key;
@@ -250,7 +252,7 @@ bool Keys::load(char *filename) {
 				if (i < LASTKEYGRAB ) {
 					if (!current_key) {
 						cerr<<"Error on line: "<<line<<endl;
-						cerr<<linebuffer.get()<<endl;
+						cerr<<linebuffer<<endl;
 						delete current_key;
 						current_key = 0;
 						last_key = 0;
@@ -277,7 +279,7 @@ bool Keys::load(char *filename) {
 						case Keys::EXECUTE:
 						last_key->execcommand = 
 								const_cast<char *>
-								(StringUtil::strcasestr(linebuffer.get(),
+								(StringUtil::strcasestr(linebuffer.c_str(),
 									getActionStr(Keys::EXECUTE))+
 							strlen(getActionStr(Keys::EXECUTE)));
 						break;
@@ -327,9 +329,9 @@ bool Keys::load(char *filename) {
 					last_key = 0;
 					
 				}	else { //destroy list if no action is found
-					#ifdef DEBUG
+#ifdef DEBUG
 					cerr<<"Didnt find action="<<val[argc]<<endl;
-					#endif					
+#endif // DEBUG			
 					//destroy current_key ... this will also destroy the last_key										
 					delete current_key;
 					current_key = 0;
@@ -421,9 +423,10 @@ unsigned int Keys::getModifier(const char *modstr) {
 		{"MOD5", Mod5Mask},
 		{0, 0}
 		};
-		
+
+	// find mod mask string
 	for (unsigned int i=0; modlist[i].string!=0; i++) {
-		if (modlist[i]==modstr)		
+		if (modlist[i] == modstr)		
 			return modlist[i].mask;		
 	}
 	
@@ -498,7 +501,7 @@ Keys::KeyAction Keys::getAction(XKeyEvent *ke) {
 // deletes the tree and load configuration
 // returns true on success else false
 //-----------------------------------
-bool Keys::reconfigure(char *filename) {
+bool Keys::reconfigure(const char *filename) {
 	deleteTree();
 	return load(filename);
 }
@@ -549,6 +552,7 @@ void Keys::showKeyTree(t_key *key, unsigned int w) {
 		cerr<<"( "<<(int)key->key<<" "<<(int)key->mod<<" ) {"<<getActionStr(key->action)<<"}"<<endl;
 }
 #endif //DEBUG
+
 //------------ mergeTree ---------------
 // Merges two chains and binds new keys
 // Returns true on success else false.
@@ -618,7 +622,9 @@ Keys::t_key::t_key(t_key *k) {
 Keys::t_key::~t_key() {	
 	while (!keylist.empty()) {		
 		t_key *k = keylist.back();
-		delete k;
-		keylist.pop_back();				
+		if (k != 0) { // make sure we don't have a bad key pointer
+			delete k;
+			keylist.pop_back();
+		}
 	}
 }
