@@ -19,8 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// stupid macros needed to access some functions in version 2 of the GNU C
-// library
+//Use GNU extensions
 #ifndef   _GNU_SOURCE
 #define   _GNU_SOURCE
 #endif // _GNU_SOURCE
@@ -51,7 +50,10 @@
 #endif // MAXPATHLEN
 
 
-Rootmenu::Rootmenu(BScreen *scrn) : Basemenu(scrn) {
+Rootmenu::Rootmenu(BScreen *scrn)
+: Basemenu(scrn),
+  auto_group_window(0)
+{
   screen = scrn;  
 }
 
@@ -74,7 +76,10 @@ void Rootmenu::itemSelected(int button, unsigned int index) {
 					sprintf(displaystring + strlen(displaystring) - 1, "%d",
 						screen->getScreenNumber());
 
+					screen->setAutoGroupWindow(useAutoGroupWindow());
+
 					bexec(item->exec().c_str(), displaystring);
+
 					#else //   __EMX__
 					spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", item->exec().c_str(), NULL);
 					#endif // !__EMX__
@@ -92,19 +97,20 @@ void Rootmenu::itemSelected(int button, unsigned int index) {
 
 			case BScreen::EXIT:
 				fluxbox->shutdown();
-				break;
+			break;
 
 			case BScreen::SETSTYLE:
 				if (item->exec().size()) {
 					fluxbox->saveStyleFilename(item->exec().c_str());
-					fluxbox->reconfigureTabs(); //TODO
+					fluxbox->reconfigureTabs();
 				}
-
+				fluxbox->reconfigure();
+				fluxbox->save_rc();
+			break;
 			case BScreen::RECONFIGURE:
 				fluxbox->reconfigure();
 				return;
 			}
-
 			if (! (screen->getRootmenu()->isTorn() || isTorn()) &&
 					item->function() != BScreen::RECONFIGURE &&
 					item->function() != BScreen::SETSTYLE)
@@ -113,3 +119,23 @@ void Rootmenu::itemSelected(int button, unsigned int index) {
 	}
 }
 
+void Rootmenu::setAutoGroupWindow(Window window)
+{
+	auto_group_window = window;
+}
+
+Window Rootmenu::useAutoGroupWindow()
+{
+	// Return and clear the auto-grouping state.
+	Window w = auto_group_window;
+	if (w)
+		auto_group_window = 0;	// clear it immediately
+	// If not set check the parent and the parent's parent, ...
+	else {
+		// TODO: dynamic_cast throws std::bad_cast!
+		Rootmenu *p = dynamic_cast<Rootmenu*>(parent());
+		if (p)
+			w = p->useAutoGroupWindow();
+	}
+	return w;
+}
