@@ -16,13 +16,13 @@
 //
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.	IN NO EVENT SHALL
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 // THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Screen.cc,v 1.46 2002/04/12 14:56:15 fluxgen Exp $
+// $Id: Screen.cc,v 1.65 2002/08/30 14:03:31 fluxgen Exp $
 
 //use GNU extensions
 #ifndef	 _GNU_SOURCE
@@ -37,7 +37,6 @@
 
 #include "i18n.hh"
 #include "fluxbox.hh"
-#include "Clientmenu.hh"
 #include "Icon.hh"
 #include "Image.hh"
 #include "StringUtil.hh"
@@ -85,10 +84,6 @@
 #define	 MAXPATHLEN 255
 #endif // MAXPATHLEN
 
-#ifndef	 FONT_ELEMENT_SIZE
-#define	 FONT_ELEMENT_SIZE 50
-#endif // FONT_ELEMENT_SIZE
-
 #include <X11/Xatom.h>
 #include <X11/keysym.h>
 
@@ -202,6 +197,8 @@ max_over_slit(rm, true, scrname+".maxOverSlit",altscrname+".MaxOverSlit"),
 tab_rotate_vertical(rm, true, scrname+".tab.rotatevertical", altscrname+".Tab.RotateVertical"),
 sloppy_window_grouping(rm, true, scrname+".sloppywindowgrouping", altscrname+".SloppyWindowGrouping"),
 workspace_warping(rm, true, scrname+".workspacewarping", altscrname+".WorkspaceWarping"),
+desktop_wheeling(rm, true, scrname+".desktopwheeling", altscrname+".DesktopWheeling"),
+show_window_pos(rm, true, scrname+".showwindowposition", altscrname+".ShowWindowPosition"),
 focus_last(rm, true, scrname+".focusLastWindow", altscrname+".FocusLastWindow"),
 focus_new(rm, true, scrname+".focusNewWindows", altscrname+".FocusNewWindows"),
 rootcommand(rm, "", scrname+".rootCommand", altscrname+".RootCommand"),
@@ -257,10 +254,6 @@ resource(rm, screenname, altscreenname)
 
 	rootmenu = 0;
 		
-	#ifdef HAVE_STRFTIME
-	resource.strftime_format = 0;
-	#endif // HAVE_STRFTIME
-
 	#ifdef HAVE_GETPID
 	pid_t bpid = getpid();
 
@@ -275,7 +268,7 @@ resource(rm, screenname, altscreenname)
 		fluxbox->getSessionCursor());
 
 	image_control =
-		new BImageControl(fluxbox, this, true, fluxbox->getColorsPerChannel(),
+		new BImageControl(fluxbox, this, true, fluxbox->colorsPerChannel(),
 			fluxbox->getCacheLife(), fluxbox->getCacheMax());
 	image_control->installRootColormap();
 	root_colormap_installed = true;
@@ -283,7 +276,7 @@ resource(rm, screenname, altscreenname)
 	fluxbox->load_rc(this);
 
 	image_control->setDither(*resource.image_dither);
-	theme = new Theme(getBaseDisplay()->getXDisplay(), getRootWindow(), getColormap(), getScreenNumber(), 
+	theme = new Theme(getBaseDisplay()->getXDisplay(), getRootWindow(), colormap(), getScreenNumber(), 
 			image_control, fluxbox->getStyleFilename(), getRootCommand().c_str());
 
 	#ifdef GNOME
@@ -327,8 +320,8 @@ resource(rm, screenname, altscreenname)
 
 	XSetWindowAttributes attrib;
 	unsigned long mask = CWBorderPixel | CWColormap | CWSaveUnder;
-	attrib.border_pixel = getBorderColor()->getPixel();
-	attrib.colormap = getColormap();
+	attrib.border_pixel = getBorderColor()->pixel();
+	attrib.colormap = colormap();
 	attrib.save_under = true;
 
 	geom_window =
@@ -337,12 +330,12 @@ resource(rm, screenname, altscreenname)
 			InputOutput, getVisual(), mask, &attrib);
 	geom_visible = false;
 
-	if (theme->getWindowStyle().l_focus.getTexture() & BImage::PARENTRELATIVE) {
-		if (theme->getWindowStyle().t_focus.getTexture() ==
-			(BImage::FLAT | BImage::SOLID)) {
+	if (theme->getWindowStyle().l_focus.type() & FbTk::Texture::PARENTRELATIVE) {
+		if (theme->getWindowStyle().t_focus.type() ==
+			(FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			geom_pixmap = None;
 			XSetWindowBackground(getBaseDisplay()->getXDisplay(), geom_window,
-				 theme->getWindowStyle().t_focus.getColor()->getPixel());
+				 theme->getWindowStyle().t_focus.color().pixel());
 		} else {
 			geom_pixmap = image_control->renderImage(geom_w, geom_h,
 				 &theme->getWindowStyle().t_focus);
@@ -350,11 +343,11 @@ resource(rm, screenname, altscreenname)
 				 geom_window, geom_pixmap);
 		}
 	} else {
-		if (theme->getWindowStyle().l_focus.getTexture() ==
-				(BImage::FLAT | BImage::SOLID)) {
+		if (theme->getWindowStyle().l_focus.type() ==
+				(FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			geom_pixmap = None;
 			XSetWindowBackground(getBaseDisplay()->getXDisplay(), geom_window,
-				 theme->getWindowStyle().l_focus.getColor()->getPixel());
+				 theme->getWindowStyle().l_focus.color().pixel());
 		} else {
 			geom_pixmap = image_control->renderImage(geom_w, geom_h,
 				 &theme->getWindowStyle().l_focus);
@@ -398,9 +391,14 @@ resource(rm, screenname, altscreenname)
 
 	initMenu();
 
-	raiseWindows(0, 0);
-	rootmenu->update();
+	raiseWindows(Workspace::Stack());
 
+	//update menus
+	rootmenu->update();
+	#ifdef SLIT
+	slit->reconfigure();
+	#endif
+	
 	changeWorkspaceID(0);
 	updateNetizenWorkspaceCount();
 	
@@ -476,7 +474,7 @@ namespace {
 	}
 }
 
-BScreen::~BScreen(void) {
+BScreen::~BScreen() {
 	#ifdef GNOME
 	XDestroyWindow(getBaseDisplay()->getXDisplay(), gnome_win);
 	#endif
@@ -490,6 +488,32 @@ BScreen::~BScreen(void) {
 
 	removeWorkspaceNames();
 
+#ifdef __INTEL_COMPILER
+	//Didn't got icc to work with std::for_each
+	//so we do this by hand.
+	
+	Workspaces::iterator w_it = workspacesList.begin();
+	Workspaces::iterator w_it_end = workspacesList.end();
+	for(; w_it != w_it_end; ++w_it) {
+		delete (*w_it);
+	}
+	workspacesList.clear();
+	
+	Icons::iterator i_it = iconList.begin();
+	Icons::iterator i_it_end = iconList.end();
+	for(; i_it != i_it_end; ++i_it) {
+		delete (*i_it);
+	}
+	iconList.clear();
+	
+	Netizens::iterator n_it = netizenList.begin();
+	Netizens::iterator n_it_end = netizenList.end();
+	for(; n_it != n_it_end; ++n_it) {
+		delete (*n_it);
+	}
+	netizenList.clear();
+		
+#else //__INTEL_COMPILER
 	std::for_each(
 		workspacesList.begin(),
 		workspacesList.end(),
@@ -506,11 +530,7 @@ BScreen::~BScreen(void) {
 		netizenList.begin(),
 		netizenList.end(),
 		delete_obj<Netizen>);
-
-#ifdef		HAVE_STRFTIME
-	if (resource.strftime_format)
-		delete [] resource.strftime_format;
-#endif // HAVE_STRFTIME
+#endif //!__INTEL_COMPILER
 
 	delete rootmenu;
 	delete workspacemenu;
@@ -528,7 +548,7 @@ BScreen::~BScreen(void) {
 
 }
 
-void BScreen::reconfigure(void) {
+void BScreen::reconfigure() {
 	#ifdef DEBUG
 	cerr<<__FILE__<<"("<<__LINE__<<"): BScreen::reconfigure"<<endl;
 	#endif
@@ -561,12 +581,12 @@ void BScreen::reconfigure(void) {
 	geom_h += getBevelWidth()*2;
 
 	Pixmap tmp = geom_pixmap;
-	if (theme->getWindowStyle().l_focus.getTexture() & BImage::PARENTRELATIVE) {
-		if (theme->getWindowStyle().t_focus.getTexture() ==
-				(BImage::FLAT | BImage::SOLID)) {
+	if (theme->getWindowStyle().l_focus.type() & FbTk::Texture::PARENTRELATIVE) {
+		if (theme->getWindowStyle().t_focus.type() ==
+				(FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			geom_pixmap = None;
 			XSetWindowBackground(getBaseDisplay()->getXDisplay(), geom_window,
-				theme->getWindowStyle().t_focus.getColor()->getPixel());
+				theme->getWindowStyle().t_focus.color().pixel());
 		} else {
 			geom_pixmap = image_control->renderImage(geom_w, geom_h,
 				&theme->getWindowStyle().t_focus);
@@ -574,11 +594,11 @@ void BScreen::reconfigure(void) {
 				geom_window, geom_pixmap);
 		}
 	} else {
-		if (theme->getWindowStyle().l_focus.getTexture() ==
-				(BImage::FLAT | BImage::SOLID)) {
+		if (theme->getWindowStyle().l_focus.type() ==
+				(FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			geom_pixmap = None;
 			XSetWindowBackground(getBaseDisplay()->getXDisplay(), geom_window,
-				theme->getWindowStyle().l_focus.getColor()->getPixel());
+				theme->getWindowStyle().l_focus.color().pixel());
 		} else {
 			geom_pixmap = image_control->renderImage(geom_w, geom_h,
 				&theme->getWindowStyle().l_focus);
@@ -589,35 +609,39 @@ void BScreen::reconfigure(void) {
 	if (tmp) image_control->removeImage(tmp);
 
 	XSetWindowBorderWidth(getBaseDisplay()->getXDisplay(), geom_window,
-												theme->getBorderWidth());
+		theme->getBorderWidth());
 	XSetWindowBorder(getBaseDisplay()->getXDisplay(), geom_window,
-									 theme->getBorderColor().getPixel());
+		theme->getBorderColor().pixel());
 
+	//reconfigure menus
 	workspacemenu->reconfigure();
 	iconmenu->reconfigure();
 
+	configmenu->reconfigure();
+	
 	{
 		int remember_sub = rootmenu->currentSubmenu();
 		initMenu();
-		raiseWindows(0, 0);
-		rootmenu->reconfigure();
+		raiseWindows(Workspace::Stack());
+		rootmenu->reconfigure();		
 		rootmenu->drawSubmenu(remember_sub);
 	}
 
-	configmenu->reconfigure();
 
 	toolbar->reconfigure();
 
 #ifdef		SLIT
 	slit->reconfigure();
 #endif // SLIT
-
+	
+	//reconfigure workspaces
 	Workspaces::iterator wit = workspacesList.begin();
 	Workspaces::iterator wit_end = workspacesList.end();
 	for (; wit != wit_end; ++wit) {
 		(*wit)->reconfigure();
 	}
 
+	//reconfigure Icons
 	Icons::iterator iit = iconList.begin();
 	Icons::iterator iit_end = iconList.end();
 	for (; iit != iit_end; ++iit) {
@@ -629,19 +653,19 @@ void BScreen::reconfigure(void) {
 }
 
 
-void BScreen::rereadMenu(void) {
+void BScreen::rereadMenu() {
 	initMenu();
-	raiseWindows(0, 0);
+	raiseWindows(Workspace::Stack());
 
 	rootmenu->reconfigure();
 }
 
 
-void BScreen::removeWorkspaceNames(void) {
+void BScreen::removeWorkspaceNames() {
 	workspaceNames.erase(workspaceNames.begin(), workspaceNames.end());
 }
 
-void BScreen::updateWorkspaceNamesAtom(void) {
+void BScreen::updateWorkspaceNamesAtom() {
 
 #ifdef GNOME	
 	XTextProperty	text;
@@ -658,7 +682,7 @@ void BScreen::updateWorkspaceNamesAtom(void) {
 	
 	if (XStringListToTextProperty(names, number_of_desks, &text)) {
 		XSetTextProperty(getBaseDisplay()->getXDisplay(), getRootWindow(),
-			 &text, getBaseDisplay()->getGnomeWorkspaceNamesAtom());
+			 &text, FbAtoms::instance()->getGnomeWorkspaceNamesAtom());
 		XFree(text.value);
 	}
 	
@@ -717,9 +741,10 @@ FluxboxWindow *BScreen::getIcon(unsigned int index) {
 }
 
 
-int BScreen::addWorkspace(void) {
+int BScreen::addWorkspace() {
 	Workspace *wkspc = new Workspace(this, workspacesList.size());
 	workspacesList.push_back(wkspc);
+	addWorkspaceName(wkspc->name().c_str()); // update names
 	//add workspace to workspacemenu
 	workspacemenu->insert(wkspc->name().c_str(), wkspc->menu(),
 		wkspc->workspaceID() + 2); //+2 so we add it after "remove last"
@@ -736,7 +761,7 @@ int BScreen::addWorkspace(void) {
 }
 
 
-int BScreen::removeLastWorkspace(void) {
+int BScreen::removeLastWorkspace() {
 	if (workspacesList.size() > 1) {
 		Workspace *wkspc = workspacesList.back();
 
@@ -769,16 +794,20 @@ void BScreen::changeWorkspaceID(unsigned int id) {
 	
 	if (id != current_workspace->workspaceID()) {
 		XSync(fluxbox->getXDisplay(), true);
-		
+		FluxboxWindow *focused = fluxbox->getFocusedWindow();
+
+		if (focused && focused->isMoving()) {
+			reassociateGroup(focused, id, true);
+			focused->pauseMoving();
+		}
+
 		current_workspace->hideAll();
 
 		workspacemenu->setItemSelected(current_workspace->workspaceID() + 2, false);
 
-		if (fluxbox->getFocusedWindow() &&
-				fluxbox->getFocusedWindow()->getScreen() == this &&
-				(! fluxbox->getFocusedWindow()->isStuck())) {
-				
-			current_workspace->setLastFocusedWindow(fluxbox->getFocusedWindow());
+		if (focused && focused->getScreen() == this &&
+				(! focused->isStuck()) && (!focused->isMoving())) {
+			current_workspace->setLastFocusedWindow(focused);
 			fluxbox->setFocusedWindow((FluxboxWindow *) 0);
 			
 		}
@@ -790,25 +819,30 @@ void BScreen::changeWorkspaceID(unsigned int id) {
 
 		current_workspace->showAll();
 
-		if (*resource.focus_last && current_workspace->getLastFocusedWindow())
+		if (*resource.focus_last && current_workspace->getLastFocusedWindow() &&
+				!(focused && focused->isMoving())) {
 			current_workspace->getLastFocusedWindow()->setInputFocus();		
-			
+		}
+
+		if (focused && focused->isMoving()) {
+			focused->resumeMoving();
+		}
 	}
 
 	updateNetizenCurrentWorkspace();
 }
 
 
-void BScreen::sendToWorkspace(unsigned int id, bool changeWS) {
-	FluxboxWindow *win;
+void BScreen::sendToWorkspace(unsigned int id, FluxboxWindow *win, bool changeWS) {
 	if (! current_workspace || id >= workspacesList.size())
 		return;
 
+	if (!win)
+		win = fluxbox->getFocusedWindow();
+
 	if (id != current_workspace->workspaceID()) {
 		XSync(fluxbox->getXDisplay(), True);
 
-		win = fluxbox->getFocusedWindow();
-
 		if (win && win->getScreen() == this &&
 				(! win->isStuck())) {
 
@@ -864,7 +898,7 @@ void BScreen::removeNetizen(Window w) {
 }
 
 
-void BScreen::updateNetizenCurrentWorkspace(void) {
+void BScreen::updateNetizenCurrentWorkspace() {
 	#ifdef NEWWMSPEC
 	//update _NET_WM_CURRENT_DESKTOP
 	int workspace = getCurrentWorkspaceID();
@@ -876,7 +910,7 @@ void BScreen::updateNetizenCurrentWorkspace(void) {
 	//update _WIN_WORKSPACE
 	int gnome_workspace = getCurrentWorkspaceID();
 	XChangeProperty(getBaseDisplay()->getXDisplay(), getRootWindow(),
-		getBaseDisplay()->getGnomeWorkspaceAtom(), XA_CARDINAL, 32, PropModeReplace,
+		FbAtoms::instance()->getGnomeWorkspaceAtom(), XA_CARDINAL, 32, PropModeReplace,
 			(unsigned char *)&gnome_workspace, 1);
 	updateGnomeClientList();
 	#endif
@@ -890,7 +924,7 @@ void BScreen::updateNetizenCurrentWorkspace(void) {
 }
 
 
-void BScreen::updateNetizenWorkspaceCount(void) {
+void BScreen::updateNetizenWorkspaceCount() {
 
 	Netizens::iterator it = netizenList.begin();
 	Netizens::iterator it_end = netizenList.end();
@@ -909,7 +943,7 @@ void BScreen::updateNetizenWorkspaceCount(void) {
 	{
 	int numworkspaces = getCount();
 	XChangeProperty(getBaseDisplay()->getXDisplay(), getRootWindow(),
-		getBaseDisplay()->getGnomeWorkspaceCountAtom(), XA_CARDINAL, 32, PropModeReplace,
+		FbAtoms::instance()->getGnomeWorkspaceCountAtom(), XA_CARDINAL, 32, PropModeReplace,
 			(unsigned char *)&numworkspaces, 1);
 	}	
 	#endif
@@ -917,7 +951,7 @@ void BScreen::updateNetizenWorkspaceCount(void) {
 }
 
 
-void BScreen::updateNetizenWindowFocus(void) {
+void BScreen::updateNetizenWindowFocus() {
 
 	Netizens::iterator it = netizenList.begin();
 	Netizens::iterator it_end = netizenList.end();
@@ -981,9 +1015,9 @@ void BScreen::updateNetizenConfigNotify(XEvent *e) {
 }
 
 
-void BScreen::raiseWindows(Window *workspace_stack, int num) {
+void BScreen::raiseWindows(const Workspace::Stack &workspace_stack) {
 
-	Window session_stack[(num + workspacesList.size() + rootmenuList.size() + 30)];
+	Window session_stack[(workspace_stack.size() + workspacesList.size() + rootmenuList.size() + 30)];
 	int i = 0;	
 	XRaiseWindow(getBaseDisplay()->getXDisplay(), iconmenu->windowID());
 	session_stack[i++] = iconmenu->windowID();
@@ -1002,14 +1036,14 @@ void BScreen::raiseWindows(Window *workspace_stack, int num) {
 	session_stack[i++] = configmenu->windowID();
 
 	#ifdef		SLIT
-	session_stack[i++] = slit->getMenu().getDirectionmenu()->windowID();
-	session_stack[i++] = slit->getMenu().getPlacementmenu()->windowID();
+	session_stack[i++] = slit->menu().getDirectionmenu()->windowID();
+	session_stack[i++] = slit->menu().getPlacementmenu()->windowID();
 	#ifdef XINERAMA
 	if (hasXinerama()) {
-		session_stack[i++] = slit->getMenu().getHeadmenu()->windowID();
+		session_stack[i++] = slit->menu().getHeadmenu()->windowID();
 	}
 	#endif // XINERAMA
-	session_stack[i++] = slit->getMenu().windowID();
+	session_stack[i++] = slit->menu().windowID();
 	#endif // SLIT
 
 	session_stack[i++] =
@@ -1035,46 +1069,57 @@ void BScreen::raiseWindows(Window *workspace_stack, int num) {
 	if (slit->isOnTop())
 		session_stack[i++] = slit->getWindowID();
 	#endif // SLIT
-	
-	int k=num;
-	while (k--)
-		session_stack[i++] = *(workspace_stack + k);
+
+	if (!workspace_stack.empty()) {
+		Workspace::Stack::const_reverse_iterator it = workspace_stack.rbegin();
+		Workspace::Stack::const_reverse_iterator it_end = workspace_stack.rend();
+		for (; it != it_end; ++it)
+			session_stack[i++] = (*it);
+	}
 
 	XRestackWindows(getBaseDisplay()->getXDisplay(), session_stack, i);
 
 }
 
 #ifdef		HAVE_STRFTIME
-void BScreen::saveStrftimeFormat(char *format) {
-	if (resource.strftime_format)
-		delete [] resource.strftime_format;
-
-	resource.strftime_format = StringUtil::strdup(format);
+void BScreen::saveStrftimeFormat(const char *format) {
+	//make sure std::string don't get 0 string
+	resource.strftime_format = (format ? format : "");
 }
 #endif // HAVE_STRFTIME
 
 
-void BScreen::addWorkspaceName(char *name) {
+void BScreen::addWorkspaceName(const char *name) {
 	workspaceNames.push_back(name);
-	
 }
 
 
-void BScreen::getNameOfWorkspace(unsigned int workspace, char **name) {
+string BScreen::getNameOfWorkspace(unsigned int workspace) const {
 	if (workspace < workspaceNames.size()) {
-		const char *wkspc_name = workspaceNames[workspace].c_str();
+		return workspaceNames[workspace];
+	} else {
+		return "";
+	}
 
-		if (wkspc_name)
-			*name = StringUtil::strdup(wkspc_name);
-	} else
-		*name = 0;
+}
+
+void BScreen::reassociateGroup(FluxboxWindow *w, unsigned int wkspc_id, bool ignore_sticky) {
+	if (w->hasTab() && (w->getTab()->next() || w->getTab()->prev())) {
+		Tab *tab_it = w->getTab()->first();
+		for (; tab_it; tab_it = tab_it->next()) {
+			reassociateWindow(tab_it->getWindow(), wkspc_id, ignore_sticky);
+		}
+	} else {
+		// no tab, juts move this one
+		reassociateWindow(w, wkspc_id, ignore_sticky);
+	}
 }
 
 
 void BScreen::reassociateWindow(FluxboxWindow *w, unsigned int wkspc_id, bool ignore_sticky) {
 	if (! w) return;
 
-	if (wkspc_id >= workspaceNames.size())
+	if (wkspc_id >= getCount())
 		wkspc_id = current_workspace->workspaceID();
 
 	if (w->getWorkspaceNumber() == wkspc_id)
@@ -1093,22 +1138,23 @@ void BScreen::reassociateWindow(FluxboxWindow *w, unsigned int wkspc_id, bool ig
 void BScreen::nextFocus(int opts) {
 	bool have_focused = false;
 	int focused_window_number = -1;
-	FluxboxWindow *focused;
+	FluxboxWindow *focused = fluxbox->getFocusedWindow();
 	const int num_windows = getCurrentWorkspace()->getCount();
 	
-	if ((focused = fluxbox->getFocusedWindow())) {
-		if (focused->getScreen()->getScreenNumber() ==
-			getScreenNumber()) {
+	if (focused != 0) {
+		if (focused->getScreen()->getScreenNumber() == 
+				getScreenNumber()) {
 			have_focused = true;
 			focused_window_number = focused->getWindowNumber();
 		}
 	}
 
-	if (num_windows > 1 && have_focused) {
+	if (num_windows > 1 && have_focused && !focused->isStuck()) {
 		Workspace *wksp = getCurrentWorkspace();
-		Workspace::Windows wins = wksp->getWindowList();
+		Workspace::Windows &wins = wksp->getWindowList();
 		Workspace::Windows::iterator it = wins.begin();
-		for (; *it != focused; ++it);
+		for (; *it != focused; ++it); //get focused window iterator
+
 		do {
 			++it;
 			if (it == wins.end())
@@ -1118,8 +1164,9 @@ void BScreen::nextFocus(int opts) {
 				break;
 		} while (*it != focused);
 
-		if (*it != focused)
+		if (*it != focused && it != wins.end())
 			wksp->raiseWindow(*it);
+
 	} else if (num_windows >= 1) {
 		FluxboxWindow *next = current_workspace->getWindow(0);
 		//don't raise next window if input focus fails
@@ -1144,7 +1191,7 @@ void BScreen::prevFocus(int opts) {
 		}
 	}
 
-	if (num_windows > 1 && have_focused) {
+	if (num_windows > 1 && have_focused && !focused->isStuck()) {
 		Workspace *wksp = getCurrentWorkspace();
 		Workspace::Windows wins = wksp->getWindowList();
 		Workspace::Windows::iterator it = wins.begin();
@@ -1171,7 +1218,7 @@ void BScreen::prevFocus(int opts) {
 //--------- raiseFocus -----------
 // Raise the current focused window
 //--------------------------------
-void BScreen::raiseFocus(void) {
+void BScreen::raiseFocus() {
 	bool have_focused = false;
 	int focused_window_number = -1;
 
@@ -1187,14 +1234,14 @@ void BScreen::raiseFocus(void) {
 			raiseWindow(fluxbox->getFocusedWindow());
 }
 
-void BScreen::initMenu(void) {
+void BScreen::initMenu() {
 	I18n *i18n = I18n::instance();
 	
 	if (rootmenu) {
 		rootmenuList.erase(rootmenuList.begin(), rootmenuList.end());
 
 		while (rootmenu->numberOfItems())
-			rootmenu->remove(0);
+			rootmenu->remove(0);			
 	} else
 		rootmenu = new Rootmenu(this);
 
@@ -1326,12 +1373,11 @@ bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						 	"no menu label and/or filename defined\n"));
 						cerr<<"Row: "<<row<<endl;
 					} else {
-						char *style;
-				
 						// perform shell style ~ home directory expansion
-						style = StringUtil::expandFilename(str_cmd.c_str());
-						menu->insert(str_label.c_str(), BScreen::SETSTYLE, style);
-						delete style;
+						// and insert style
+						menu->insert(str_label.c_str(), BScreen::SETSTYLE, 
+							StringUtil::expandFilename(str_cmd).c_str());
+						
 					}
 				} // end of style
 				
@@ -1357,13 +1403,11 @@ bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						 	"no filename defined\n"));
 						cerr<<"Row: "<<row<<endl;
 					} else {	// start of else 'x'
-						char *newfile;
-
 						// perform shell style ~ home directory expansion
-						newfile = StringUtil::expandFilename(str_label.c_str());
+						string newfile(StringUtil::expandFilename(str_label));
 
-						if (newfile) {
-							FILE *submenufile = fopen(newfile, "r");
+						if (newfile.size() != 0) {
+							FILE *submenufile = fopen(newfile.c_str(), "r");
 
 							if (submenufile) {
 								struct stat buf;
@@ -1374,19 +1418,18 @@ bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 									getMessage(
 							 			FBNLS::ScreenSet, FBNLS::ScreenINCLUDEErrorReg,
 							 			"BScreen::parseMenuFile: [include] error: "
-							 			"'%s' is not a regular file\n"), newfile);
+							 			"'%s' is not a regular file\n"), newfile.c_str());
 									cerr<<"Row: "<<row<<endl;
 								}
 								
 								if (! feof(submenufile)) {
 									fclose(submenufile);
-									ifstream subfile(newfile);
+									ifstream subfile(newfile.c_str());
 									if (! parseMenuFile(subfile, menu, row))
-										fluxbox->saveMenuFilename(newfile);
+										fluxbox->saveMenuFilename(newfile.c_str());
 								}
 							} else
-								perror(newfile);
-							delete newfile;
+								perror(newfile.c_str());
 						} 
 					} // end of else 'x'
 				} // end of include
@@ -1475,14 +1518,14 @@ bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 }
 
 void BScreen::createStyleMenu(Rootmenu *menu, bool newmenu, const char *label, const char *directory) {
-	I18n *i18n = I18n::instance();
-	
+
 	// perform shell style ~ home directory expansion
-	auto_ptr<char> stylesdir(StringUtil::expandFilename(directory));
-						
+	string stylesdir(StringUtil::expandFilename(directory ? directory : ""));
+
+	I18n *i18n = I18n::instance();						
 	struct stat statbuf;
 
-	if (! stat(stylesdir.get(), &statbuf)) { // stat
+	if (! stat(stylesdir.c_str(), &statbuf)) { // stat
 		if (S_ISDIR(statbuf.st_mode)) { // dir
 			Rootmenu *stylesmenu;
 			if (newmenu)
@@ -1490,7 +1533,7 @@ void BScreen::createStyleMenu(Rootmenu *menu, bool newmenu, const char *label, c
 			else
 				stylesmenu = menu;
 
-			DIR *d = opendir(stylesdir.get());
+			DIR *d = opendir(stylesdir.c_str());
 			int entries = 0;
 			struct dirent *p;
 
@@ -1506,11 +1549,11 @@ void BScreen::createStyleMenu(Rootmenu *menu, bool newmenu, const char *label, c
 
 			qsort(ls, entries, sizeof(char *), dcmp);
 
-			int n, slen = strlen(stylesdir.get());
+			int n, slen = strlen(stylesdir.c_str());
 			for (n = 0; n < entries; n++) { // for
 				int nlen = strlen(ls[n]);
 				char style[MAXPATHLEN + 1];
-				strncpy(style, stylesdir.get(), slen);
+				strncpy(style, stylesdir.c_str(), slen);
 				*(style + slen) = '/';
 				strncpy(style + slen + 1, ls[n], nlen + 1);
 				if ((! stat(style, &statbuf)) && S_ISREG(statbuf.st_mode))
@@ -1527,7 +1570,7 @@ void BScreen::createStyleMenu(Rootmenu *menu, bool newmenu, const char *label, c
 				rootmenuList.push_back(stylesmenu);
 			}
 
-			fluxbox->saveMenuFilename(stylesdir.get());
+			fluxbox->saveMenuFilename(stylesdir.c_str());
 		} else { // dir
 			fprintf(stderr,
 			i18n->
@@ -1535,7 +1578,7 @@ void BScreen::createStyleMenu(Rootmenu *menu, bool newmenu, const char *label, c
 				FBNLS::ScreenSet, FBNLS::ScreenSTYLESDIRErrorNotDir,
 				"BScreen::parseMenuFile:"
 				" [stylesdir/stylesmenu] error, %s is not a"
-				" directory\n"), stylesdir.get());
+				" directory\n"), stylesdir.c_str());
 		} // end of 'dir'
 	} else { // stat
 		fprintf(stderr,
@@ -1543,12 +1586,12 @@ void BScreen::createStyleMenu(Rootmenu *menu, bool newmenu, const char *label, c
 		getMessage(
 			FBNLS::ScreenSet, FBNLS::ScreenSTYLESDIRErrorNoExist,		
 			"BScreen::parseMenuFile: [stylesdir/stylesmenu]"
-			" error, %s does not exist\n"), stylesdir.get());
+			" error, %s does not exist\n"), stylesdir.c_str());
 	} // end of 'stat'
 
 }
 
-void BScreen::shutdown(void) {
+void BScreen::shutdown() {
 	fluxbox->grab();
 
 	XSelectInput(getBaseDisplay()->getXDisplay(), getRootWindow(), NoEventMask);
@@ -1564,7 +1607,7 @@ void BScreen::shutdown(void) {
 
 	{
 		while (!iconList.empty()) {
-			iconList.back()->restore();
+			iconList.back()->restore(true); // restore with remap
 			delete iconList.back(); // the window removes it self from iconlist
 		}
 	}
@@ -1665,7 +1708,7 @@ void BScreen::showGeometry(unsigned int gx, unsigned int gy) {
 }
 
 
-void BScreen::hideGeometry(void) {
+void BScreen::hideGeometry() {
 	if (geom_visible) {
 		XUnmapWindow(getBaseDisplay()->getXDisplay(), geom_window);
 		geom_visible = false;
@@ -1712,35 +1755,44 @@ bool BScreen::doSkipWindow(const FluxboxWindow *w, int opts) {
 			(opts & CYCLESKIPSHADED) != 0 && w->isShaded()); // skip if shaded
 }
 
+//----------- useAutoGroupWindow -------------
+// Access and clear the auto-group window
+//--------------------------------------------
+FluxboxWindow* BScreen::useAutoGroupWindow() {
+	Window w = auto_group_window;
+	auto_group_window = 0;
+	return w ? Fluxbox::instance()->searchWindow(w) : 0;
+}
+
 #ifdef GNOME
-void BScreen::initGnomeAtoms(void) {
+void BScreen::initGnomeAtoms() {
 
 	/* create the GNOME window */
 	gnome_win = XCreateSimpleWindow(getBaseDisplay()->getXDisplay(),
 		getRootWindow(), 0, 0, 5, 5, 0, 0, 0);
-
+	FbAtoms *fba = FbAtoms::instance();
 	/* supported WM check */
 	 XChangeProperty(getBaseDisplay()->getXDisplay(),
-		getRootWindow(), getBaseDisplay()->getGnomeSupportingWMCheckAtom(), 
+		getRootWindow(), fba->getGnomeSupportingWMCheckAtom(), 
 		XA_CARDINAL, 32, PropModeReplace, (unsigned char *) &gnome_win, 1);
 
 	XChangeProperty(getBaseDisplay()->getXDisplay(), gnome_win, 
-		getBaseDisplay()->getGnomeSupportingWMCheckAtom(), 
+		fba->getGnomeSupportingWMCheckAtom(), 
 		XA_CARDINAL, 32, PropModeReplace, (unsigned char *) &gnome_win, 1);
 
 	Atom gnomeatomlist[] = {
-		getBaseDisplay()->getGnomeWorkspaceAtom(),
-		getBaseDisplay()->getGnomeWorkspaceCountAtom(),
-		getBaseDisplay()->getGnomeStateAtom(),
-		getBaseDisplay()->getGnomeWorkspaceNamesAtom(),
-		getBaseDisplay()->getGnomeHintsAtom(),
-		getBaseDisplay()->getGnomeClientListAtom(),
+		fba->getGnomeWorkspaceAtom(),
+		fba->getGnomeWorkspaceCountAtom(),
+		fba->getGnomeStateAtom(),
+		fba->getGnomeWorkspaceNamesAtom(),
+		fba->getGnomeHintsAtom(),
+		fba->getGnomeClientListAtom(),
 //		getBaseDisplay()->getGnomeLayerAtom(), // not supported yet
 	};
 
 	//list atoms that we support
 	XChangeProperty(getBaseDisplay()->getXDisplay(), getRootWindow(), 
-		getBaseDisplay()->getGnomeProtAtom(), XA_ATOM, 32, PropModeReplace,
+		fba->getGnomeProtAtom(), XA_ATOM, 32, PropModeReplace,
 		(unsigned char *)gnomeatomlist, (sizeof gnomeatomlist)/sizeof gnomeatomlist[0]);
 
 }
@@ -1755,13 +1807,14 @@ void BScreen::updateGnomeClientList() {
 	//int num = getCurrentWorkspace()->getWindowList().size();
 	
 	Window *wl = new Window[num];
+	//start the iterator from begining
 	workspace_it = workspacesList.begin();
 	int win=0;
 	for (; workspace_it != workspace_it_end; ++workspace_it) {
 	
 		// Fill in array of window ID's
-		Workspace::Windows::iterator it = (*workspace_it)->getWindowList().begin();
-		Workspace::Windows::iterator it_end = (*workspace_it)->getWindowList().end();		
+		Workspace::Windows::const_iterator it = (*workspace_it)->getWindowList().begin();
+		Workspace::Windows::const_iterator it_end = (*workspace_it)->getWindowList().end();		
 		for (; it != it_end; ++it) {
 			//check if the window don't want to be visible in the list
 			if (! ( (*it)->getGnomeHints() & FluxboxWindow::WIN_STATE_HIDDEN) ) {
@@ -1772,7 +1825,7 @@ void BScreen::updateGnomeClientList() {
 	//number of windows to show in client list
 	num = win;
 	XChangeProperty(getBaseDisplay()->getXDisplay(), 
-		getRootWindow(), getBaseDisplay()->getGnomeClientListAtom(), XA_CARDINAL, 32,
+		getRootWindow(), FbAtoms::instance()->getGnomeClientListAtom(), XA_CARDINAL, 32,
 		PropModeReplace, (unsigned char *)wl, num);
 	
 	if (wl)
