@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Slit.cc,v 1.12 2002/04/08 22:26:58 fluxgen Exp $
+// $Id: Slit.cc,v 1.20 2002/08/14 21:51:07 fluxgen Exp $
 
 //use GNU extensions
 #ifndef	 _GNU_SOURCE
@@ -42,9 +42,59 @@
 #include "Toolbar.hh"
 
 #include <algorithm>
+#include <iostream>
+#include <cassert>
+
+#ifdef HAVE_SYS_STAT_H
+#	include <sys/types.h>
+#	include <sys/stat.h>
+#endif // HAVE_SYS_STAT_H
 
 
-Slit::Slit(BScreen *scr):screen(scr), timer(this), slitmenu(this) {
+// Utility method for extracting name from window
+namespace {
+	void getWMName(BScreen *screen, Window window, std::string& name) {
+		name = "";
+	
+		if (screen != 0 && window != None) {
+			Display *display = screen->getBaseDisplay()->getXDisplay();
+	
+			XTextProperty text_prop;
+			char **list;
+			int num;
+			I18n *i18n = I18n::instance();
+		
+			if (XGetWMName(display, window, &text_prop)) {
+				if (text_prop.value && text_prop.nitems > 0) {
+					if (text_prop.encoding != XA_STRING) {
+						
+						text_prop.nitems = strlen((char *) text_prop.value);
+						
+						if ((XmbTextPropertyToTextList(display, &text_prop,
+									&list, &num) == Success) &&
+								(num > 0) && *list) {
+							name = static_cast<char *>(*list);
+							XFreeStringList(list);
+						} else
+							name = (char *)text_prop.value;
+							
+					} else
+						name = (char *)text_prop.value;
+				} else
+					name = i18n->getMessage(
+						FBNLS::WindowSet, FBNLS::WindowUnnamed,
+						"Unnamed");
+			} else {
+				name = i18n->getMessage(
+					FBNLS::WindowSet, FBNLS::WindowUnnamed,
+					"Unnamed");
+			}
+		}
+	}
+}; // End anonymous namespace
+
+Slit::Slit(BScreen *scr):screen(scr), timer(this), slitmenu(*this) {
+	assert(scr);
 	fluxbox = Fluxbox::instance();
 
 	on_top = screen->isSlitOnTop();
@@ -62,11 +112,11 @@ Slit::Slit(BScreen *scr):screen(scr), timer(this), slitmenu(this) {
 		CWColormap | CWOverrideRedirect | CWEventMask;
 	attrib.background_pixmap = None;
 	attrib.background_pixel = attrib.border_pixel =
-		screen->getBorderColor()->getPixel();
-	attrib.colormap = screen->getColormap();
+		screen->getBorderColor()->pixel();
+	attrib.colormap = screen->colormap();
 	attrib.override_redirect = True;
 	attrib.event_mask = SubstructureRedirectMask | ButtonPressMask |
-											EnterWindowMask | LeaveWindowMask;
+		EnterWindowMask | LeaveWindowMask;
 
 	frame.x = frame.y = 0;
 	frame.width = frame.height = 1;
@@ -78,6 +128,9 @@ Slit::Slit(BScreen *scr):screen(scr), timer(this), slitmenu(this) {
 			create_mask, &attrib);
 	fluxbox->saveSlitSearch(frame.window, this);
 
+	// Get client list for sorting purposes
+	loadClientList();
+
 	reconfigure();
 }
 
@@ -96,10 +149,43 @@ Slit::~Slit() {
 
 
 void Slit::addClient(Window w) {
+	//Can't add non existent window
+	if (w == None)
+		return;
+
 	fluxbox->grab();
 	if (fluxbox->validateWindow(w)) {
-		SlitClient *client = new SlitClient;
-		client->client_window = w;
+		// Look for slot in client list by name
+		SlitClient *client = 0;
+		std::string match_name;
+		::getWMName(screen, w, match_name);
+		SlitClients::iterator it = clientList.begin();
+		SlitClients::iterator it_end = clientList.end();
+		bool found_match = false;
+		for (; it != it_end; ++it) {
+			// If the name matches...
+			if ((*it)->match_name == match_name) {
+				// Use the slot if no window is assigned
+				if ((*it)->window == None) {
+					client = (*it);
+					client->initialize(screen, w);
+					break;
+				}
+				// Otherwise keep looking for an unused match or a non-match
+				found_match = true;		// Possibly redundant
+				
+			} else if (found_match) {
+				// Insert before first non-match after a previously found match?
+				client = new SlitClient(screen, w);
+				clientList.insert(it, client);
+				break;
+			}
+		}
+		// Append to client list?
+		if (client == 0) {
+			client = new SlitClient(screen, w);
+			clientList.push_back(client);
+		}
 
 		XWMHints *wmhints = XGetWMHints(display, w);
 
@@ -180,22 +266,26 @@ void Slit::addClient(Window w) {
 			SubstructureNotifyMask | EnterWindowMask);
 		XFlush(display);
 
-		clientList.push_back(client);
-
 		fluxbox->saveSlitSearch(client->client_window, this);
 		fluxbox->saveSlitSearch(client->icon_window, this);
 		reconfigure();
+
+		saveClientList();
 	}
 
 	fluxbox->ungrab();
 }
 
 
-void Slit::removeClient(SlitClient *client, bool remap) {
+void Slit::removeClient(SlitClient *client, bool remap, bool destroy) {
 	fluxbox->removeSlitSearch(client->client_window);
 	fluxbox->removeSlitSearch(client->icon_window);
 
-    clientList.remove(client);
+	// Destructive removal?
+	if (destroy)
+	    clientList.remove(client);
+	else // Clear the window info, but keep around to help future sorting?
+		client->initialize();
 
 	screen->removeNetizen(client->window);
 
@@ -210,7 +300,9 @@ void Slit::removeClient(SlitClient *client, bool remap) {
 		XFlush(display);
 	}
 
-	delete client;
+	// Destructive removal?
+	if (destroy)
+		delete client;
 }
 
 
@@ -223,7 +315,7 @@ void Slit::removeClient(Window w, bool remap) {
 	SlitClients::iterator it_end = clientList.end();
 	for (; it != it_end; ++it) {
 		if ((*it)->window == w) {
-			removeClient((*it), remap);
+			removeClient((*it), remap, false);
 			reconf = true;
 
 			break;
@@ -239,16 +331,25 @@ void Slit::reconfigure(void) {
 	frame.width = 0;
 	frame.height = 0;
 
+	// Need to count windows because not all client list entries
+	// actually correspond to mapped windows.
+	int num_windows = 0;
+
 	switch (screen->getSlitDirection()) {
 	case VERTICAL:
 		{
 			SlitClients::iterator it = clientList.begin();
 			SlitClients::iterator it_end = clientList.end();
 			for (; it != it_end; ++it) {
-				frame.height += (*it)->height + screen->getBevelWidth();
-
-				if (frame.width < (*it)->width)
-					frame.width = (*it)->width;
+				//client created window?
+				if ((*it)->window != None) {
+					num_windows++;
+					frame.height += (*it)->height + screen->getBevelWidth();
+					
+					//frame width < client window?
+					if (frame.width < (*it)->width) 
+						frame.width = (*it)->width;
+				}
 			}
 		}
 
@@ -269,10 +370,14 @@ void Slit::reconfigure(void) {
 			SlitClients::iterator it = clientList.begin();
 			SlitClients::iterator it_end = clientList.end();
 			for (; it != it_end; ++it) {
-				frame.width += (*it)->width + screen->getBevelWidth();
-
-				if (frame.height < (*it)->height)
-					frame.height = (*it)->height;
+				//client created window?
+				if ((*it)->window != None) {
+					num_windows++;
+					frame.width += (*it)->width + screen->getBevelWidth();
+					//frame height < client height?
+					if (frame.height < (*it)->height)
+						frame.height = (*it)->height;
+				}
 			}
 		}
 
@@ -293,26 +398,29 @@ void Slit::reconfigure(void) {
 
 	XSetWindowBorderWidth(display ,frame.window, screen->getBorderWidth());
 	XSetWindowBorder(display, frame.window,
-		screen->getBorderColor()->getPixel());
+		screen->getBorderColor()->pixel());
 
-	if (clientList.size()==0)
+	//did we actually use slit slots
+	if (num_windows == 0)
 		XUnmapWindow(display, frame.window);
 	else
 		XMapWindow(display, frame.window);
 
 	Pixmap tmp = frame.pixmap;
 	BImageControl *image_ctrl = screen->getImageControl();
-	BTexture *texture = &(screen->getToolbarStyle()->toolbar);
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	const FbTk::Texture *texture = &(screen->getTheme()->getSlitTexture());
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 		frame.pixmap = None;
 		XSetWindowBackground(display, frame.window,
-			 texture->getColor()->getPixel());
+			 texture->color().pixel());
 	} else {
 		frame.pixmap = image_ctrl->renderImage(frame.width, frame.height,
 			texture);
 		XSetWindowBackgroundPixmap(display, frame.window, frame.pixmap);
 	}
-	if (tmp) image_ctrl->removeImage(tmp);
+
+	if (tmp) 
+		image_ctrl->removeImage(tmp);
 	XClearWindow(display, frame.window);
 
 	int x, y;
@@ -326,6 +434,10 @@ void Slit::reconfigure(void) {
 			SlitClients::iterator it = clientList.begin();
 			SlitClients::iterator it_end = clientList.end();
 			for (; it != it_end; ++it) {
+				//client created window?
+				if ((*it)->window == None)
+					continue;
+
 				x = (frame.width - (*it)->width) / 2;
 
 				XMoveResizeWindow(display, (*it)->window, x, y,
@@ -367,6 +479,10 @@ void Slit::reconfigure(void) {
 			SlitClients::iterator it = clientList.begin();
 			SlitClients::iterator it_end = clientList.end();
 			for (; it != it_end; ++it) {
+				//client created window?
+				if ((*it)->window == None)
+					continue;
+
 				y = (frame.height - (*it)->height) / 2;
 
 				XMoveResizeWindow(display, (*it)->window, x, y,
@@ -405,12 +521,12 @@ void Slit::reconfigure(void) {
 }
 
 
-void Slit::reposition(void) {
+void Slit::reposition() {
 	int head_x = 0,
 			head_y = 0,
 			head_w,
 			head_h;
-#ifdef XINERMA
+#ifdef XINERAMA
 	if (screen->hasXinerama()) {
 		unsigned int head = screen->getSlitOnHead();
 
@@ -551,9 +667,10 @@ void Slit::reposition(void) {
 }
 
 
-void Slit::shutdown(void) {
+void Slit::shutdown() {
+	saveClientList();
 	while (clientList.size() != 0)
-		removeClient(clientList.front());
+		removeClient(clientList.front(), true, true);
 }
 
 
@@ -562,7 +679,9 @@ void Slit::buttonPressEvent(XButtonEvent *e) {
 
 	if (e->button == Button1 && (! on_top)) {
 		Window w[1] = { frame.window };
-		screen->raiseWindows(w, 1);
+		Workspace::Stack st;
+		st.push_back(frame.window);
+		screen->raiseWindows(st);
 	} else if (e->button == Button2 && (! on_top)) {
 		XLowerWindow(display, frame.window);
 	} else if (e->button == Button3) {
@@ -593,9 +712,11 @@ void Slit::enterNotifyEvent(XCrossingEvent *) {
 		return;
 
 	if (hidden) {
-		if (! timer.isTiming()) timer.start();
+		if (! timer.isTiming()) 
+			timer.start();
 	} else {
-		if (timer.isTiming()) timer.stop();
+		if (timer.isTiming()) 
+			timer.stop();
 	}
 }
 
@@ -605,9 +726,11 @@ void Slit::leaveNotifyEvent(XCrossingEvent *) {
 		return;
 
 	if (hidden) {
-		if (timer.isTiming()) timer.stop();
+		if (timer.isTiming()) 
+			timer.stop();
 	} else if (! slitmenu.isVisible()) {
-		if (! timer.isTiming()) timer.start();
+		if (! timer.isTiming()) 
+			timer.start();
 	}
 }
 
@@ -651,7 +774,7 @@ void Slit::configureRequestEvent(XConfigureRequestEvent *e) {
 }
 
 
-void Slit::timeout(void) {
+void Slit::timeout() {
 	hidden = ! hidden;
 	if (hidden)
 		XMoveWindow(display, frame.window, frame.x_hidden, frame.y_hidden);
@@ -659,9 +782,43 @@ void Slit::timeout(void) {
 		XMoveWindow(display, frame.window, frame.x, frame.y);
 }
 
+void Slit::loadClientList() {
+	const std::string &filename = fluxbox->getSlitlistFilename();
+	struct stat buf;
+	if (!stat(filename.c_str(), &buf)) {
+		std::ifstream file(fluxbox->getSlitlistFilename().c_str());
+		std::string name;
+		while (! file.eof()) {
+			name = "";
+			std::getline(file, name); // get the entire line
+			if (name.size() > 0) {
+				SlitClient *client = new SlitClient(name.c_str());
+				clientList.push_back(client);
+			}
+		}
+	}
+}
+
+void Slit::saveClientList(void) {
+	const std::string &filename = fluxbox->getSlitlistFilename();
+	std::ofstream file(filename.c_str());
+	SlitClients::iterator it = clientList.begin();
+	SlitClients::iterator it_end = clientList.end();
+	std::string prevName;
+	std::string name;
+	for (; it != it_end; ++it) {
+		name = (*it)->match_name;
+		if (name != prevName)
+			file << name.c_str() << std::endl;
+
+		prevName = name;
+	}
+}
+
+
+Slitmenu::Slitmenu(Slit &sl) : Basemenu(sl.screen),
+slit(sl) {
 
-Slitmenu::Slitmenu(Slit *sl) : Basemenu(sl->screen) {
-	slit = sl;
 	I18n *i18n = I18n::instance();
 	using namespace FBNLS;
 	setLabel(i18n->getMessage(
@@ -672,7 +829,7 @@ Slitmenu::Slitmenu(Slit *sl) : Basemenu(sl->screen) {
 	directionmenu = new Directionmenu(this);
 	placementmenu = new Placementmenu(this);
 #ifdef XINERAMA
-	if (slit->screen->hasXinerama()) { // only create if we need
+	if (screen()->hasXinerama()) { // only create if we need
 		headmenu = new Headmenu(this);
 	}
 #endif // XINERAMA
@@ -688,7 +845,7 @@ Slitmenu::Slitmenu(Slit *sl) : Basemenu(sl->screen) {
 
 #ifdef XINERAMA
 	//TODO: NLS
-	if (slit->screen->hasXinerama()) {
+	if (screen()->hasXinerama()) {
 		insert(i18n->getMessage(0, 0, "Place on Head"), headmenu);
 	}
 #endif // XINERAMA
@@ -700,18 +857,19 @@ Slitmenu::Slitmenu(Slit *sl) : Basemenu(sl->screen) {
 		CommonSet, CommonAutoHide,
 		"Auto hide"), 2);
 
+	setItemSelected(2, slit.isOnTop());
+	setItemSelected(3, slit.doAutoHide());
+	
 	update();
 
-	if (slit->isOnTop()) setItemSelected(2, true);
-	if (slit->doAutoHide()) setItemSelected(3, true);
 }
 
 
-Slitmenu::~Slitmenu(void) {
+Slitmenu::~Slitmenu() {
 	delete directionmenu;
 	delete placementmenu;
 #ifdef XINERAMA
-	if (slit->screen->hasXinerama()) {
+	if (screen()->hasXinerama()) {
 		delete headmenu;
 	}
 #endif // XINERAMA
@@ -726,33 +884,37 @@ void Slitmenu::itemSelected(int button, unsigned int index) {
 		switch (item->function()) {
 		case 1: // always on top
 		{
-			bool change = ((slit->isOnTop()) ?	false : true);
-			slit->on_top = change;
+			bool change = ((slit.isOnTop()) ?	false : true);
+			slit.on_top = change;
+			screen()->saveSlitOnTop(change);
 			setItemSelected(2, change);
-
-			if (slit->isOnTop())
-				slit->screen->raiseWindows((Window *) 0, 0);
 			
+			if (slit.isOnTop())
+				screen()->raiseWindows(Workspace::Stack());
+		
 		break;
 		}
 
 		case 2: // auto hide
 		{
-			bool change = ((slit->doAutoHide()) ?	false : true);
-			slit->do_auto_hide = change;
+			bool change = (slit.doAutoHide() ?	false : true);
+			slit.do_auto_hide = change;
+			screen()->saveSlitAutoHide(change);
 			setItemSelected(3, change);
-
 		break;
 		}
 		}
+		//save the new configuration
+		Fluxbox::instance()->save_rc();
+		update();
 	}
 }
 
 
 void Slitmenu::internal_hide(void) {
 	Basemenu::internal_hide();
-	if (slit->doAutoHide())
-		slit->timeout();
+	if (slit.doAutoHide())
+		slit.timeout();
 }
 
 
@@ -760,17 +922,20 @@ void Slitmenu::reconfigure(void) {
 	directionmenu->reconfigure();
 	placementmenu->reconfigure();
 #ifdef XINERAMA
-	if (slit->screen->hasXinerama()) {
+	if (screen()->hasXinerama()) {
 		headmenu->reconfigure();
 	}
 #endif // XINERAMA
+	setItemSelected(2, slit.isOnTop());
+	setItemSelected(3, slit.doAutoHide());
 
 	Basemenu::reconfigure();
 }
 
 
-Slitmenu::Directionmenu::Directionmenu(Slitmenu *sm) : Basemenu(sm->slit->screen) {
-	slitmenu = sm;
+Slitmenu::Directionmenu::Directionmenu(Slitmenu *sm) : Basemenu(sm->screen()),
+slitmenu(sm) {
+
 	I18n *i18n = I18n::instance();
 	using namespace FBNLS;	
 	setLabel(i18n->getMessage(
@@ -789,7 +954,7 @@ Slitmenu::Directionmenu::Directionmenu(Slitmenu *sm) : Basemenu(sm->slit->screen
 
 	update();
 
-	if (sm->slit->screen->getSlitDirection() == Slit::HORIZONTAL)
+	if (screen()->getSlitDirection() == Slit::HORIZONTAL)
 		setItemSelected(0, true);
 	else
 		setItemSelected(1, true);
@@ -801,7 +966,7 @@ void Slitmenu::Directionmenu::itemSelected(int button, unsigned int index) {
 		BasemenuItem *item = find(index);
 		if (! item) return;
 
-		slitmenu->slit->screen->saveSlitDirection(item->function());
+		screen()->saveSlitDirection(item->function());
 
 		if (item->function() == Slit::HORIZONTAL) {
 			setItemSelected(0, true);
@@ -810,15 +975,16 @@ void Slitmenu::Directionmenu::itemSelected(int button, unsigned int index) {
 			setItemSelected(0, false);
 			setItemSelected(1, true);
 		}
-
-		hide();
-		slitmenu->slit->reconfigure();
+		Fluxbox::instance()->save_rc();
+		hide();		
+		slitmenu->slit.reconfigure();
 	}
 }
 
 
-Slitmenu::Placementmenu::Placementmenu(Slitmenu *sm) : Basemenu(sm->slit->screen) {
-	slitmenu = sm;
+Slitmenu::Placementmenu::Placementmenu(Slitmenu *sm) : Basemenu(sm->screen()),
+slitmenu(sm) {
+
 	I18n *i18n = I18n::instance();
 	using namespace FBNLS;	
 	setLabel(i18n->getMessage(
@@ -871,24 +1037,25 @@ void Slitmenu::Placementmenu::itemSelected(int button, unsigned int index) {
 		if (! item) return;
 
 		if (item->function()) {
-			slitmenu->slit->screen->saveSlitPlacement(item->function());
+			screen()->saveSlitPlacement(item->function());
 			hide();
-			slitmenu->slit->reconfigure();
+			slitmenu->slit.reconfigure();
+			Fluxbox::instance()->save_rc();
 		}
 	}
 }
 
 #ifdef XINERAMA
 
-Slitmenu::Headmenu::Headmenu(Slitmenu *sm)
-	: Basemenu(sm->slit->screen) {
-	slitmenu = sm;
+Slitmenu::Headmenu::Headmenu(Slitmenu *sm): Basemenu(sm->screen()),
+slitmenu(sm) {
+
 	I18n *i18n = I18n::instance();
 
 	setLabel(i18n->getMessage(0, 0, "Place on Head")); //TODO: NLS
 	setInternalMenu();
 
-	int numHeads = slitmenu->slit->screen->getNumHeads();
+	int numHeads = screen()->getNumHeads();
 	// fill menu with head entries
 	for (int i = 0; i < numHeads; i++) {
 		char headName[32];
@@ -905,12 +1072,33 @@ void Slitmenu::Headmenu::itemSelected(int button, unsigned int index) {
 		if (! item)
 			return;
 
-		slitmenu->slit->screen->saveSlitOnHead(item->function());
+		screen()->saveSlitOnHead(item->function());
 		hide();
-		slitmenu->slit->reconfigure();
+		slitmenu->slit.reconfigure();
+		Fluxbox::instance()->save_rc();
 	}
 }
 
 #endif // XINERAMA
 
+Slit::SlitClient::SlitClient(const char *name)
+{
+	initialize();
+	match_name = name;
+}
+
+Slit::SlitClient::SlitClient(BScreen *screen, Window w)
+{
+	initialize(screen, w);
+}
+
+void Slit::SlitClient::initialize(BScreen *screen, Window w) {
+	client_window = w;
+	window = icon_window = None;
+	x = y = 0;
+	width = height = 0;
+	if (match_name.size() == 0)
+		getWMName(screen, client_window, match_name);
+}
+
 #endif // SLIT
