@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Tab.cc,v 1.24 2002/04/14 22:27:57 fluxgen Exp $
+// $Id: Tab.cc,v 1.31 2002/08/11 21:12:10 fluxgen Exp $
 
 #include "Tab.hh"
 
@@ -63,7 +63,8 @@ Tab::Tab(FluxboxWindow *win, Tab *prev, Tab *next) {
 	
 	if ((m_win->getScreen()->getTabPlacement() == PLEFT ||
 			m_win->getScreen()->getTabPlacement() == PRIGHT) &&
-			m_win->getScreen()->isTabRotateVertical()) {
+			m_win->getScreen()->isTabRotateVertical() &&
+			!m_win->isShaded()) {
 		m_size_w = m_win->getScreen()->getTabHeight();
 		m_size_h = m_win->getScreen()->getTabWidth();
 	} else {
@@ -96,8 +97,8 @@ void Tab::createTabWindow() {
 	XSetWindowAttributes attrib;
 	attrib.background_pixmap = None;
 	attrib.background_pixel = attrib.border_pixel =
-		m_win->getScreen()->getWindowStyle()->tab.border_color.getPixel();
-	attrib.colormap = m_win->getScreen()->getColormap();
+		m_win->getScreen()->getWindowStyle()->tab.border_color.pixel();
+	attrib.colormap = m_win->getScreen()->colormap();
 	attrib.override_redirect = True;
 	attrib.event_mask = ButtonPressMask | ButtonReleaseMask |
 		ButtonMotionMask | ExposureMask | EnterWindowMask;
@@ -154,8 +155,11 @@ void Tab::raise() {
 	//get first tab
 	Tab *tab = 0;
 	//raise tabs
-	for (tab = getFirst(this); tab!=0; tab = tab->m_next)
-		m_win->getScreen()->raiseWindows(&tab->m_tabwin, 1);
+	Workspace::Stack st;
+	for (tab = getFirst(this); tab!=0; tab = tab->m_next) {
+		st.push_back(tab->m_tabwin);
+	}
+	m_win->getScreen()->raiseWindows(st);
 }
 
 //-------------- lower --------------------
@@ -188,13 +192,13 @@ void Tab::lower() {
 void Tab::loadTheme() {
 	BImageControl *image_ctrl = m_win->getScreen()->getImageControl();
 	Pixmap tmp = m_focus_pm;
-	BTexture *texture = &(m_win->getScreen()->getWindowStyle()->tab.l_focus);
+	FbTk::Texture *texture = &(m_win->getScreen()->getWindowStyle()->tab.l_focus);
 
-	if (texture->getTexture() & BImage::PARENTRELATIVE ) {
-		BTexture *pt = &(m_win->getScreen()->getWindowStyle()->tab.t_focus);
-		if (pt->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	if (texture->type() & FbTk::Texture::PARENTRELATIVE ) {
+		FbTk::Texture *pt = &(m_win->getScreen()->getWindowStyle()->tab.t_focus);
+		if (pt->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			m_focus_pm = None;
-			m_focus_pixel = pt->getColor()->getPixel();
+			m_focus_pixel = pt->color().pixel();
 		} else
 			m_focus_pm =
 				image_ctrl->renderImage(m_size_w, m_size_h, pt);
@@ -202,9 +206,9 @@ void Tab::loadTheme() {
 		if (tmp) image_ctrl->removeImage(tmp);
 
 	} else {
-		if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+		if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			m_focus_pm = None;
-			m_focus_pixel = texture->getColor()->getPixel();
+			m_focus_pixel = texture->color().pixel();
 		} else
 			m_focus_pm =
 				image_ctrl->renderImage(m_size_w, m_size_h, texture);
@@ -214,18 +218,18 @@ void Tab::loadTheme() {
 	tmp = m_unfocus_pm;
 	texture = &(m_win->getScreen()->getWindowStyle()->tab.l_unfocus);
 
-	if (texture->getTexture() & BImage::PARENTRELATIVE ) {
-		BTexture *pt = &(m_win->getScreen()->getWindowStyle()->tab.t_unfocus);
-		if (pt->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	if (texture->type() & FbTk::Texture::PARENTRELATIVE ) {
+		FbTk::Texture *pt = &(m_win->getScreen()->getWindowStyle()->tab.t_unfocus);
+		if (pt->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			m_unfocus_pm = None;
-			m_unfocus_pixel = pt->getColor()->getPixel();
+			m_unfocus_pixel = pt->color().pixel();
 		} else
 			m_unfocus_pm =
 			image_ctrl->renderImage(m_size_w, m_size_h, pt);
 	} else {
-		if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+		if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			m_unfocus_pm = None;
-			m_unfocus_pixel = texture->getColor()->getPixel();
+			m_unfocus_pixel = texture->color().pixel();
 		} else
 			m_unfocus_pm =
 				image_ctrl->renderImage(m_size_w, m_size_h, texture);
@@ -243,7 +247,7 @@ void Tab::decorate() {
 	XSetWindowBorderWidth(m_display, m_tabwin,
 		m_win->getScreen()->getWindowStyle()->tab.border_width);
 	XSetWindowBorder(m_display, m_tabwin,
-		m_win->getScreen()->getWindowStyle()->tab.border_color.getPixel());
+		m_win->getScreen()->getWindowStyle()->tab.border_color.pixel());
 }
 
 //-------------- deiconify -----------------
@@ -351,7 +355,7 @@ void Tab::shade() {
 // mode else it draws it in normal mode
 // TODO: the "draw in pressed mode" 
 //-----------------------------------
-void Tab::draw(bool pressed) {	
+void Tab::draw(bool pressed) const {	
 	unsigned int tabtext_w;
 
 	GC gc = ((m_win->isFocused()) ? m_win->getScreen()->getWindowStyle()->tab.l_text_focus_gc :
@@ -536,15 +540,26 @@ void Tab::setPosition() {
 	m_stoptabs = false;//thaw tablist
 }
 
+//Moves the tab to the left
+void Tab::movePrev() {
+	insert(m_prev);
+}
+
+//Moves the tab to the next tab if m_next != 0
+void Tab::moveNext() {
+	if(m_next == 0)
+		return;
+	Tab *tmp = m_next; 
+	disconnect();
+	tmp->insert(this);
+}
+
+
 //------------- calcIncrease ----------------
 // calculates m_inc_x and m_inc_y for tabs
 // used for positioning the tabs.
 //-------------------------------------------
-void Tab::calcIncrease(void) {
-	#ifdef DEBUG
-	cerr << "Calculating tab increase" << endl;
-	#endif // DEBUG
-
+void Tab::calcIncrease() {
 	Tab *tab;
 	int inc_x = 0, inc_y = 0;
 	unsigned int i = 0, tabs = numObjects();
@@ -638,11 +653,25 @@ void Tab::buttonPressEvent(XButtonEvent *be) {
 	//draw in pressed mode
 	draw(true);
 	
-	//set window to titlewindow so we can take advatage of drag function
-	be->window = m_win->frame.title;
+	//invoke root menu with auto-glueing?
+	if (be->button == 3) {
+		BScreen *screen = m_win->getScreen();
+		Rootmenu *rootmenu = screen->getRootmenu();
+		if (! rootmenu->isVisible()) {
+			Fluxbox::instance()->checkMenu();
+			screen->getRootmenu()->move(be->x_root, be->y_root-rootmenu->titleHeight());
+			rootmenu->setAutoGroupWindow(m_win->getClientWindow());
+			rootmenu->show();
+		}
+	}
+	//otherwise let the window handle the event
+	else {
+		//set window to titlewindow so we can take advantage of drag function
+		be->window = m_win->frame.title;
 	
-	//call windows buttonpress eventhandler
-	m_win->buttonPressEvent(be);
+		//call windows buttonpress eventhandler
+		m_win->buttonPressEvent(be);
+	}
 }
 
 //----------- buttonReleaseEvent ----------
@@ -1048,10 +1077,10 @@ void Tab::setTabHeight(unsigned int h) {
 // to get right size/width of tabs when
 // PLeft || PRight && isTabRotateVertical
 // ---------------------------------------
-void Tab::resizeGroup(void) {
-	#ifdef DEBUG
+void Tab::resizeGroup() {
+#ifdef DEBUG
 	cerr <<__FILE__<<"("<<__LINE__<<"): Resizing group"<<endl;
-	#endif //DEBUG
+#endif //DEBUG
 	Tab *first;
 	for (first = getFirst(this); first != 0; first = first->m_next) {
 		if ((m_win->getScreen()->getTabPlacement() == PLEFT ||
@@ -1170,3 +1199,17 @@ Tab::Alignment Tab::getTabAlignmentNum(const char *string) {
 	}
 	return ANONE;
 }
+
+//---------- addWindowToGroup ------------
+// Add a window the the tabbed group
+//----------------------------------------
+bool Tab::addWindowToGroup(FluxboxWindow *otherWindow)
+{
+	if (!otherWindow || otherWindow == m_win)
+		return false;
+	Tab *otherTab = otherWindow->getTab();
+	if (!otherTab)
+		return false;
+	insert(otherTab);
+	return true;
+}
