@@ -41,7 +41,7 @@
 //  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 //  DEALINGS IN THE SOFTWARE.
 
-// $Id: Theme.cc,v 1.19 2002/04/04 11:28:19 fluxgen Exp $
+// $Id: Theme.cc,v 1.26 2002/08/24 19:06:06 fluxgen Exp $
 
 #ifndef   _GNU_SOURCE
 #define   _GNU_SOURCE
@@ -71,6 +71,7 @@ using namespace std;
 
 Theme::Theme(Display *display, Window rootwindow, Colormap colormap, 
 	int screennum, BImageControl *ic, const char *filename, const char *rootcommand):
+m_menustyle(display),
 m_imagecontrol(ic),
 m_display(display),
 m_colormap(colormap),
@@ -81,8 +82,7 @@ m_rootcommand(rootcommand==0 ? "" : rootcommand) //we dont want to send 0-pointe
 	cerr<<__FILE__<<"("<<__LINE__<<"): Creating."<<endl;
 	#endif //DEBUG
 	//default settings	
-	m_menustyle.titlefont = new FbTk::Font(m_display, "fixed"); //create with default font
-	m_menustyle.framefont = new FbTk::Font(m_display, "fixed");
+
 	m_windowstyle.font.set = m_toolbarstyle.font.set = m_windowstyle.tab.font.set =  0;
 	
 	m_toolbarstyle.font.fontstruct = m_windowstyle.font.fontstruct = m_windowstyle.tab.font.fontstruct = 0;
@@ -104,7 +104,7 @@ m_rootcommand(rootcommand==0 ? "" : rootcommand) //we dont want to send 0-pointe
 	m_opgc = XCreateGC(m_display, rootwindow,
 		GCForeground | GCFunction | GCSubwindowMode, &gcv);
 
-	gcv.foreground = m_windowstyle.l_text_focus.getPixel();
+	gcv.foreground = m_windowstyle.l_text_focus.pixel();
 	if (m_windowstyle.font.fontstruct)
 		gcv.font = m_windowstyle.font.fontstruct->fid;
 		
@@ -112,7 +112,7 @@ m_rootcommand(rootcommand==0 ? "" : rootcommand) //we dont want to send 0-pointe
 		XCreateGC(m_display, rootwindow,
 			gc_value_mask, &gcv);
 	
-	gcv.foreground = m_windowstyle.l_text_unfocus.getPixel();
+	gcv.foreground = m_windowstyle.l_text_unfocus.pixel();
 	if (m_windowstyle.font.fontstruct)
 		gcv.font = m_windowstyle.font.fontstruct->fid;
 	m_windowstyle.l_text_unfocus_gc =
@@ -120,7 +120,7 @@ m_rootcommand(rootcommand==0 ? "" : rootcommand) //we dont want to send 0-pointe
 				gc_value_mask, &gcv);
 	
 	//---- Tab 
-	gcv.foreground = m_windowstyle.tab.l_text_focus.getPixel();
+	gcv.foreground = m_windowstyle.tab.l_text_focus.pixel();
 	if (m_windowstyle.tab.font.fontstruct)
 		gcv.font = m_windowstyle.tab.font.fontstruct->fid;
 	
@@ -128,7 +128,7 @@ m_rootcommand(rootcommand==0 ? "" : rootcommand) //we dont want to send 0-pointe
 		XCreateGC(m_display, rootwindow,
 			gc_value_mask, &gcv);
 	
-	gcv.foreground = m_windowstyle.tab.l_text_unfocus.getPixel();
+	gcv.foreground = m_windowstyle.tab.l_text_unfocus.pixel();
 	m_windowstyle.tab.l_text_unfocus_gc =
 		XCreateGC(m_display, rootwindow,
 			gc_value_mask, &gcv);
@@ -136,64 +136,64 @@ m_rootcommand(rootcommand==0 ? "" : rootcommand) //we dont want to send 0-pointe
 	//---end Tab			
 	
 				
-	gcv.foreground = m_windowstyle.b_pic_focus.getPixel();
+	gcv.foreground = m_windowstyle.b_pic_focus.pixel();
 	m_windowstyle.b_pic_focus_gc =
 		XCreateGC(m_display, rootwindow,
 				GCForeground, &gcv);
 
-	gcv.foreground = m_windowstyle.b_pic_unfocus.getPixel();
+	gcv.foreground = m_windowstyle.b_pic_unfocus.pixel();
 	m_windowstyle.b_pic_unfocus_gc =
 		XCreateGC(m_display, rootwindow,
 				GCForeground, &gcv);
 
-	gcv.foreground = m_menustyle.t_text.getPixel();
-	if (m_menustyle.titlefont->getFontStruct())
-		gcv.font = m_menustyle.titlefont->getFontStruct()->fid;
+	gcv.foreground = m_menustyle.t_text.pixel();
+	if (m_menustyle.titlefont.fontStruct())
+		gcv.font = m_menustyle.titlefont.fontStruct()->fid;
 	m_menustyle.t_text_gc =
 		XCreateGC(m_display, rootwindow,
 				gc_value_mask, &gcv);
 
-	gcv.foreground = m_menustyle.f_text.getPixel();
-	if (m_menustyle.framefont->getFontStruct())
-		gcv.font = m_menustyle.framefont->getFontStruct()->fid;
+	gcv.foreground = m_menustyle.f_text.pixel();
+	if (m_menustyle.framefont.fontStruct())
+		gcv.font = m_menustyle.framefont.fontStruct()->fid;
 
 	m_menustyle.f_text_gc =
 		XCreateGC(m_display, rootwindow,
 				gc_value_mask, &gcv);
 
-	gcv.foreground = m_menustyle.h_text.getPixel();
+	gcv.foreground = m_menustyle.h_text.pixel();
 	m_menustyle.h_text_gc =
 		XCreateGC(m_display, rootwindow,
 				gc_value_mask, &gcv);
 
-	gcv.foreground = m_menustyle.d_text.getPixel();
+	gcv.foreground = m_menustyle.d_text.pixel();
 	m_menustyle.d_text_gc =
 		XCreateGC(m_display, rootwindow,
 				gc_value_mask, &gcv);
 
-	gcv.foreground = m_menustyle.hilite.getColor()->getPixel();
+	gcv.foreground = m_menustyle.hilite.color().pixel();
 	m_menustyle.hilite_gc =
 		XCreateGC(m_display, rootwindow,
 				gc_value_mask, &gcv);
 
-	gcv.foreground = m_toolbarstyle.l_text.getPixel();
+	gcv.foreground = m_toolbarstyle.l_text.pixel();
 	if (m_toolbarstyle.font.fontstruct)
 		gcv.font = m_toolbarstyle.font.fontstruct->fid;
 	m_toolbarstyle.l_text_gc =
 		XCreateGC(m_display, rootwindow,
 				gc_value_mask, &gcv);
 
-	gcv.foreground = m_toolbarstyle.w_text.getPixel();
+	gcv.foreground = m_toolbarstyle.w_text.pixel();
 	m_toolbarstyle.w_text_gc =
 		XCreateGC(m_display, rootwindow,
 				gc_value_mask, &gcv);
 
-	gcv.foreground = m_toolbarstyle.c_text.getPixel();
+	gcv.foreground = m_toolbarstyle.c_text.pixel();
 	m_toolbarstyle.c_text_gc =
 		XCreateGC(m_display, rootwindow,
 				gc_value_mask, &gcv);
 
-	gcv.foreground = m_toolbarstyle.b_pic.getPixel();
+	gcv.foreground = m_toolbarstyle.b_pic.pixel();
 	m_toolbarstyle.b_pic_gc =
 		XCreateGC(m_display, rootwindow,
 				gc_value_mask, &gcv);
@@ -213,10 +213,6 @@ Theme::~Theme() {
 // should only be called from ~Theme
 //--------------------
 void Theme::freeMenuStyle() {
-	delete m_menustyle.framefont;
-	m_menustyle.framefont = 0;
-	delete m_menustyle.titlefont;
-	m_menustyle.titlefont = 0;
 		
 	XFreeGC(m_display, m_menustyle.t_text_gc);
 	XFreeGC(m_display, m_menustyle.f_text_gc);
@@ -290,7 +286,7 @@ void Theme::load(const char *filename){
 		m_database = XrmGetFileDatabase(DEFAULTSTYLE);
 
 	loadMenuStyle();
-	loadToolbarStyle();
+	loadToolbarStyle();	
 	loadWindowStyle();
 	loadTabStyle();
 	loadRootCommand();
@@ -300,8 +296,6 @@ void Theme::load(const char *filename){
 }
 
 void Theme::loadMenuStyle() {
-	if (m_menustyle.titlefont==0)
-		m_menustyle.titlefont = new FbTk::Font(m_display);
 
 	readDatabaseTexture("menu.title", "Menu.Title",
 		&m_menustyle.title,
@@ -381,56 +375,53 @@ void Theme::loadMenuStyle() {
 		m_menustyle.bullet_pos = Basemenu::LEFT;
 
 	//---------- font
-	m_menustyle.framefont->loadFromDatabase(m_database, "menu.frame.font", "Menu.Frame.Font");
-	m_menustyle.titlefont->loadFromDatabase(m_database, "menu.Title.font", "Menu.Title.Font");
-	/*
-	if (I18n::instance()->multibyte()) {
-		
-		readDatabaseFontSet("menu.title.font", "Menu.Title.Font",
-			&m_menustyle.titlefont.set);					
-		readDatabaseFontSet("menu.frame.font", "Menu.Frame.Font",
-			&m_menustyle.framefont.set);
-	
-		m_menustyle.titlefont.set_extents =
-				XExtentsOfFontSet(m_menustyle.titlefont.set);
-		m_menustyle.framefont.set_extents =
-			XExtentsOfFontSet(m_menustyle.framefont.set);	
-	
-	
-	} else {
-		
-		readDatabaseFont("menu.title.font", "Menu.Title.Font",
-				&m_menustyle.titlefont.fontstruct);			
-		
-		readDatabaseFont("menu.frame.font", "Menu.Frame.Font",
-				&m_menustyle.framefont.fontstruct);
-	}*/
+	m_menustyle.framefont.loadFromDatabase(m_database, "menu.frame.font", "Menu.Frame.Font");
+	m_menustyle.titlefont.loadFromDatabase(m_database, "menu.title.font", "Menu.Title.Font");
 	
 }
 
 void Theme::loadWindowStyle() {
 	
+	//read textures
+	
 	readDatabaseTexture("window.title.focus", "Window.Title.Focus",
 		&m_windowstyle.t_focus,
 		WhitePixel(m_display, m_screennum));
+	if ( (m_windowstyle.t_focus.type() & FbTk::Texture::PARENTRELATIVE) ) {
+		cerr<<"Warning: window.title.focus type is ParentRelative. Using flat solid!"<<endl;
+		m_windowstyle.t_focus.setType(FbTk::Texture::FLAT|FbTk::Texture::SOLID);
+	}
 	readDatabaseTexture("window.title.unfocus", "Window.Title.Unfocus",
 		&m_windowstyle.t_unfocus,
 		BlackPixel(m_display, m_screennum));
+	if ( (m_windowstyle.t_unfocus.type() & FbTk::Texture::PARENTRELATIVE) ) {
+		cerr<<"Warning: window.title.unfocus type is ParentRelative. Using flat solid!"<<endl;
+		m_windowstyle.t_unfocus.setType(FbTk::Texture::FLAT|FbTk::Texture::SOLID);
+	}
+	
 	readDatabaseTexture("window.label.focus", "Window.Label.Focus",
 		&m_windowstyle.l_focus,
 		WhitePixel(m_display, m_screennum));
-		
 	readDatabaseTexture("window.label.unfocus", "Window.Label.Unfocus",
 		&m_windowstyle.l_unfocus,
-		BlackPixel(m_display, m_screennum));
-
+		BlackPixel(m_display, m_screennum));	
+	
 	
 	readDatabaseTexture("window.handle.focus", "Window.Handle.Focus",
 		&m_windowstyle.h_focus,
 		WhitePixel(m_display, m_screennum));
+	if ( (m_windowstyle.h_focus.type() & FbTk::Texture::PARENTRELATIVE) ) {
+		cerr<<"Warning: window.handle.focus is ParentReleative. Using flat solid instead."<<endl;
+		m_windowstyle.h_focus.setType(FbTk::Texture::FLAT|FbTk::Texture::SOLID);
+	}
 	readDatabaseTexture("window.handle.unfocus", "Window.Handle.Unfocus",
 		&m_windowstyle.h_unfocus,
 		BlackPixel(m_display, m_screennum));
+	if ( (m_windowstyle.h_unfocus.type() & FbTk::Texture::PARENTRELATIVE) ) {
+		cerr<<"Warning: window.handle.unfocus is ParentReleative. Using flat solid instead."<<endl;
+		m_windowstyle.h_unfocus.setType(FbTk::Texture::FLAT|FbTk::Texture::SOLID);
+	}
+
 	readDatabaseTexture("window.grip.focus", "Window.Grip.Focus",
 		&m_windowstyle.g_focus,
 		WhitePixel(m_display, m_screennum));
@@ -446,6 +437,9 @@ void Theme::loadWindowStyle() {
 	readDatabaseTexture("window.button.pressed", "Window.Button.Pressed",
 		&m_windowstyle.b_pressed,
 		BlackPixel(m_display, m_screennum));
+	
+	// read colors
+	
 	readDatabaseColor("window.frame.focusColor",
 		"Window.Frame.FocusColor",
 		&m_windowstyle.f_focus,
@@ -605,6 +599,7 @@ void Theme::loadToolbarStyle() {
 	readDatabaseTexture("toolbar.clock", "Toolbar.Clock",
 		&m_toolbarstyle.clock,
 		BlackPixel(m_display, m_screennum));
+
 	readDatabaseColor("toolbar.label.textColor", "Toolbar.Label.TextColor",
 		&m_toolbarstyle.l_text,
 		WhitePixel(m_display, m_screennum));
@@ -653,7 +648,7 @@ void Theme::loadRootCommand() {
 	char *value_type;
 	
 	if (m_rootcommand.size()) {
-		#ifndef         __EMX__		
+		#ifndef  __EMX__		
 		char tmpstring[256]; //to hold m_screennum 
 		tmpstring[0]=0;
 		sprintf(tmpstring, "%d", m_screennum);
@@ -664,7 +659,7 @@ void Theme::loadRootCommand() {
 		bexec(m_rootcommand.c_str(), const_cast<char *>(displaystring.c_str()));
 
 		#else //         __EMX__
-		spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", m_rootcommand.c_str(), NULL);  
+		spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", m_rootcommand.c_str(), 0);  
 		#endif // !__EMX__     
 
 	} else if (XrmGetResource(m_database, "rootCommand", "RootCommand",
@@ -680,7 +675,7 @@ void Theme::loadRootCommand() {
 		bexec(value.addr, const_cast<char *>(displaystring.c_str()));
 		#else //	 __EMX__
 	
-		spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", value.addr, NULL);
+		spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", value.addr, 0);
 	
 		#endif // !__EMX__
 		
@@ -692,7 +687,7 @@ void Theme::loadRootCommand() {
 
 }
 
-void Theme::loadMisc(void) {
+void Theme::loadMisc() {
 	unsigned int screen_width_div2 = WidthOfScreen(ScreenOfDisplay(m_display, m_screennum)) / 2;
 	XrmValue value;
 	char *value_type=0;
@@ -732,11 +727,19 @@ void Theme::loadMisc(void) {
 
 	readDatabaseColor("borderColor", "BorderColor", &m_border_color,
 		BlackPixel(m_display, m_screennum));		
+
+	// load slit style, if it wasn't found fall back to toolbarstyle
+	if (!readDatabaseTexture("slit", "Slit",
+		&m_slit_texture, 
+		BlackPixel(m_display, m_screennum)) ) {
+		m_slit_texture = m_toolbarstyle.toolbar;
+	}
+
 }
 
 
 bool Theme::readDatabaseTexture(char *rname, char *rclass,
-	BTexture *texture,
+	FbTk::Texture *texture,
 	unsigned long default_pixel)
 {
 	XrmValue value;
@@ -747,9 +750,9 @@ bool Theme::readDatabaseTexture(char *rname, char *rclass,
 			&value))
 		m_imagecontrol->parseTexture(texture, value.addr);
 	else
-		texture->setTexture(BImage::SOLID | BImage::FLAT);
+		texture->setType(FbTk::Texture::SOLID | FbTk::Texture::FLAT);
 
-	if (texture->getTexture() & BImage::SOLID) {
+	if (texture->type() & FbTk::Texture::SOLID) {
 		int clen = strlen(rclass) + 32, nlen = strlen(rname) + 32;
 
 		char *colorclass = new char[clen], *colorname = new char[nlen];
@@ -757,7 +760,7 @@ bool Theme::readDatabaseTexture(char *rname, char *rclass,
 		sprintf(colorclass, "%s.Color", rclass);
 		sprintf(colorname,	"%s.color", rname);
 		
-		if (!readDatabaseColor(colorname, colorclass, texture->getColor(),
+		if (!readDatabaseColor(colorname, colorclass, &texture->color(),
 				default_pixel))
 			retval = false;
 
@@ -765,52 +768,51 @@ bool Theme::readDatabaseTexture(char *rname, char *rclass,
 		sprintf(colorclass, "%s.ColorTo", rclass);
 		sprintf(colorname,	"%s.colorTo", rname);
 
-		readDatabaseColor(colorname, colorclass, texture->getColorTo(),
-			default_pixel);
+		readDatabaseColor(colorname, colorclass, &texture->colorTo(), default_pixel);
 #endif // INTERLACE
 
 		delete [] colorclass;
 		delete [] colorname;
 
-		if ((! texture->getColor()->isAllocated()) ||
-				(texture->getTexture() & BImage::FLAT))
+		if ((! texture->color().isAllocated()) ||
+				(texture->type() & FbTk::Texture::FLAT))
 			return retval;
 
 		XColor xcol;
 
-		xcol.red = (unsigned int) (texture->getColor()->getRed() +
-			(texture->getColor()->getRed() >> 1));
+		xcol.red = (unsigned int) (texture->color().red() +
+			(texture->color().red() >> 1));
 		if (xcol.red >= 0xff) xcol.red = 0xffff;
 		else xcol.red *= 0xff;
-		xcol.green = (unsigned int) (texture->getColor()->getGreen() +
-			(texture->getColor()->getGreen() >> 1));
+		xcol.green = (unsigned int) (texture->color().green() +
+			(texture->color().green() >> 1));
 		if (xcol.green >= 0xff) xcol.green = 0xffff;
 		else xcol.green *= 0xff;
-		xcol.blue = (unsigned int) (texture->getColor()->getBlue() +
-			(texture->getColor()->getBlue() >> 1));
+		xcol.blue = (unsigned int) (texture->color().blue() +
+			(texture->color().blue() >> 1));
 		if (xcol.blue >= 0xff) xcol.blue = 0xffff;
 		else xcol.blue *= 0xff;
 
 		if (! XAllocColor(m_display, m_colormap, &xcol))
 			xcol.pixel = 0;
 
-		texture->getHiColor()->setPixel(xcol.pixel);
+		texture->hiColor().setPixel(xcol.pixel);
 
 		xcol.red =
-			(unsigned int) ((texture->getColor()->getRed() >> 2) +
-			(texture->getColor()->getRed() >> 1)) * 0xff;
+			(unsigned int) ((texture->color().red() >> 2) +
+			(texture->color().red() >> 1)) * 0xff;
 		xcol.green =
-			(unsigned int) ((texture->getColor()->getGreen() >> 2) +
-			(texture->getColor()->getGreen() >> 1)) * 0xff;
+			(unsigned int) ((texture->color().green() >> 2) +
+			(texture->color().green() >> 1)) * 0xff;
 		xcol.blue =
-			(unsigned int) ((texture->getColor()->getBlue() >> 2) +
-			(texture->getColor()->getBlue() >> 1)) * 0xff;
+			(unsigned int) ((texture->color().blue() >> 2) +
+			(texture->color().blue() >> 1)) * 0xff;
 
 		if (! XAllocColor(m_display, m_colormap, &xcol))
 			xcol.pixel = 0;
 
-		texture->getLoColor()->setPixel(xcol.pixel);
-	} else if (texture->getTexture() & BImage::GRADIENT) {
+		texture->loColor().setPixel(xcol.pixel);
+	} else if (texture->type() & FbTk::Texture::GRADIENT) {
 		int clen = strlen(rclass) + 10, nlen = strlen(rname) + 10;
 
 		char *colorclass = new char[clen], *colorname = new char[nlen],
@@ -822,12 +824,11 @@ bool Theme::readDatabaseTexture(char *rname, char *rclass,
 		sprintf(colortoclass, "%s.ColorTo", rclass);
 		sprintf(colortoname,	"%s.colorTo", rname);
 
-		if (!readDatabaseColor(colorname, colorclass, texture->getColor(),
+		if (!readDatabaseColor(colorname, colorclass, &texture->color(),
 				default_pixel))
 			retval = false;	//report failure in loading
 				
-		readDatabaseColor(colortoname, colortoclass, texture->getColorTo(),
-				default_pixel);
+		readDatabaseColor(colortoname, colortoclass, &texture->colorTo(), default_pixel);
 
 		delete [] colorclass;
 		delete [] colorname;
@@ -836,13 +837,13 @@ bool Theme::readDatabaseTexture(char *rname, char *rclass,
 	}
 
 	if (!retval)
-		fprintf(stderr, "Faild in readTexture\n");
+		cerr<<"Failed to load texture for: "<<rname<<endl;
 	
 	return retval;
 }
 
 
-bool Theme::readDatabaseColor(char *rname, char *rclass, BColor *color,
+bool Theme::readDatabaseColor(char *rname, char *rclass, FbTk::Color *color,
 				unsigned long default_pixel)
 {
 	XrmValue value;
@@ -914,7 +915,7 @@ void Theme::readDatabaseFont(char *rname, char *rclass, XFontStruct **font) {
 		#ifdef DEBUG
 		cerr<<__FILE__<<"("<<__LINE__<<"): Load font:"<<value.addr<<endl;
 		#endif
-		if ((*font = XLoadQueryFont(m_display, value.addr)) == NULL) {
+		if ((*font = XLoadQueryFont(m_display, value.addr)) == 0) {
 			fprintf(stderr,
 				I18n::instance()->
 				getMessage(
@@ -928,8 +929,7 @@ void Theme::readDatabaseFont(char *rname, char *rclass, XFontStruct **font) {
 		load_default = true;
 
 	if (load_default) {
-		if ((*font = XLoadQueryFont(m_display,
-				defaultFont)) == NULL) {
+		if ((*font = XLoadQueryFont(m_display, defaultFont)) == 0) {
 			fprintf(stderr,
 				I18n::instance()->
 				getMessage(
@@ -956,80 +956,80 @@ void Theme::reconfigure() {
 	XChangeGC(m_display, m_opgc,
 		GCForeground | GCFunction | GCSubwindowMode, &gcv);
 
-	gcv.foreground = m_windowstyle.l_text_focus.getPixel();
+	gcv.foreground = m_windowstyle.l_text_focus.pixel();
 	if (m_windowstyle.font.fontstruct)
 		gcv.font = m_windowstyle.font.fontstruct->fid;
 		
 	XChangeGC(m_display, m_windowstyle.l_text_focus_gc,
 		gc_value_mask, &gcv);
 
-	gcv.foreground = m_windowstyle.l_text_unfocus.getPixel();
+	gcv.foreground = m_windowstyle.l_text_unfocus.pixel();
 	XChangeGC(m_display, m_windowstyle.l_text_unfocus_gc,
 		gc_value_mask, &gcv);
 
 	//---- Tab 
-	gcv.foreground = m_windowstyle.tab.l_text_focus.getPixel();
+	gcv.foreground = m_windowstyle.tab.l_text_focus.pixel();
 	if (m_windowstyle.tab.font.fontstruct)
 		gcv.font = m_windowstyle.tab.font.fontstruct->fid;
 	
 	XChangeGC(m_display, m_windowstyle.tab.l_text_focus_gc,
 		gc_value_mask, &gcv);
 	
-	gcv.foreground = m_windowstyle.tab.l_text_unfocus.getPixel();
+	gcv.foreground = m_windowstyle.tab.l_text_unfocus.pixel();
 	XChangeGC(m_display, m_windowstyle.tab.l_text_unfocus_gc,
 		gc_value_mask, &gcv);
 	
 	//--- end tab
 		
-	gcv.foreground = m_windowstyle.b_pic_focus.getPixel();
+	gcv.foreground = m_windowstyle.b_pic_focus.pixel();
 	XChangeGC(m_display, m_windowstyle.b_pic_focus_gc,
 		GCForeground, &gcv);
 
-	gcv.foreground = m_windowstyle.b_pic_unfocus.getPixel();
+	gcv.foreground = m_windowstyle.b_pic_unfocus.pixel();
 	XChangeGC(m_display, m_windowstyle.b_pic_unfocus_gc,
 		GCForeground, &gcv);
 
-	gcv.foreground = m_menustyle.t_text.getPixel();
-	if (m_menustyle.titlefont->getFontStruct())
-		gcv.font = m_menustyle.titlefont->getFontStruct()->fid;
+	gcv.foreground = m_menustyle.t_text.pixel();
+	if (m_menustyle.titlefont.fontStruct())
+		gcv.font = m_menustyle.titlefont.fontStruct()->fid;
 	XChangeGC(m_display, m_menustyle.t_text_gc,
 		gc_value_mask, &gcv);
 
-	gcv.foreground = m_menustyle.f_text.getPixel();	
-	if (m_menustyle.framefont->getFontStruct())
-		gcv.font = m_menustyle.framefont->getFontStruct()->fid;
+	gcv.foreground = m_menustyle.f_text.pixel();	
+	if (m_menustyle.framefont.fontStruct())
+		gcv.font = m_menustyle.framefont.fontStruct()->fid;
 		
 	XChangeGC(m_display, m_menustyle.f_text_gc,
 		gc_value_mask, &gcv);
 
-	gcv.foreground = m_menustyle.h_text.getPixel();
+	gcv.foreground = m_menustyle.h_text.pixel();
 	XChangeGC(m_display, m_menustyle.h_text_gc,
 		gc_value_mask, &gcv);
 
-	gcv.foreground = m_menustyle.d_text.getPixel();
+	gcv.foreground = m_menustyle.d_text.pixel();
 	XChangeGC(m_display, m_menustyle.d_text_gc,
 		gc_value_mask, &gcv);
 
-	gcv.foreground = m_menustyle.hilite.getColor()->getPixel();
+	gcv.foreground = m_menustyle.hilite.color().pixel();
 	XChangeGC(m_display, m_menustyle.hilite_gc,
 			gc_value_mask, &gcv);
 
-	gcv.foreground = m_toolbarstyle.l_text.getPixel();
+	gcv.foreground = m_toolbarstyle.l_text.pixel();
 	if (m_toolbarstyle.font.fontstruct)
 		gcv.font = m_toolbarstyle.font.fontstruct->fid;
 		
 	XChangeGC(m_display, m_toolbarstyle.l_text_gc,
 		gc_value_mask, &gcv);
 
-	gcv.foreground = m_toolbarstyle.w_text.getPixel();
+	gcv.foreground = m_toolbarstyle.w_text.pixel();
 	XChangeGC(m_display, m_toolbarstyle.w_text_gc,
 		gc_value_mask, &gcv);
 
-	gcv.foreground = m_toolbarstyle.c_text.getPixel();
+	gcv.foreground = m_toolbarstyle.c_text.pixel();
 	XChangeGC(m_display, m_toolbarstyle.c_text_gc,
 			gc_value_mask, &gcv);
 
-	gcv.foreground = m_toolbarstyle.b_pic.getPixel();
+	gcv.foreground = m_toolbarstyle.b_pic.pixel();
 	XChangeGC(m_display, m_toolbarstyle.b_pic_gc,
 			gc_value_mask, &gcv);
 			
@@ -1065,9 +1065,9 @@ XFontSet Theme::createFontSet(char *fontname) {
 	}
 
 	getFontElement(fontname, weight, FONT_ELEMENT_SIZE,
-		 "-medium-", "-bold-", "-demibold-", "-regular-", NULL);
+		 "-medium-", "-bold-", "-demibold-", "-regular-", 0);
 	getFontElement(fontname, slant, FONT_ELEMENT_SIZE,
-		 "-r-", "-i-", "-o-", "-ri-", "-ro-", NULL);
+		 "-r-", "-i-", "-o-", "-ri-", "-ro-", 0);
 	getFontSize(fontname, &pixel_size);
 
 	if (! strcmp(weight, "*")) 
@@ -1107,22 +1107,22 @@ const char *Theme::getFontSize(const char *pattern, int *size) {
 
 	for (p=pattern; 1; p++) {
 		if (!*p) {
-			if (p2!=NULL && n>1 && n<72) {
+			if (p2!=0 && n>1 && n<72) {
 				*size = n; return p2+1;
 			} else {
-				*size = 16; return NULL;
+				*size = 16; return 0;
 			}
 		} else if (*p=='-') {
-			if (n>1 && n<72 && p2!=NULL) {
+			if (n>1 && n<72 && p2!=0) {
 				*size = n;
 				return p2+1;
 			}
 			p2=p; n=0;
-		} else if (*p>='0' && *p<='9' && p2!=NULL) {
+		} else if (*p>='0' && *p<='9' && p2!=0) {
 			n *= 10;
 			n += *p-'0';
 		} else {
-			p2=NULL; n=0;
+			p2=0; n=0;
 		}
 	}
 }
@@ -1135,7 +1135,7 @@ const char *Theme::getFontElement(const char *pattern, char *buf, int bufsiz, ..
 	va_start(va, bufsiz);
 	buf[bufsiz-1] = 0;
 	buf[bufsiz-2] = '*';
-	while((v = va_arg(va, char *)) != NULL) {
+	while((v = va_arg(va, char *)) != 0) {
 		p = StringUtil::strcasestr(pattern, v);
 		if (p) {
 			strncpy(buf, p+1, bufsiz-2);
@@ -1147,5 +1147,5 @@ const char *Theme::getFontElement(const char *pattern, char *buf, int bufsiz, ..
 	}
 	va_end(va);
 	strncpy(buf, "*", bufsiz);
-	return NULL;
+	return 0;
 }
