@@ -19,18 +19,20 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
+#include "Timer.hh"
+
 //use GNU extensions
-#ifndef	 _GNU_SOURCE
+#ifndef	_GNU_SOURCE
 #define _GNU_SOURCE
 #endif // _GNU_SOURCE
 
-#ifdef		HAVE_CONFIG_H
+#ifdef	HAVE_CONFIG_H
 #include "../config.h"
 #endif // HAVE_CONFIG_H
 
-#include "Timer.hh"
 #include <sys/types.h>
 #include <unistd.h>
+#include <cassert>
 
 //static var
 BTimer::TimerList BTimer::m_timerlist;
@@ -42,7 +44,7 @@ m_once(false) {
 }
 
 
-BTimer::~BTimer(void) {
+BTimer::~BTimer() {
 	if (isTiming()) stop();
 }
 
@@ -61,7 +63,7 @@ void BTimer::setTimeout(timeval t) {
 }
 
 
-void BTimer::start(void) {
+void BTimer::start() {
 	gettimeofday(&m_start, 0);
 
 	if (! m_timing) {
@@ -71,13 +73,13 @@ void BTimer::start(void) {
 }
 
 
-void BTimer::stop(void) {
+void BTimer::stop() {
 	m_timing = false;
 	removeTimer(this); //remove us from the list
 }
 
 
-void BTimer::fireTimeout(void) {
+void BTimer::fireTimeout() {
 	if (m_handler) m_handler->timeout();
 }
 
@@ -127,22 +129,24 @@ void BTimer::updateTimers(int fd) {
 	//must check end ...the timer might remove
 	//it self from the list (should be fixed in the future)
 	for(; it != m_timerlist.end(); ++it) {
-		tm.tv_sec = (*it)->getStartTime().tv_sec +
-		(*it)->getTimeout().tv_sec;
-		tm.tv_usec = (*it)->getStartTime().tv_usec +
-		(*it)->getTimeout().tv_usec;
+		//This is to make sure we don't get an invalid iterator
+		//when we do fireTimeout
+		BTimer &t = *(*it);
+		tm.tv_sec = t.getStartTime().tv_sec +
+		t.getTimeout().tv_sec;
+		tm.tv_usec = t.getStartTime().tv_usec +
+		t.getTimeout().tv_usec;
 
 		if ((now.tv_sec < tm.tv_sec) ||
 			(now.tv_sec == tm.tv_sec && now.tv_usec < tm.tv_usec))
 			break;
 
-		(*it)->fireTimeout();
-
+		t.fireTimeout();
 		// restart the current timer so that the start time is updated
-		if (! (*it)->doOnce())
-			(*it)->start();
-		else { 
-			(*it)->stop();			
+		if (! t.doOnce())
+			t.start();
+		else {
+			t.stop();			
 			it--;
 		}					
 	}
