@@ -22,10 +22,21 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Toolbar.cc,v 1.21 2002/04/09 23:16:28 fluxgen Exp $
+// $Id: Toolbar.cc,v 1.29 2002/08/24 19:31:24 fluxgen Exp $
 
-// stupid macros needed to access some functions in version 2 of the GNU C
-// library
+#include "Toolbar.hh"
+
+#include "i18n.hh"
+#include "fluxbox.hh"
+#include "Clientmenu.hh"
+#include "Icon.hh"
+#include "Rootmenu.hh"
+#include "Screen.hh"
+#include "Window.hh"
+#include "Workspace.hh"
+#include "Workspacemenu.hh"
+
+// use GNU extensions
 #ifndef	 _GNU_SOURCE
 #define	 _GNU_SOURCE
 #endif // _GNU_SOURCE
@@ -34,37 +45,22 @@
 #	include "../config.h"
 #endif // HAVE_CONFIG_H
 
-#include "i18n.hh"
-#include "fluxbox.hh"
-#include "Clientmenu.hh"
-#include "Icon.hh"
-#include "Rootmenu.hh"
-#include "Screen.hh"
-#include "Toolbar.hh"
-#include "Window.hh"
-#include "Workspace.hh"
-#include "Workspacemenu.hh"
-
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
-#ifdef		STDC_HEADERS
-#	include <string.h>
-#endif // STDC_HEADERS
 
-#ifdef		HAVE_STDIO_H
-#	include <stdio.h>
-#endif // HAVE_STDIO_H
+#include <cstring>
+#include <cstdio>
 
-#ifdef		TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <sys/time.h>
+#include <time.h>
 #else // !TIME_WITH_SYS_TIME
-# ifdef		HAVE_SYS_TIME_H
-#	include <sys/time.h>
-# else // !HAVE_SYS_TIME_H
-#	include <time.h>
-# endif // HAVE_SYS_TIME_H
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#else // !HAVE_SYS_TIME_H
+#include <time.h>
+#endif // HAVE_SYS_TIME_H
 #endif // TIME_WITH_SYS_TIME
 
 #include <iostream>
@@ -72,29 +68,29 @@
 using namespace std;
 
 Toolbar::Toolbar(BScreen *scrn):
+on_top(scrn->isToolbarOnTop()),
+editing(false),
+hidden(scrn->doToolbarAutoHide()), 
+do_auto_hide(scrn->doToolbarAutoHide()),
 screen(scrn),
-image_ctrl(screen->getImageControl()),
+image_ctrl(scrn->getImageControl()),
 clock_timer(this), 	// get the clock updating every minute
-iconbar(0)
-{
+iconbar(0) {
 
-	fluxbox = Fluxbox::instance();	
+	
+	
+	fluxbox = Fluxbox::instance();
 
-
-	timeval now;
-	gettimeofday(&now, 0);
-	clock_timer.setTimeout((60 - (now.tv_sec % 60)) * 1000);
+	timeval delay;
+	delay.tv_sec = 1;
+	delay.tv_usec = 0;
+	clock_timer.setTimeout(delay);
 	clock_timer.start();
 
 	hide_handler.toolbar = this;
 	hide_timer = new BTimer(&hide_handler);
 	hide_timer->setTimeout(fluxbox->getAutoRaiseDelay());
-	hide_timer->fireOnce(True);
-
-
-	editing = False;
-	on_top = screen->isToolbarOnTop();
-	hidden = do_auto_hide = screen->doToolbarAutoHide();
+	hide_timer->fireOnce(true);
 
 	frame.grab_x = frame.grab_y = 0;
 
@@ -107,9 +103,9 @@ iconbar(0)
 		
 	attrib.background_pixmap = None;
 	attrib.background_pixel = attrib.border_pixel =
-		screen->getBorderColor()->getPixel();
-	attrib.colormap = screen->getColormap();
-	attrib.override_redirect = True;
+		screen->getBorderColor()->pixel();
+	attrib.colormap = screen->colormap();
+	attrib.override_redirect = true;
 	attrib.event_mask = ButtonPressMask | ButtonReleaseMask |
 		EnterWindowMask | LeaveWindowMask;
 
@@ -174,7 +170,7 @@ iconbar(0)
 }
 
 
-Toolbar::~Toolbar(void) {
+Toolbar::~Toolbar() {
 	XUnmapWindow(display, frame.window);
 	 
 	if (frame.base) image_ctrl->removeImage(frame.base);
@@ -217,7 +213,7 @@ void Toolbar::delIcon(FluxboxWindow *w) {
 		Fluxbox::instance()->removeToolbarSearch(iconbar->delIcon(w));
 }
 		
-void Toolbar::reconfigure(void) {
+void Toolbar::reconfigure() {
 	int head_x = 0,
 			head_y = 0,
 			head_w,
@@ -430,11 +426,11 @@ void Toolbar::reconfigure(void) {
 		frame.label_h);
 
 	Pixmap tmp = frame.base;
-	BTexture *texture = &(screen->getToolbarStyle()->toolbar);
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	FbTk::Texture *texture = &(screen->getToolbarStyle()->toolbar);
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 		frame.base = None;
 		XSetWindowBackground(display, frame.window,
-			 texture->getColor()->getPixel());
+			 texture->color().pixel());
 	} else {
 		frame.base =
 			image_ctrl->renderImage(frame.width, frame.height, texture);
@@ -444,10 +440,10 @@ void Toolbar::reconfigure(void) {
 
 	tmp = frame.label;
 	texture = &(screen->getToolbarStyle()->window);
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 		frame.label = None;
 		XSetWindowBackground(display, frame.window_label,
-			 texture->getColor()->getPixel());
+			 texture->color().pixel());
 	} else {
 		frame.label =
 			image_ctrl->renderImage(frame.window_label_w, frame.label_h, texture);
@@ -457,10 +453,10 @@ void Toolbar::reconfigure(void) {
 
 	tmp = frame.wlabel;
 	texture = &(screen->getToolbarStyle()->label);
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 		frame.wlabel = None;
 		XSetWindowBackground(display, frame.workspace_label,
-			 texture->getColor()->getPixel());
+			 texture->color().pixel());
 	} else {
 		frame.wlabel =
 			image_ctrl->renderImage(frame.workspace_label_w, frame.label_h, texture);
@@ -470,10 +466,10 @@ void Toolbar::reconfigure(void) {
 
 	tmp = frame.clk;
 	texture = &(screen->getToolbarStyle()->clock);
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 		frame.clk = None;
 		XSetWindowBackground(display, frame.clock,
-			 texture->getColor()->getPixel());
+			 texture->color().pixel());
 	} else {
 		frame.clk =
 			image_ctrl->renderImage(frame.clock_w, frame.label_h, texture);
@@ -483,10 +479,10 @@ void Toolbar::reconfigure(void) {
 
 	tmp = frame.button;
 	texture = &(screen->getToolbarStyle()->button);
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 		frame.button = None;
 
-		frame.button_pixel = texture->getColor()->getPixel();
+		frame.button_pixel = texture->color().pixel();
 		XSetWindowBackground(display, frame.psbutton, frame.button_pixel);
 		XSetWindowBackground(display, frame.nsbutton, frame.button_pixel);
 		XSetWindowBackground(display, frame.pwbutton, frame.button_pixel);
@@ -504,16 +500,16 @@ void Toolbar::reconfigure(void) {
 
 	tmp = frame.pbutton;
 	texture = &(screen->getToolbarStyle()->pressed);
-	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
+	if (texture->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 		frame.pbutton = None;
-		frame.pbutton_pixel = texture->getColor()->getPixel();
+		frame.pbutton_pixel = texture->color().pixel();
 	} else
 		frame.pbutton =
 			image_ctrl->renderImage(frame.button_w, frame.button_w, texture);
 	if (tmp) image_ctrl->removeImage(tmp);
 
 	XSetWindowBorder(display, frame.window,
-			 screen->getBorderColor()->getPixel());
+			 screen->getBorderColor()->pixel());
 	XSetWindowBorderWidth(display, frame.window, screen->getBorderWidth());
 
 	XClearWindow(display, frame.window);
@@ -531,7 +527,7 @@ void Toolbar::reconfigure(void) {
 	redrawNextWorkspaceButton();
 	redrawPrevWindowButton();
 	redrawNextWindowButton();
-	checkClock(True);
+	checkClock(true);
 
 	toolbarmenu->reconfigure();
 
@@ -540,7 +536,7 @@ void Toolbar::reconfigure(void) {
 		if (!iconbar) {
 			iconbar = new IconBar(screen, frame.window_label);
 			if (screen->getIconCount()) {
-                BScreen::Icons & l = screen->getIconList();
+				BScreen::Icons & l = screen->getIconList();
 				BScreen::Icons::iterator it = l.begin();
 				BScreen::Icons::iterator it_end = l.end();
 				for(; it != it_end; ++it) {
@@ -566,9 +562,9 @@ void Toolbar::reconfigure(void) {
 
 
 #ifdef		HAVE_STRFTIME
-void Toolbar::checkClock(Bool redraw) {
+void Toolbar::checkClock(bool redraw) {
 #else // !HAVE_STRFTIME
-void Toolbar::checkClock(Bool redraw, Bool date) {
+void Toolbar::checkClock(bool redraw, bool date) {
 #endif // HAVE_STRFTIME
 	time_t tmp = 0;
 	struct tm *tt = 0;
@@ -582,13 +578,14 @@ void Toolbar::checkClock(Bool redraw, Bool date) {
 			frame.hour = tt->tm_hour;
 			frame.minute = tt->tm_min;
 			XClearWindow(display, frame.clock);
-			redraw = True;
+			redraw = true;
 		}
 	} else
 		cerr<<__FILE__<<"("<<__LINE__<<"): time(null)<0"<<endl;
 	
 
 	if (redraw) {
+		XClearWindow(display, frame.clock);
 		#ifdef HAVE_STRFTIME
 		char t[1024];
 		if (! strftime(t, 1024, screen->getStrftimeFormat(), tt))
@@ -692,14 +689,13 @@ void Toolbar::checkClock(Bool redraw, Bool date) {
 		}
 
 		switch (screen->getToolbarStyle()->font.justify) {
-		case DrawUtil::Font::RIGHT:
-			dx += frame.clock_w - l;
+			case DrawUtil::Font::RIGHT:
+				dx += frame.clock_w - l;
 			break;
-
-		case DrawUtil::Font::CENTER:
-			dx += (frame.clock_w - l) / 2;
+			case DrawUtil::Font::CENTER:
+				dx += (frame.clock_w - l) / 2;
 			break;
-		default: //LEFT
+			default: //LEFT, no justification
 			break;
 		}
 
@@ -718,15 +714,15 @@ void Toolbar::checkClock(Bool redraw, Bool date) {
 }
 
 
-void Toolbar::redrawWindowLabel(Bool redraw) {
+void Toolbar::redrawWindowLabel(bool redraw) {
 	if (Fluxbox::instance()->getFocusedWindow()) {
 		if (redraw)
 			XClearWindow(display, frame.window_label);
 
 		FluxboxWindow *foc = Fluxbox::instance()->getFocusedWindow();
-		if (foc->getScreen() != screen)
+		if (foc->getScreen() != screen || foc->getTitle().size() == 0)
 			return;
-
+		
 		int dx = (frame.bevel_w * 2), dlen = foc->getTitle().size();
 		unsigned int l;
 		I18n *i18n = I18n::instance();
@@ -785,7 +781,7 @@ void Toolbar::redrawWindowLabel(Bool redraw) {
 }
  
  
-void Toolbar::redrawWorkspaceLabel(Bool redraw) {
+void Toolbar::redrawWorkspaceLabel(bool redraw) {
 	if (screen->getCurrentWorkspace()->name().size()>0) {
 		
 		if (redraw)
@@ -863,7 +859,7 @@ void Toolbar::redrawWorkspaceLabel(Bool redraw) {
 }
 
 
-void Toolbar::redrawPrevWorkspaceButton(Bool pressed, Bool redraw) {
+void Toolbar::redrawPrevWorkspaceButton(bool pressed, bool redraw) {
 	if (redraw) {
 		if (pressed) {
 			if (frame.pbutton)
@@ -891,7 +887,7 @@ void Toolbar::redrawPrevWorkspaceButton(Bool pressed, Bool redraw) {
 }
 
 
-void Toolbar::redrawNextWorkspaceButton(Bool pressed, Bool redraw) {
+void Toolbar::redrawNextWorkspaceButton(bool pressed, bool redraw) {
 	if (redraw) {
 		if (pressed) {
 			if (frame.pbutton)
@@ -919,7 +915,7 @@ void Toolbar::redrawNextWorkspaceButton(Bool pressed, Bool redraw) {
 }
 
 
-void Toolbar::redrawPrevWindowButton(Bool pressed, Bool redraw) {
+void Toolbar::redrawPrevWindowButton(bool pressed, bool redraw) {
 	if (redraw) {
 		if (pressed) {
 			if (frame.pbutton)
@@ -947,7 +943,7 @@ void Toolbar::redrawPrevWindowButton(Bool pressed, Bool redraw) {
 }
 
 
-void Toolbar::redrawNextWindowButton(Bool pressed, Bool redraw) {
+void Toolbar::redrawNextWindowButton(bool pressed, bool redraw) {
 	if (redraw) {
 		if (pressed) {
 			if (frame.pbutton)
@@ -975,11 +971,11 @@ void Toolbar::redrawNextWindowButton(Bool pressed, Bool redraw) {
 }
 
 
-void Toolbar::edit(void) {
+void Toolbar::edit() {
 	Window window;
 	int foo;
 
-	editing = True;	//mark for editing
+	editing = true;	//mark for editing
 	
 	//workspace labe already got intput focus ?
 	if (XGetInputFocus(display, &window, &foo) &&
@@ -993,9 +989,9 @@ void Toolbar::edit(void) {
 
 	XClearWindow(display, frame.workspace_label);	//clear workspace text
 
-	fluxbox->setNoFocus(True);
+	fluxbox->setNoFocus(true);
 	if (fluxbox->getFocusedWindow())	//disable focus on current focused window
-		fluxbox->getFocusedWindow()->setFocusFlag(False);
+		fluxbox->getFocusedWindow()->setFocusFlag(false);
 
 	XDrawRectangle(display, frame.workspace_label,
 		screen->getWindowStyle()->l_text_focus_gc,
@@ -1008,13 +1004,13 @@ void Toolbar::buttonPressEvent(XButtonEvent *be) {
 	FluxboxWindow *fluxboxwin=0;
 	if (be->button == 1) {
 		if (be->window == frame.psbutton)
-			redrawPrevWorkspaceButton(True, True);
+			redrawPrevWorkspaceButton(true, true);
 		else if (be->window == frame.nsbutton)
-			redrawNextWorkspaceButton(True, True);
+			redrawNextWorkspaceButton(true, true);
 		else if (be->window == frame.pwbutton)
-			redrawPrevWindowButton(True, True);
+			redrawPrevWindowButton(true, true);
 		else if (be->window == frame.nwbutton)
-			redrawNextWindowButton(True, True);
+			redrawNextWindowButton(true, true);
 		else if ( iconbar ) {
 			if ( (fluxboxwin = iconbar->findWindow(be->window)) )
 				fluxboxwin->deiconify();
@@ -1022,17 +1018,22 @@ void Toolbar::buttonPressEvent(XButtonEvent *be) {
 #ifndef	 HAVE_STRFTIME
 		else if (be->window == frame.clock) {
 			XClearWindow(display, frame.clock);
-			checkClock(True, True);
+			checkClock(true, true);
 		}
 #endif // HAVE_STRFTIME
 		else if (! on_top) {
-			Window w[1] = { frame.window };
-			screen->raiseWindows(w, 1);
+			Workspace::Stack st;
+			st.push_back(frame.window);
+			screen->raiseWindows(st);
 		}
 	} else if (be->button == 2 && (! on_top)) {
 		XLowerWindow(display, frame.window);
 	} else if (be->button == 3) {
-		if (! toolbarmenu->isVisible()) {
+		FluxboxWindow *fluxboxwin = 0;
+		if ( iconbar && (fluxboxwin = iconbar->findWindow(be->window)) ) {
+			Windowmenu *wm = fluxboxwin->getWindowmenu();
+			fluxboxwin->showMenu(be->x_root, be->y_root - wm->height());
+		} else if (! toolbarmenu->isVisible()) {
 			int x, y;
 
 			x = be->x_root - (toolbarmenu->width() / 2);
@@ -1061,25 +1062,25 @@ void Toolbar::buttonPressEvent(XButtonEvent *be) {
 void Toolbar::buttonReleaseEvent(XButtonEvent *re) {
 	if (re->button == 1) {
 		if (re->window == frame.psbutton) {
-			redrawPrevWorkspaceButton(False, True);
+			redrawPrevWorkspaceButton(false, true);
 
 			if (re->x >= 0 && re->x < (signed) frame.button_w &&
 					re->y >= 0 && re->y < (signed) frame.button_w)
 			 screen->prevWorkspace(1);
 		} else if (re->window == frame.nsbutton) {
-			redrawNextWorkspaceButton(False, True);
+			redrawNextWorkspaceButton(false, true);
 
 			if (re->x >= 0 && re->x < (signed) frame.button_w &&
 					re->y >= 0 && re->y < (signed) frame.button_w)
 				screen->nextWorkspace(1);
 		} else if (re->window == frame.pwbutton) {
-			redrawPrevWindowButton(False, True);
+			redrawPrevWindowButton(false, true);
 
 			if (re->x >= 0 && re->x < (signed) frame.button_w &&
 					re->y >= 0 && re->y < (signed) frame.button_w)
 				screen->prevFocus();
 		} else if (re->window == frame.nwbutton) {
-			redrawNextWindowButton(False, True);
+			redrawNextWindowButton(false, true);
 
 			if (re->x >= 0 && re->x < (signed) frame.button_w &&
 					re->y >= 0 && re->y < (signed) frame.button_w)
@@ -1089,7 +1090,7 @@ void Toolbar::buttonReleaseEvent(XButtonEvent *re) {
 #ifndef	 HAVE_STRFTIME
 		else if (re->window == frame.clock) {
 			XClearWindow(display, frame.clock);
-			checkClock(True);
+			checkClock(true);
 		}
 #endif // HAVE_STRFTIME
 	} else if (re->button == 4) //mousewheel scroll up
@@ -1123,7 +1124,8 @@ void Toolbar::leaveNotifyEvent(XCrossingEvent *) {
 
 
 void Toolbar::exposeEvent(XExposeEvent *ee) {
-	if (ee->window == frame.clock) checkClock(True);
+	if (ee->window == frame.clock) 
+		checkClock(true);
 	else if (ee->window == frame.workspace_label && (! editing))
 		redrawWorkspaceLabel();
 	else if (ee->window == frame.window_label) redrawWindowLabel();
@@ -1146,12 +1148,12 @@ void Toolbar::keyPressEvent(XKeyEvent *ke) {
 		if (ks == XK_Return || ks == XK_Escape) {
 			
 
-			editing = False;
+			editing = false;
 			
-			fluxbox->setNoFocus(False);
+			fluxbox->setNoFocus(false);
 			if (fluxbox->getFocusedWindow()) {
 				fluxbox->getFocusedWindow()->setInputFocus();
-				fluxbox->getFocusedWindow()->setFocusFlag(True);
+				fluxbox->getFocusedWindow()->setFocusFlag(true);
 			} else
 				XSetInputFocus(display, PointerRoot, None, CurrentTime);
 			
@@ -1165,18 +1167,15 @@ void Toolbar::keyPressEvent(XKeyEvent *ke) {
 				remove(screen->getCurrentWorkspace()->workspaceID() + 2);
 			screen->getWorkspacemenu()->
 				insert(screen->getCurrentWorkspace()->name().c_str(),
-			screen->getCurrentWorkspace()->menu(),
-			screen->getCurrentWorkspace()->workspaceID() + 1);
+					screen->getCurrentWorkspace()->menu(),
+					screen->getCurrentWorkspace()->workspaceID() + 2);
 			screen->getWorkspacemenu()->update();
 
 			reconfigure();
-		} else if (! (ks == XK_Shift_L || ks == XK_Shift_R ||
-			ks == XK_Control_L || ks == XK_Control_R ||
-			ks == XK_Caps_Lock || ks == XK_Shift_Lock ||
-			ks == XK_Meta_L || ks == XK_Meta_R ||
-			ks == XK_Alt_L || ks == XK_Alt_R ||
-			ks == XK_Super_L || ks == XK_Super_R ||
-			ks == XK_Hyper_L || ks == XK_Hyper_R)) {
+			//save workspace names
+			Fluxbox::instance()->save_rc();
+
+		} else if (! IsModifierKey(ks) && !IsCursorKey(ks)) {
 
 			if (ks == XK_BackSpace && new_workspace_name.size())
 				new_workspace_name.erase(new_workspace_name.size()-1);
@@ -1193,26 +1192,26 @@ void Toolbar::keyPressEvent(XKeyEvent *ke) {
 				XmbTextExtents(screen->getToolbarStyle()->font.set,
 					new_workspace_name.c_str(), l, &ink, &logical);
 				tw = logical.width;
-			} else
+			} else {
 				tw = XTextWidth(screen->getToolbarStyle()->font.fontstruct,
 					new_workspace_name.c_str(), l);
-			
+			}
 				x = (frame.workspace_label_w - tw) / 2;
 
 			if (x < (signed) frame.bevel_w) x = frame.bevel_w;
 
-			if (i18n->multibyte())
+			if (i18n->multibyte()) {
 				XmbDrawString(display, frame.workspace_label,
 					screen->getWindowStyle()->font.set,
 					screen->getWindowStyle()->l_text_focus_gc, x, 1 -
 					screen->getWindowStyle()->font.set_extents->max_ink_extent.y,
 					new_workspace_name.c_str(), l);
-			else
+			} else {
 				XDrawString(display, frame.workspace_label,
 					screen->getWindowStyle()->l_text_focus_gc, x,
 					screen->getToolbarStyle()->font.fontstruct->ascent + 1,
 					new_workspace_name.c_str(), l);
-			
+			}
 			XDrawRectangle(display, frame.workspace_label,
 				screen->getWindowStyle()->l_text_focus_gc, x + tw, 0, 1,
 				frame.label_h - 1);
@@ -1222,16 +1221,17 @@ void Toolbar::keyPressEvent(XKeyEvent *ke) {
 }
 
 
-void Toolbar::timeout(void) {
-	checkClock(True);
+void Toolbar::timeout() {
+	checkClock(true);
 
-	timeval now;
-	gettimeofday(&now, 0);
-	clock_timer.setTimeout((60 - (now.tv_sec % 60)) * 1000);
+	timeval delay;
+	delay.tv_sec = 1;
+	delay.tv_usec = 0;	
+	clock_timer.setTimeout(delay);
 }
 
 
-void Toolbar::HideHandler::timeout(void) {
+void Toolbar::HideHandler::timeout() {
 	toolbar->hidden = ! toolbar->hidden;
 	if (toolbar->hidden)
 		XMoveWindow(toolbar->display, toolbar->frame.window,
@@ -1291,7 +1291,7 @@ Toolbarmenu::Toolbarmenu(Toolbar *tb) : Basemenu(tb->screen) {
 }
 
 
-Toolbarmenu::~Toolbarmenu(void) {
+Toolbarmenu::~Toolbarmenu() {
 	delete placementmenu;
 #ifdef XINERAMA
 	if (toolbar->screen->hasXinerama()) {
@@ -1305,30 +1305,35 @@ Toolbarmenu::~Toolbarmenu(void) {
 void Toolbarmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
 		BasemenuItem *item = find(index);
-		if (! item) return;
+		if (item == 0)
+			return;
 
 		switch (item->function()) {
-		case 1: // always on top
-			{
-			bool change = ((toolbar->isOnTop()) ? False : True);
+		case 1:  {// always on top
+			bool change = ((toolbar->isOnTop()) ? false : true);
 			toolbar->on_top = change;
-				setItemSelected(1, change);
-
-			if (toolbar->isOnTop()) toolbar->screen->raiseWindows((Window *) 0, 0);
+			screen()->saveToolbarOnTop(toolbar->on_top);
+			setItemSelected(1, change);
+			
+			if (toolbar->isOnTop())
+				toolbar->screen->raiseWindows(Workspace::Stack());
+			
+			Fluxbox::instance()->save_rc();
 			break;
-			}
+		}
 
-		case 2: // auto hide
-			{
-	bool change = ((toolbar->doAutoHide()) ?	False : True);
-	toolbar->do_auto_hide = change;
-	setItemSelected(2, change);
+		case 2: { // auto hide
+			bool change = ((toolbar->doAutoHide()) ? false : true);
+			toolbar->do_auto_hide = change;
+			screen()->saveToolbarAutoHide(toolbar->do_auto_hide);
+			setItemSelected(2, change);
 
-#ifdef		SLIT
-	toolbar->screen->getSlit()->reposition();
+#ifdef SLIT
+			toolbar->screen->getSlit()->reposition();
 #endif // SLIT
-	break;
-			}
+			Fluxbox::instance()->save_rc();
+		break;
+		}
 
 		case 3: // edit current workspace name
 			toolbar->edit();	//set edit mode
@@ -1340,14 +1345,14 @@ void Toolbarmenu::itemSelected(int button, unsigned int index) {
 }
 
 
-void Toolbarmenu::internal_hide(void) {
+void Toolbarmenu::internal_hide() {
 	Basemenu::internal_hide();
 	if (toolbar->doAutoHide() && ! toolbar->isEditing())
 		toolbar->hide_handler.timeout();
 }
 
 
-void Toolbarmenu::reconfigure(void) {
+void Toolbarmenu::reconfigure() {
 	placementmenu->reconfigure();
 #ifdef XINERAMA
 	if (toolbar->screen->hasXinerama()) {
