@@ -22,16 +22,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Workspace.cc,v 1.18 2002/04/09 23:18:12 fluxgen Exp $
+// $Id: Workspace.cc,v 1.26 2002/08/31 10:40:50 fluxgen Exp $
 
-// use GNU extensions
-#ifndef	 _GNU_SOURCE
-#define	 _GNU_SOURCE
-#endif // _GNU_SOURCE
-
-#ifdef		HAVE_CONFIG_H
-#	include "../config.h"
-#endif // HAVE_CONFIG_H
+#include "Workspace.hh"
 
 #include "i18n.hh"
 #include "fluxbox.hh"
@@ -39,50 +32,67 @@
 #include "Screen.hh"
 #include "Toolbar.hh"
 #include "Window.hh"
-#include "Workspace.hh"
 #include "Windowmenu.hh"
 #include "StringUtil.hh"
 
-#ifdef		HAVE_STDIO_H
-#	include <stdio.h>
-#endif // HAVE_STDIO_H
+// use GNU extensions
+#ifndef	 _GNU_SOURCE
+#define	 _GNU_SOURCE
+#endif // _GNU_SOURCE
 
-#ifdef		STDC_HEADERS
-#	include <string.h>
-#endif // STDC_HEADERS
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif // HAVE_CONFIG_H
 
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
+#include <cstdio>
+#include <cstring>
+
 #include <algorithm>
 #include <iostream>
+
 using namespace std;
 
+namespace { // anonymous
+
+int countTransients(const FluxboxWindow &win) {
+  if (win.getTransients().size() == 0)
+	  return 0;
+  // now go throu the entire tree and count transients
+  size_t ret = win.getTransients().size();    
+  std::list<FluxboxWindow *>::const_iterator it = win.getTransients().begin();
+  std::list<FluxboxWindow *>::const_iterator it_end = win.getTransients().end();
+  for (; it != it_end; ++it)
+	  ret += countTransients(*(*it));
+
+  return ret;
+}
+
+};
+
+Workspace::GroupList Workspace::m_groups;
+
 Workspace::Workspace(BScreen *scrn, unsigned int i):
 screen(scrn),
 lastfocus(0),
+m_clientmenu(this),
 m_name(""),
 m_id(i),
-cascade_x(32), cascade_y(32)
-{
+cascade_x(32), cascade_y(32) {
 
-	clientmenu = new Clientmenu(this);
+	setName(screen->getNameOfWorkspace(m_id));
 
-	char *tmp;
-	screen->getNameOfWorkspace(m_id, &tmp);
-	setName(tmp);
-
-	if (tmp)
-		delete [] tmp;
 }
 
 
 Workspace::~Workspace() {
-	delete clientmenu;
+
 }
 
 
-const int Workspace::addWindow(FluxboxWindow *w, bool place) {
+int Workspace::addWindow(FluxboxWindow *w, bool place) {
 	if (! w)
 		return -1;
 
@@ -95,24 +105,38 @@ const int Workspace::addWindow(FluxboxWindow *w, bool place) {
 	stackingList.push_front(w);
 
 	//insert window after the currently focused window	
-	FluxboxWindow *focused = Fluxbox::instance()->getFocusedWindow();	
+	//FluxboxWindow *focused = Fluxbox::instance()->getFocusedWindow();	
+
 	//if there isn't any window that's focused, just add it to the end of the list
+	/*
 	if (focused == 0) {
 		windowList.push_back(w);
+		//Add client to clientmenu
+		m_clientmenu.insert(w->getTitle().c_str());
 	} else {
 		Windows::iterator it = windowList.begin();
-		for (; it != windowList.end(); ++it) {
+		size_t client_insertpoint=0;
+		for (; it != windowList.end(); ++it, ++client_insertpoint) {
 			if (*it == focused) {
-				++it;
+				++it;				
 				break;
 			}
 		}
+
 		windowList.insert(it, w);
+		//Add client to clientmenu
+		m_clientmenu.insert(w->getTitle().c_str(), client_insertpoint);
+		
+
 	}
-
-	clientmenu->insert(w->getTitle().c_str());
-	clientmenu->update();
-
+	*/
+	//add to list
+	m_clientmenu.insert(w->getTitle().c_str());
+	windowList.push_back(w);
+	
+	//update menugraphics
+	m_clientmenu.update();
+	
 	screen->updateNetizenWindowAdd(w->getClientWindow(), m_id);
 
 	raiseWindow(w);
@@ -121,24 +145,30 @@ const int Workspace::addWindow(FluxboxWindow *w, bool place) {
 }
 
 
-const int Workspace::removeWindow(FluxboxWindow *w) {
-	if (! w) return -1;
+int Workspace::removeWindow(FluxboxWindow *w) {
+	if (! w) 
+		return -1;
 
 	stackingList.remove(w);
 
 	if (w->isFocused()) {
-		if (screen->isSloppyFocus())
-			Fluxbox::instance()->setFocusedWindow((FluxboxWindow *) 0);
-		else if (w->isTransient() && w->getTransientFor() &&
-				w->getTransientFor()->isVisible())
+		if (screen->isSloppyFocus()) {
+			Fluxbox::instance()->setFocusedWindow(0); // set focused window to none
+		} else if (w->isTransient() && w->getTransientFor() &&
+				w->getTransientFor()->isVisible()) {
+			/* TODO: check transient
+			if (w->getTransientFor() == w) { // FATAL ERROR, this should not happend
+				cerr<<"w->getTransientFor() == w: aborting!"<<endl;
+				abort();
+			}*/
 			w->getTransientFor()->setInputFocus();
-		else {
+		} else {
 			FluxboxWindow *top = 0;
 			if (stackingList.size()!=0)
 				top = stackingList.front();
 
 			if (!top || !top->setInputFocus()) {
-				Fluxbox::instance()->setFocusedWindow((FluxboxWindow *) 0);
+				Fluxbox::instance()->setFocusedWindow(0); // set focused window to none
 				XSetInputFocus(Fluxbox::instance()->getXDisplay(),
 					screen->getToolbar()->getWindowID(),
 					RevertToParent, CurrentTime);						
@@ -147,7 +177,7 @@ const int Workspace::removeWindow(FluxboxWindow *w) {
 	}
 	
 	if (lastfocus == w)
-		lastfocus = (FluxboxWindow *) 0;
+		lastfocus = 0;
 
 	//could be faster?
 	Windows::iterator it = windowList.begin();
@@ -159,8 +189,8 @@ const int Workspace::removeWindow(FluxboxWindow *w) {
 		}
 	}
 
-	clientmenu->remove(w->getWindowNumber());
-	clientmenu->update();
+	m_clientmenu.remove(w->getWindowNumber());
+	m_clientmenu.update();
 
 	screen->updateNetizenWindowDel(w->getClientWindow());
 
@@ -205,45 +235,27 @@ void Workspace::removeAll(void) {
 
 
 void Workspace::raiseWindow(FluxboxWindow *w) {
-	FluxboxWindow *win = (FluxboxWindow *) 0, *bottom = w;
+	FluxboxWindow *win = w;
 
-	while (bottom->isTransient() && bottom->getTransientFor() &&
-			bottom->getTransientFor() != bottom) //prevent infinite loop
-		bottom = bottom->getTransientFor();
+	while (win->isTransient() && win->getTransientFor())
+		win = win->getTransientFor();
+	
+	int i = 1 + countTransients(*win);
 
-	int i = 1;
-	win = bottom;
-	while (win->hasTransient() && win->getTransient() &&
-			win->getTransient() != win) {//prevent infinite loop
-		win = win->getTransient();
+	Stack nstack(i);
+	Stack::iterator stackit = nstack.begin();
+	
+	*(stackit++) = win->getFrameWindow();
+	screen->updateNetizenWindowRaise(win->getClientWindow());
+	if (! win->isIconic()) {
+		Workspace *wkspc = screen->getWorkspace(win->getWorkspaceNumber());
+		wkspc->stackingList.remove(win);
+		wkspc->stackingList.push_front(win);
+	}	
+	
+	raiseAndFillStack(stackit, *win);
 
-		i++;
-	}
-
-	Window *nstack = new Window[i], *curr = nstack;
-	Workspace *wkspc;
-
-	win = bottom;
-	while (True) {
-		*(curr++) = win->getFrameWindow();
-		screen->updateNetizenWindowRaise(win->getClientWindow());
-
-		if (! win->isIconic()) {
-			wkspc = screen->getWorkspace(win->getWorkspaceNumber());
-			wkspc->stackingList.remove(win);
-			wkspc->stackingList.push_front(win);
-		}
-
-		if (! win->hasTransient() || ! win->getTransient() ||
-			win->getTransient() == win) //prevent infinite loop
-			break;
-		
-		win = win->getTransient();
-	}
-
-	screen->raiseWindows(nstack, i);
-
-	delete [] nstack;
+	screen->raiseWindows(nstack);
 }
 
 void Workspace::lowerWindow(FluxboxWindow *w) {
@@ -253,48 +265,29 @@ void Workspace::lowerWindow(FluxboxWindow *w) {
 			&& bottom->getTransientFor() != bottom) //prevent infinite loop
 		bottom = bottom->getTransientFor();
 
-	int i = 1;
 	win = bottom;
-	while (win->hasTransient() && win->getTransient() && 
-			win->getTransient() != win) { //prevent infinite loop
-		win = win->getTransient();
+	int i = 1 + countTransients(*w);
 
-		i++;
+	Stack st(i);
+	Stack::iterator stackit = st.begin();
+	lowerAndFillStack(stackit, *win);
+	(*stackit) = win->getFrameWindow();
+
+	screen->updateNetizenWindowLower(win->getClientWindow());
+	if (! win->isIconic()) {
+		Workspace *wkspc = screen->getWorkspace(win->getWorkspaceNumber());
+		wkspc->stackingList.remove(win);
+		wkspc->stackingList.push_back(win);
 	}
 
-	Window *nstack = new Window[i], *curr = nstack;
-	Workspace *wkspc;
+	XLowerWindow(BaseDisplay::getXDisplay(), st.front());
+	XRestackWindows(BaseDisplay::getXDisplay(), &st[0], st.size());
 
-	while (True) {
-		*(curr++) = win->getFrameWindow();
-		screen->updateNetizenWindowLower(win->getClientWindow());
-
-		if (! win->isIconic()) {
-			wkspc = screen->getWorkspace(win->getWorkspaceNumber());
-			wkspc->stackingList.remove(win);
-			wkspc->stackingList.push_back(win);
-		}
-
-		if (! win->getTransientFor() || 
-			win->getTransientFor() == win)//prevent infinite loop
-			break;
-
-		win = win->getTransientFor();
-	}
-
-	Fluxbox::instance()->grab();
-
-	XLowerWindow(screen->getBaseDisplay()->getXDisplay(), *nstack);
-	XRestackWindows(screen->getBaseDisplay()->getXDisplay(), nstack, i);
-
-	Fluxbox::instance()->ungrab();
-
-	delete [] nstack;
 }
 
 
 void Workspace::reconfigure(void) {
-	clientmenu->reconfigure();
+	m_clientmenu.reconfigure();
 
 	Windows::iterator it = windowList.begin();
 	Windows::iterator it_end = windowList.end();
@@ -305,26 +298,104 @@ void Workspace::reconfigure(void) {
 }
 
 
-FluxboxWindow *Workspace::getWindow(unsigned int index) const{
+const FluxboxWindow *Workspace::getWindow(unsigned int index) const {
 	if (index < windowList.size())
 		return windowList[index];
+	return 0;
+}
 
+FluxboxWindow *Workspace::getWindow(unsigned int index) {
+	if (index < windowList.size())
+		return windowList[index];
 	return 0;
 }
 
 
-const int Workspace::getCount(void) const {
+int Workspace::getCount() const {
 	return windowList.size();
 }
 
+namespace {
+// helper class for checkGrouping
+class FindInGroup {
+public:
+	FindInGroup(const FluxboxWindow &w):m_w(w) { }
+	bool operator ()(const string &name) {
+		return (name == m_w.instanceName());
+	}
+private:
+	const FluxboxWindow &m_w;
+};
 
-void Workspace::update(void) {
-	clientmenu->update();
+};
+
+//Note: this function doesn't check if the window is groupable
+void Workspace::checkGrouping(FluxboxWindow &win) {
+#ifdef DEBUG
+	cerr<<__FILE__<<"("<<__LINE__<<"): Checking grouping. ("<<win.instanceName()<<"/"<<
+		win.className()<<")"<<endl;	
+#endif // DEBUG
+
+	// go throu every group and search for matching win instancename
+	GroupList::iterator g(m_groups.begin());
+	GroupList::iterator g_end(m_groups.end());
+	for (; g != g_end; ++g) {
+		Group::iterator name((*g).begin());
+		Group::iterator name_end((*g).end());
+		for (; name != name_end; ++name) {
+
+			if ((*name) != win.instanceName())
+				continue;
+
+			// find a window with the specific name
+			Windows::iterator wit(getWindowList().begin());
+			Windows::iterator wit_end(getWindowList().end());
+			for (; wit != wit_end; ++wit) {
+#ifdef DEBUG
+				cerr<<__FILE__<<" check group with : "<<(*wit)->instanceName()<<endl;
+#endif // DEBUG
+				if (find_if((*g).begin(), (*g).end(), FindInGroup(*(*wit))) != (*g).end()) {
+					//toggle tab on
+					if ((*wit)->getTab() == 0)
+						(*wit)->setTab(true);
+					if (win.getTab() == 0)
+						win.setTab(true);
+					(*wit)->getTab()->insert(win.getTab());
+
+					return; // grouping done
+				}
+			}
+
+		}
+
+	}
+}
+
+bool Workspace::loadGroups(const std::string &filename) {
+	ifstream infile(filename.c_str());
+	if (!infile)
+		return false;
+	m_groups.clear(); // erase old groups
+
+	// load new groups
+	while (!infile.eof()) {
+		string line;
+		vector<string> names;
+		getline(infile, line);
+		StringUtil::stringtok(names, line);
+		m_groups.push_back(names);
+	}
+	
+	return true;
+}
+
+void Workspace::update() {
+	m_clientmenu.update();
 	screen->getToolbar()->redrawWindowLabel(True);
 }
 
 
-bool Workspace::isCurrent(void) const{
+bool Workspace::isCurrent() const{
 	return (m_id == screen->getCurrentWorkspaceID());
 }
 
@@ -338,9 +409,8 @@ void Workspace::setCurrent(void) {
 }
 
 
-void Workspace::setName(const char *name) {
-
-	if (name) {
+void Workspace::setName(const std::string &name) {
+	if (name.size() != 0) {
 		m_name = name;
 	} else { //if name == 0 then set default name from nls
 		char tname[128];
@@ -353,8 +423,8 @@ void Workspace::setName(const char *name) {
 	
 	screen->updateWorkspaceNamesAtom();
 	
-	clientmenu->setLabel(m_name.c_str());
-	clientmenu->update();
+	m_clientmenu.setLabel(m_name.c_str());
+	m_clientmenu.update();
 }
 
 //------------ shutdown ---------
@@ -362,10 +432,10 @@ void Workspace::setName(const char *name) {
 // on the workspace and then
 // clears the windowList
 //-------------------------------
-void Workspace::shutdown(void) {
-
+void Workspace::shutdown() {
+	// note: when the window dies it'll remove it self from the list
 	while (!windowList.empty()) {
-		windowList.back()->restore();
+		windowList.back()->restore(true); // restore with remap
 		delete windowList.back(); //delete window (the window removes it self from windowList)
 	}
 }
@@ -375,10 +445,10 @@ void Workspace::placeWindow(FluxboxWindow *win) {
 	Bool placed = False;
 	int borderWidth4x = screen->getBorderWidth2x() * 2,
 #ifdef		SLIT
-		slit_x = screen->getSlit()->getX() - screen->getBorderWidth(),
-		slit_y = screen->getSlit()->getY() - screen->getBorderWidth(),
-		slit_w = screen->getSlit()->getWidth() + borderWidth4x,
-		slit_h = screen->getSlit()->getHeight() + borderWidth4x,
+		slit_x = screen->getSlit()->x() - screen->getBorderWidth(),
+		slit_y = screen->getSlit()->y() - screen->getBorderWidth(),
+		slit_w = screen->getSlit()->width() + borderWidth4x,
+		slit_h = screen->getSlit()->height() + borderWidth4x,
 #endif // SLIT
 		toolbar_x = screen->getToolbar()->getX() - screen->getBorderWidth(),
 		toolbar_y = screen->getToolbar()->getY() - screen->getBorderWidth(),
@@ -500,12 +570,6 @@ void Workspace::placeWindow(FluxboxWindow *win) {
 									? screen->getTabHeight()
 									: screen->getTabWidth();
 								break;
-							default:
-								#ifdef DEBUG
-								cerr << __FILE__ << ":" <<__LINE__ << ": " <<
-									"Unsupported Placement" << endl;
-								#endif // DEBUG
-								break;
 							}
 						} else { // shaded window
 							if (screen->getTabPlacement() == Tab::PTOP)
@@ -718,3 +782,55 @@ void Workspace::placeWindow(FluxboxWindow *win) {
 
 	win->configure(place_x, place_y, win->getWidth(), win->getHeight());
 }
+
+
+void Workspace::raiseAndFillStack(Stack::iterator &stackit, const FluxboxWindow &w) {
+	if (w.getTransients().empty())
+		return;
+
+	std::list<FluxboxWindow *>::const_iterator it = w.getTransients().begin();
+	std::list<FluxboxWindow *>::const_iterator it_end = w.getTransients().end();
+	for (; it != it_end; ++it) {		
+		*stackit++ = (*it)->getFrameWindow();
+		
+		screen->updateNetizenWindowRaise((*it)->getClientWindow());
+
+		if (! (*it)->isIconic()) {
+			Workspace *wkspc = screen->getWorkspace((*it)->getWorkspaceNumber());
+			wkspc->stackingList.remove((*it));
+			wkspc->stackingList.push_front((*it));
+		}
+
+		
+	}
+	
+	it = w.getTransients().begin();
+	for (; it != it_end; ++it)
+		raiseAndFillStack(stackit, *(*it));
+	
+
+}
+
+void Workspace::lowerAndFillStack(Stack::iterator &stackit, const FluxboxWindow &win) {
+	if (win.getTransients().empty()) // nothing to lower and stack
+		return;
+
+	std::list<FluxboxWindow *>::const_reverse_iterator it = win.getTransients().rbegin();
+	std::list<FluxboxWindow *>::const_reverse_iterator it_end = win.getTransients().rend();
+	for (; it != it_end; ++it)
+		lowerAndFillStack(stackit, *(*it));
+	
+	it = win.getTransients().rbegin();
+	
+	for (; it != it_end; ++it) {
+		(*stackit) = (*it)->getFrameWindow();
+		++stackit;
+		screen->updateNetizenWindowLower((*it)->getClientWindow());
+		if (! (*it)->isIconic()) {
+			Workspace *wkspc = screen->getWorkspace((*it)->getWorkspaceNumber());
+			wkspc->stackingList.remove((*it));
+			wkspc->stackingList.push_back((*it));
+		}
+	}
+
+}
