@@ -22,28 +22,16 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: fluxbox.cc,v 1.52 2002/04/19 09:33:42 fluxgen Exp $
+// $Id: fluxbox.cc,v 1.73 2002/08/30 14:07:38 fluxgen Exp $
 
-//Use some GNU extensions
-#ifndef	 _GNU_SOURCE
-#define	 _GNU_SOURCE
-#endif // _GNU_SOURCE
 
-#ifdef		HAVE_CONFIG_H
-#	include "../config.h"
-#endif // HAVE_CONFIG_H
+#include "fluxbox.hh"
 
 #include "i18n.hh"
-#include "fluxbox.hh"
 #include "Basemenu.hh"
 #include "Clientmenu.hh"
 #include "Rootmenu.hh"
 #include "Screen.hh"
-
-#ifdef SLIT
-#include "Slit.hh"
-#endif // SLIT
-
 #include "Toolbar.hh"
 #include "Window.hh"
 #include "Workspace.hh"
@@ -51,46 +39,53 @@
 #include "StringUtil.hh"
 #include "Resource.hh"
 #include "XrmDatabaseHelper.hh"
+#ifdef SLIT
+#include "Slit.hh"
+#endif // SLIT
 
+//Use GNU extensions
+#ifndef	 _GNU_SOURCE
+#define	 _GNU_SOURCE
+#endif // _GNU_SOURCE
+
+#ifdef		HAVE_CONFIG_H
+#include "../config.h"
+#endif // HAVE_CONFIG_H
+
+// X headers
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xresource.h>
 #include <X11/Xatom.h>
 #include <X11/keysym.h>
-
-#ifdef		SHAPE
+#include <X11/cursorfont.h>
+#ifdef SHAPE
 #include <X11/extensions/shape.h>
 #endif // SHAPE
 
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 
-#ifdef		HAVE_STDIO_H
-#	include <stdio.h>
-#endif // HAVE_STDIO_H
-
-#ifdef		STDC_HEADERS
-#	include <stdlib.h>
-#	include <string.h>
-#endif // STDC_HEADERS
-
-#ifdef		HAVE_UNISTD_H
-#	include <sys/types.h>
-#	include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#include <sys/types.h>
+#include <unistd.h>
 #endif // HAVE_UNISTD_H
 
-#ifdef		HAVE_SYS_PARAM_H
-#	include <sys/param.h>
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
 #endif // HAVE_SYS_PARAM_H
 
 #ifndef	 MAXPATHLEN
 #define	 MAXPATHLEN 255
 #endif // MAXPATHLEN
 
-#ifdef		HAVE_SYS_SELECT_H
-#	include <sys/select.h>
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
 #endif // HAVE_SYS_SELECT_H
 
-#ifdef		HAVE_SIGNAL_H
-#	include <signal.h>
+#ifdef HAVE_SIGNAL_H
+#include <signal.h>
 #endif // HAVE_SIGNAL_H
 
 #ifdef		HAVE_SYS_SIGNAL_H
@@ -117,22 +112,32 @@
 #	include <libgen.h>
 #endif // HAVE_LIBGEN_H
 
+#include <sys/wait.h>
+
 #include <iostream>
 #include <string>
-#include <strstream>
 #include <memory>
+#include <algorithm>
 
 using namespace std;
+using namespace FbTk;
 
 #ifndef	 HAVE_BASENAME
-static inline char *basename (char *);
-static inline char *basename (char *s) {
+namespace {
+
+char *basename (char *s) {
 	char *save = s;
 
-	while (*s) if (*s++ == '/') save = s;
+	while (*s) {
+		if (*s++ == '/')
+			save = s;
+	}
 
 	return save;
 }
+
+}; // end anonymous namespace
+
 #endif // HAVE_BASENAME
 
 #define RC_PATH "fluxbox"
@@ -281,20 +286,13 @@ getString() {
 //static singleton var
 Fluxbox *Fluxbox::singleton=0;
 
-//------------ instance ---------------------
-//returns singleton object of Fluxbox class
-//since we only need to create one instance of Fluxbox
-//-------------------------------------------
-Fluxbox *Fluxbox::instance(int m_argc, char **m_argv, char *dpy_name, char *rc) {
-	return singleton;
-}
 //default values for titlebar left and right
 //don't forget to change last value in m_rc_titlebar_* if you add more to these
 Fluxbox::Titlebar Fluxbox::m_titlebar_left[] = {STICK};
 Fluxbox::Titlebar Fluxbox::m_titlebar_right[] = {MINIMIZE, MAXIMIZE, CLOSE};
 
-Fluxbox::Fluxbox(int m_argc, char **m_argv, char *dpy_name, char *rc)
-: BaseDisplay(m_argv[0], dpy_name),
+Fluxbox::Fluxbox(int m_argc, char **m_argv, const char *dpy_name, const char *rc)
+: BaseDisplay(m_argv[0], dpy_name), FbAtoms(getXDisplay()),
 m_resourcemanager(), m_screen_rm(),
 m_rc_tabs(m_resourcemanager, true, "session.tabs", "Session.Tabs"),
 m_rc_iconbar(m_resourcemanager, true, "session.iconbar", "Session.Iconbar"),
@@ -302,6 +300,8 @@ m_rc_colors_per_channel(m_resourcemanager, 4, "session.colorsPerChannel", "Sessi
 m_rc_stylefile(m_resourcemanager, "", "session.styleFile", "Session.StyleFile"),
 m_rc_menufile(m_resourcemanager, DEFAULTMENU, "session.menuFile", "Session.MenuFile"),
 m_rc_keyfile(m_resourcemanager, DEFAULTKEYSFILE, "session.keyFile", "Session.KeyFile"),
+m_rc_slitlistfile(m_resourcemanager, "", "session.slitlistFile", "Session.SlitlistFile"),
+m_rc_groupfile(m_resourcemanager, "", "session.groupFile", "Session.GroupFile"),
 m_rc_titlebar_left(m_resourcemanager, TitlebarList(&m_titlebar_left[0], &m_titlebar_left[1]), "session.titlebar.left", "Session.Titlebar.Left"),
 m_rc_titlebar_right(m_resourcemanager, TitlebarList(&m_titlebar_right[0], &m_titlebar_right[3]), "session.titlebar.right", "Session.Titlebar.Right"),
 m_rc_cache_life(m_resourcemanager, 5, "session.cacheLife", "Session.CacheLife"),
@@ -309,22 +309,44 @@ m_rc_cache_max(m_resourcemanager, 200, "session.cacheMax", "Session.CacheMax"),
 focused_window(0), masked_window(0),
 timer(this),
 no_focus(false),
-rc_file(rc),
+rc_file(rc ? rc : ""),
 argv(m_argv), argc(m_argc), 
 key(0)
 {
 
+	if (singleton != 0) {
+		cerr<<"Fatal! There can only one instance of fluxbox class."<<endl;
+		abort();
+	}
+	
+	//setup signals
+	SignalHandler *sigh = SignalHandler::instance();
+	
+	sigh->registerHandler(SIGSEGV, this);
+	sigh->registerHandler(SIGFPE, this);
+	sigh->registerHandler(SIGTERM, this);
+	sigh->registerHandler(SIGINT, this);
+	sigh->registerHandler(SIGCHLD, this);
+	sigh->registerHandler(SIGHUP, this);
+	sigh->registerHandler(SIGUSR1, this);	
+	sigh->registerHandler(SIGUSR2, this);
+
+	//setup cursor bitmaps
+	cursor.session = XCreateFontCursor(getXDisplay(), XC_left_ptr);
+	cursor.move = XCreateFontCursor(getXDisplay(), XC_fleur);
+	cursor.ll_angle = XCreateFontCursor(getXDisplay(), XC_ll_angle);
+	cursor.lr_angle = XCreateFontCursor(getXDisplay(), XC_lr_angle);
+
 	//singleton pointer
 	singleton = this;
-	BaseDisplay::GrabGuard gg(*this);
-	gg.grab();
+	grab();
 	
 	setupConfigFiles();
 	
 	if (! XSupportsLocale())
 		fprintf(stderr, "X server does not support locale\n");
 
-	if (XSetLocaleModifiers("") == NULL)
+	if (XSetLocaleModifiers("") == 0)
 		fprintf(stderr, "cannot set locale modifiers\n");
 
 // Set default values to member variables
@@ -333,18 +355,18 @@ key(0)
 	
 	masked = None;
 	
-	#ifdef SLIT
-	#ifdef KDE
+#ifdef SLIT
+#ifdef KDE
 	//For KDE dock applets
 	kwm1_dockwindow = XInternAtom(getXDisplay(), "KWM_DOCKWINDOW", False); //KDE v1.x
 	kwm2_dockwindow = XInternAtom(getXDisplay(), "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", False); //KDE v2.x
-	#endif //KDE
+#endif //KDE
 
-	#endif // SLIT
+#endif // SLIT
 
-	#ifdef HAVE_GETPID
+#ifdef HAVE_GETPID
 	fluxbox_pid = XInternAtom(getXDisplay(), "_BLACKBOX_PID", False);
-	#endif // HAVE_GETPID
+#endif // HAVE_GETPID
 
 	int i;
 	load_rc();
@@ -362,7 +384,7 @@ key(0)
 
 		screenList.push_back(screen);
 	}
-	
+
 	I18n *i18n = I18n::instance();
 	if (screenList.size() == 0) {
 		fprintf(stderr,
@@ -373,7 +395,7 @@ key(0)
 
 		throw static_cast<int>(3);
 	}
-
+	
 	XSynchronize(getXDisplay(), False);
 	XSync(getXDisplay(), False);
 
@@ -382,10 +404,8 @@ key(0)
 	timer.setTimeout(0);
 	timer.fireOnce(True);
 
-	//create keybindings handler and load keys file
-	char *keyfilename = StringUtil::expandFilename((*m_rc_keyfile).c_str());
-	key = new Keys(getXDisplay(), const_cast<char *>(keyfilename));
-	delete keyfilename;
+	//create keybindings handler and load keys file	
+	key = new Keys(getXDisplay(), StringUtil::expandFilename(*m_rc_keyfile).c_str());
 
 	ungrab();
 }
@@ -417,7 +437,7 @@ void Fluxbox::setupConfigFiles() {
 	createInit = createKeys = createMenu = false;
 
 	string dirname = getenv("HOME")+string("/.")+string(RC_PATH) + "/";
-	string initFile, keysFile, menuFile;
+	string initFile, keysFile, menuFile, slitlistFile;
 	initFile = dirname+RC_INIT_FILE;
 	keysFile = dirname+"keys";
 	menuFile = dirname+"menu";
@@ -511,7 +531,7 @@ void Fluxbox::setupConfigFiles() {
 	}
 }
 
-void Fluxbox::process_event(XEvent *e) {
+void Fluxbox::handleEvent(XEvent * const e) {
 
 	if ((masked == e->xany.window) && masked_window &&
 			(e->type == MotionNotify)) {
@@ -530,17 +550,17 @@ void Fluxbox::process_event(XEvent *e) {
 	{
 		FluxboxWindow *win = (FluxboxWindow *) 0;
 
-		#ifdef SLIT
+#ifdef SLIT
 		Slit *slit = (Slit *) 0;
-		#endif // SLIT
+#endif // SLIT
 		
 		if ((win = searchWindow(e->xconfigurerequest.window))) {
 			win->configureRequestEvent(&e->xconfigurerequest);
 
-		#ifdef SLIT
+#ifdef SLIT
 		} else if ((slit = searchSlit(e->xconfigurerequest.window))) {
 			slit->configureRequestEvent(&e->xconfigurerequest);
-		#endif // SLIT
+#endif // SLIT
 
 		} else {
 			grab();
@@ -567,20 +587,20 @@ void Fluxbox::process_event(XEvent *e) {
 	break;
 	case MapRequest:
 	{
-		#ifdef		DEBUG
+#ifdef DEBUG
 		fprintf(stderr,
 			I18n::instance()->
 				getMessage(
 				FBNLS::blackboxSet, FBNLS::blackboxMapRequest,
 				 "Fluxbox::process_event(): MapRequest for 0x%lx\n"),
 					e->xmaprequest.window);
-		#endif // DEBUG
+#endif // DEBUG
 		
-		#ifdef SLIT
-		#ifdef KDE
+#ifdef SLIT
+#ifdef KDE
 		//Check and see if client is KDE dock applet.
 		//If so add to Slit
-		bool iskdedockapp = False;
+		bool iskdedockapp = false;
 		Atom ajunk;
 		int ijunk;
 		unsigned long *data = (unsigned long *) 0, uljunk;
@@ -610,14 +630,14 @@ void Fluxbox::process_event(XEvent *e) {
 
 		if (iskdedockapp) {
 			XSelectInput(getXDisplay(), e->xmaprequest.window, StructureNotifyMask);
-			std::list<BScreen *>::iterator it = screenList.begin();			
+			ScreenList::iterator it = screenList.begin();			
 			for (; (*it) == screenList.back(); ++it) {
 				(*it)->getSlit()->addClient(e->xmaprequest.window);
 			}
 			return;
 		}
-		#endif //KDE
-		#endif // SLIT
+#endif //KDE
+#endif // SLIT
 			
 		FluxboxWindow *win = searchWindow(e->xmaprequest.window);
 
@@ -637,7 +657,7 @@ void Fluxbox::process_event(XEvent *e) {
 	case MapNotify:
 	{
 		FluxboxWindow *win = searchWindow(e->xmap.window);
-		if (win!=0)
+		if (win != 0)
 			win->mapNotifyEvent(&e->xmap);
 
 	}
@@ -645,33 +665,34 @@ void Fluxbox::process_event(XEvent *e) {
 	
 
 	case UnmapNotify:
-	handleUnmapNotify(e->xunmap);
+		handleUnmapNotify(e->xunmap);
 	break;	
 	case CreateNotify:
-		break;
-	case DestroyNotify:
-		{
+	break;
+	case DestroyNotify: {
 			FluxboxWindow *win = (FluxboxWindow *) 0;
 
-			#ifdef SLIT
+#ifdef SLIT
 			Slit *slit = 0;
-			#endif // SLIT
+#endif // SLIT
 
 			if ((win = searchWindow(e->xdestroywindow.window))) {
 				if (win->destroyNotifyEvent(&e->xdestroywindow)) {
 					delete win;
 					win = 0;
 				}
-			#ifdef SLIT
-			} else if ((slit = searchSlit(e->xdestroywindow.window))) {
-				slit->removeClient(e->xdestroywindow.window, False);
-			#endif // SLIT
+
+			} 
+#ifdef SLIT
+			else if ((slit = searchSlit(e->xdestroywindow.window))) {
+				slit->removeClient(e->xdestroywindow.window, false);
+
 			}
+#endif // SLIT
 
 		}
 	break;
-	case MotionNotify:
-		{
+	case MotionNotify: {
 			last_time = e->xmotion.time;
 
 			FluxboxWindow *win = 0;
@@ -688,8 +709,7 @@ void Fluxbox::process_event(XEvent *e) {
 			
 		}
 	break;
-	case PropertyNotify:
-	{
+	case PropertyNotify: {
 			
 		last_time = e->xproperty.time;
 
@@ -702,8 +722,7 @@ void Fluxbox::process_event(XEvent *e) {
 			
 	}
 	break;
-	case EnterNotify:
-		{
+	case EnterNotify: {
 			last_time = e->xcrossing.time;
 
 			BScreen *screen = (BScreen *) 0;
@@ -760,7 +779,7 @@ void Fluxbox::process_event(XEvent *e) {
 				}
 			}
 				
-#ifdef		SLIT
+#ifdef SLIT
 			else if ((slit = searchSlit(e->xcrossing.window)))
 				slit->enterNotifyEvent(&e->xcrossing);
 #endif // SLIT
@@ -815,19 +834,16 @@ void Fluxbox::process_event(XEvent *e) {
 	case KeyPress:
 		handleKeyEvent(e->xkey);
 	break;
-	case ColormapNotify:
-	{
+	case ColormapNotify: {
 		BScreen *screen = searchScreen(e->xcolormap.window);
 
-		if (screen)
-		screen->setRootColormapInstalled((e->xcolormap.state ==
+		if (screen != 0) {
+			screen->setRootColormapInstalled((e->xcolormap.state ==
 				ColormapInstalled) ? True : False);
-
-	
+		}
 	}
 	break;
-	case FocusIn:
-	{
+	case FocusIn: {
 		if (e->xfocus.mode == NotifyUngrab ||
 				e->xfocus.detail == NotifyPointer)
 			break;
@@ -836,16 +852,13 @@ void Fluxbox::process_event(XEvent *e) {
 		if (win && ! win->isFocused())
 			setFocusedWindow(win);
 	
-	}
-	break;
+	} break;
 	case FocusOut:
 	break;
-
 	case ClientMessage:
 		handleClientMessage(e->xclient);
 	break;
-	default:
-		{
+	default: {
 
 		#ifdef SHAPE
 		if (e->type == getShapeEventBase()) {
@@ -898,8 +911,8 @@ void Fluxbox::handleButtonEvent(XButtonEvent &be) {
 		} else if ((tab = searchTab(be.window))) {
 			tab->buttonPressEvent(&be);
 		} else {
-			std::list<BScreen *>::iterator it = screenList.begin();
-			std::list<BScreen *>::iterator it_end = screenList.end();
+			ScreenList::iterator it = screenList.begin();
+			ScreenList::iterator it_end = screenList.end();
 
 			for (; it != it_end; ++it) {
 
@@ -974,9 +987,9 @@ void Fluxbox::handleButtonEvent(XButtonEvent &be) {
 						checkMenu();
 						screen->getRootmenu()->show();
 					}
-				} else if (be.button == 4) {
+				} else if (screen->isDesktopWheeling() && be.button == 4) {
 					screen->nextWorkspace(1);
-				} else if (be.button == 5) {
+				} else if (screen->isDesktopWheeling() && be.button == 5) {
 					screen->prevWorkspace(1);
 				}
 			}
@@ -1012,28 +1025,29 @@ void Fluxbox::handleUnmapNotify(XUnmapEvent &ue) {
 		
 	FluxboxWindow *win = 0;
 	
-	#ifdef SLIT
+#ifdef SLIT
 	Slit *slit = (Slit *) 0;
-	#endif // SLIT
+#endif // SLIT
 	BScreen *screen = searchScreen(ue.event);
 	
 	if ( (ue.event != ue.window) && (screen != 0 || !ue.send_event))
 	 	return;
 	
-	if ( (win = searchWindow(ue.window)) !=0 ) {
+	if ((win = searchWindow(ue.window)) != 0) {
 
-		if (win->unmapNotifyEvent(&ue))
+		if (win->unmapNotifyEvent(&ue)) {
 			delete win;
-		      
+			if (focused_window == win) // some extra checking
+				focused_window = 0;
+			win = 0;
+		}
+  
 	}
-	#ifdef SLIT
+#ifdef SLIT
 	else if ((slit = searchSlit(ue.window))!=0) {
 		slit->removeClient(ue.window);
-		#ifdef DEBUG
-		cerr<<__FILE__<<"("<<__LINE__<<"): Here"<<endl;
-		#endif 
 	}
-	#endif // SLIT
+#endif // SLIT
 
 }
 
@@ -1041,11 +1055,11 @@ void Fluxbox::handleUnmapNotify(XUnmapEvent &ue) {
 // Handles XClientMessageEvent
 //-----------------------------------------
 void Fluxbox::handleClientMessage(XClientMessageEvent &ce) {
-	#ifdef DEBUG
+#ifdef DEBUG
 	cerr<<__FILE__<<"("<<__LINE__<<"): ClientMessage. data.l[0]=0x"<<hex<<ce.data.l[0]<<
-		"  type=0x"<<ce.message_type<<dec<<endl;
+	"  message_type=0x"<<ce.message_type<<dec<<endl;
 	
-	#endif
+#endif // DEBUG
 
 	if (ce.format != 32)
 		return;
@@ -1093,7 +1107,7 @@ void Fluxbox::handleClientMessage(XClientMessageEvent &ce) {
 			win->changeBlackboxHints(&net);
 		}
 	} else {
-		bool val  = false;
+		bool val = false;
 		#ifdef GNOME
 		val  = checkGnomeAtoms(ce);
 		#endif //!GNOME
@@ -1102,6 +1116,8 @@ void Fluxbox::handleClientMessage(XClientMessageEvent &ce) {
 		if (!val)
 			val = checkNETWMAtoms(ce);
 		#endif //!NEWWMSPEC
+		//disable `unused`-warning
+		val = true;
 	}
 }
 //----------- handleKeyEvent ---------------
@@ -1130,16 +1146,46 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
 			if (action==Keys::LASTKEYGRAB) //if action not found end case
 				break;
 
+			// what to allow if moving
+			if (focused_window && focused_window->isMoving()) {
+				int allowed = false;
+				switch (action) {
+				case Keys::WORKSPACE:
+				case Keys::SENDTOWORKSPACE:
+				case Keys::WORKSPACE1:
+				case Keys::WORKSPACE2:
+				case Keys::WORKSPACE3:
+				case Keys::WORKSPACE4:
+				case Keys::WORKSPACE5:
+				case Keys::WORKSPACE6:
+				case Keys::WORKSPACE7:
+				case Keys::WORKSPACE8:
+				case Keys::WORKSPACE9:
+				case Keys::WORKSPACE10:
+				case Keys::WORKSPACE11:
+				case Keys::WORKSPACE12:
+				case Keys::NEXTWORKSPACE:
+				case Keys::PREVWORKSPACE:
+				case Keys::LEFTWORKSPACE:
+				case Keys::RIGHTWORKSPACE:
+					allowed = true;
+					break;
+				default:
+					allowed = false;
+				}
+				if (!allowed) break;
+			}
+
 			switch (action) {					
 			case Keys::WORKSPACE:
-                            // Workspace1 has id 0, hence -1
-			    screen->changeWorkspaceID(key->getParam()-1);
+				// Workspace1 has id 0, hence -1
+				screen->changeWorkspaceID(key->getParam()-1);
 			break;
 			case Keys::SENDTOWORKSPACE:
-                            // Workspace1 has id 0, hence -1
-			    screen->sendToWorkspace(key->getParam()-1);
+				// Workspace1 has id 0, hence -1
+				screen->sendToWorkspace(key->getParam()-1);
 			break;
-            // NOTE!!! The WORKSPACEn commands are not needed anymore
+			// NOTE!!! The WORKSPACEn commands are not needed anymore
 			case Keys::WORKSPACE1:
 				screen->changeWorkspaceID(0);
 			break;
@@ -1189,14 +1235,20 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
 				screen->rightWorkspace(key->getParam());
 			break;
 			case Keys::KILLWINDOW: //kill the current window
-				XKillClient(screen->getBaseDisplay()->getXDisplay(),
-					focused_window->getClientWindow());
+				if (focused_window) {
+					XKillClient(screen->getBaseDisplay()->getXDisplay(),
+						focused_window->getClientWindow());
+				}
 			break;
 			case Keys::NEXTWINDOW:	//activate next window
 				screen->nextFocus(key->getParam());
+				if (focused_window && focused_window->getTab())
+					focused_window->getTab()->raise();
 			break;
 			case Keys::PREVWINDOW:	//activate prev window
 				screen->prevFocus(key->getParam());
+				if (focused_window && focused_window->getTab())
+					focused_window->getTab()->raise();
 			break;
 			case Keys::NEXTTAB: 
 				if (focused_window && focused_window->getTab()) {
@@ -1226,6 +1278,32 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
 					}
 				}
 			break;
+			case Keys::FIRSTTAB:
+				if (focused_window && focused_window->getTab()) {
+					Tab *tab = focused_window->getTab();
+					screen->getCurrentWorkspace()->raiseWindow(
+						tab->first()->getWindow());
+					tab->first()->getWindow()->setInputFocus();
+				}
+			break;
+			case Keys::LASTTAB:
+				if (focused_window && focused_window->getTab()) {
+					Tab *tab = focused_window->getTab();
+					screen->getCurrentWorkspace()->raiseWindow(
+						tab->last()->getWindow());
+					tab->last()->getWindow()->setInputFocus();
+				}
+			break;
+			case Keys::MOVETABPREV:
+				if (focused_window && focused_window->getTab()) {
+					focused_window->getTab()->movePrev();
+				}
+			break;
+			case Keys::MOVETABNEXT:
+				if (focused_window && focused_window->getTab()) {
+					focused_window->getTab()->moveNext();
+				}
+			break;
 			case Keys::EXECUTE: //execute command on keypress
 			{
 				#ifndef		__EMX__
@@ -1248,8 +1326,8 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
 			break;
 			case Keys::ROOTMENU: //show root menu
 			{
-                std::list<BScreen *>::iterator it = screenList.begin();
-                std::list<BScreen *>::iterator it_end = screenList.end();
+				ScreenList::iterator it = screenList.begin();
+				ScreenList::iterator it_end = screenList.end();
 
 				for (; it != it_end; ++it) {
 
@@ -1365,7 +1443,7 @@ void Fluxbox::doWindowAction(Keys::KeyAction action, const int param) {
 				focused_window->getXFrame(), focused_window->getYFrame()+param,
 				focused_window->getWidth(), focused_window->getHeight());
 		break;
-        // NOTE !!! BIGNUDGExxxx is not needed, just use 10 as a parameter
+		// NOTE !!! BIGNUDGExxxx is not needed, just use 10 as a parameter
 		case Keys::BIGNUDGERIGHT:		
 			focused_window->configure(
 				focused_window->getXFrame()+10, focused_window->getYFrame(),
@@ -1463,11 +1541,11 @@ bool Fluxbox::checkGnomeAtoms(XClientMessageEvent &ce) {
 	FluxboxWindow *win = 0;
 	win = searchWindow(ce.window);
 	screen = searchScreen(ce.window);
-	
+
 	if (ce.message_type == getGnomeWorkspaceAtom()) {
-		#ifdef DEBUG
+#ifdef DEBUG
 		cerr<<__FILE__<<"("<<__LINE__<<"): Got workspace atom="<<ce.data.l[0]<<endl;
-		#endif//!DEBUG
+#endif//!DEBUG
 		if ( win !=0 && // the message sent to client window?
 				win->getScreen() && ce.data.l[0] >= 0 &&
 				ce.data.l[0] < (signed)win->getScreen()->getCount()) {
@@ -1479,17 +1557,16 @@ bool Fluxbox::checkGnomeAtoms(XClientMessageEvent &ce) {
 			screen->changeWorkspaceID(ce.data.l[0]);
 		return true;
 	} else if (win) {
-	
 		if (ce.message_type == getGnomeStateAtom()) {
-			#ifdef DEBUG
+#ifdef DEBUG
 			cerr<<__FILE__<<"("<<__LINE__<<"): _WIN_STATE"<<endl;
-			#endif
+#endif // DEBUG
 			
-			#ifdef DEBUG
+#ifdef DEBUG
 			cerr<<__FILE__<<"("<<__LINE__<<"): Mask of members to change:"<<
 				hex<<ce.data.l[0]<<dec<<endl; // mask_of_members_to_change
 			cerr<<"New members:"<<ce.data.l[1]<<endl;
-			#endif
+#endif // DEBUG
 	
 			//get new states			
 			int flag = ce.data.l[0] & ce.data.l[1];
@@ -1497,9 +1574,10 @@ bool Fluxbox::checkGnomeAtoms(XClientMessageEvent &ce) {
 			win->setGnomeState(flag);
 			
 		} else if (ce.message_type == getGnomeHintsAtom()) {
-			#ifdef DEBUG
+#ifdef DEBUG
 			cerr<<__FILE__<<"("<<__LINE__<<"): _WIN_HINTS"<<endl;
-			#endif
+#endif // DEBUG
+
 		} else 
 			return false; //the gnome atom wasn't found or not supported
 	} else	
@@ -1528,37 +1606,65 @@ bool Fluxbox::checkNETWMAtoms(XClientMessageEvent &ce) {
 }
 #endif //!NEWWMSPEC
 
-Bool Fluxbox::handleSignal(int sig) {
-	switch (sig) {
-	case SIGHUP:
-		load_rc();
-		break;
+void Fluxbox::handleEvent(SignalEvent * const sig) {
+	I18n *i18n = I18n::instance();
+	static int re_enter = 0;
 
-	case SIGUSR1:
-		reload_rc();
+	switch (sig->signum) {
+		case SIGCHLD: // we don't want the child process to kill us
+			waitpid(-1, 0, WNOHANG | WUNTRACED);
 		break;
-
-	case SIGUSR2:
-		rereadMenu();
+		case SIGHUP:
+			load_rc();
 		break;
+		case SIGUSR1:
+			reload_rc();
+		break;
+		case SIGUSR2:
+			rereadMenu();
+		break;
+		case SIGSEGV:
+			abort();
+		break;
+		case SIGFPE:
+		case SIGINT:
+		case SIGTERM:
+			shutdown();
+		break;
+		default:
+			fprintf(stderr,
+			i18n->getMessage(
+				 FBNLS::BaseDisplaySet, FBNLS::BaseDisplaySignalCaught,
+				 "%s:	signal %d caught\n"),
+				getApplicationName(), sig);
 
-	case SIGSEGV:
-	case SIGFPE:
-	case SIGINT:
-	case SIGTERM:
-		shutdown();
-	default:
-		return False;
+			if (! isStartup() && ! re_enter) {
+				re_enter = 1;
+				fprintf(stderr,
+					i18n->getMessage(
+						FBNLS::BaseDisplaySet, FBNLS::BaseDisplayShuttingDown,
+						"shutting down\n"));
+				shutdown();
+			}
+
+			
+			fprintf(stderr,
+			i18n->getMessage(
+				 FBNLS::BaseDisplaySet, FBNLS::BaseDisplayAborting,
+				 "aborting... dumping core\n"));
+			abort();
+		break;
 	}
 
-	return True;
+	
+
 }
 
 
 BScreen *Fluxbox::searchScreen(Window window) {
 	BScreen *screen = (BScreen *) 0;
-	std::list<BScreen *>::iterator it = screenList.begin();
-	std::list<BScreen *>::iterator it_end = screenList.end();
+	ScreenList::iterator it = screenList.begin();
+	ScreenList::iterator it_end = screenList.end();
 
 	for (; it != it_end; ++it) {
 		if (*it) {
@@ -1691,17 +1797,16 @@ void Fluxbox::shutdown(void) {
 
 	XSetInputFocus(getXDisplay(), PointerRoot, None, CurrentTime);
 
-	std::list<BScreen *>::iterator it = screenList.begin();
-	std::list<BScreen *>::iterator it_end = screenList.end();
+	//send shutdown to all screens
+	ScreenList::iterator it = screenList.begin();
+	ScreenList::iterator it_end = screenList.end();
 	for (; it != it_end; ++it) {
-		if(*it) {
+		if(*it)
 			(*it)->shutdown();
-		}
 	}
 
 	XSync(getXDisplay(), False);
 
-	save_rc();
 }
 
 //------ save_rc --------
@@ -1713,16 +1818,11 @@ void Fluxbox::save_rc(void) {
 	
 	char rc_string[1024];
 
-	auto_ptr<char> dbfile(getRcFilename());
-
-	// load_rc();
-	// This overwrites configs made while running, for example
-	// usage of iconbar and tabs
+	string dbfile(getRcFilename());
 	
-	
-	if (*dbfile) {
-		m_resourcemanager.save(dbfile.get(), dbfile.get());
-		m_screen_rm.save(dbfile.get(), dbfile.get());
+	if (dbfile.size() != 0) {
+		m_resourcemanager.save(dbfile.c_str(), dbfile.c_str());
+		m_screen_rm.save(dbfile.c_str(), dbfile.c_str());
 	} else
 		cerr<<"database filename is invalid!"<<endl;
 	
@@ -1736,8 +1836,8 @@ void Fluxbox::save_rc(void) {
 					(resource.auto_raise_delay.tv_usec / 1000)));
 	XrmPutLineResource(&new_blackboxrc, rc_string);
 
-	std::list<BScreen *>::iterator it = screenList.begin();
-	std::list<BScreen *>::iterator it_end = screenList.end();
+	ScreenList::iterator it = screenList.begin();
+	ScreenList::iterator it_end = screenList.end();
 
 	//Save screen resources
 
@@ -1868,40 +1968,45 @@ void Fluxbox::save_rc(void) {
 	
 	}
 
-	XrmDatabase old_blackboxrc = XrmGetFileDatabase(dbfile.get());
+	XrmDatabase old_blackboxrc = XrmGetFileDatabase(dbfile.c_str());
 
 	XrmMergeDatabases(new_blackboxrc, &old_blackboxrc); //merge database together
-	XrmPutFileDatabase(old_blackboxrc, dbfile.get());
+	XrmPutFileDatabase(old_blackboxrc, dbfile.c_str());
 	XrmDestroyDatabase(old_blackboxrc);
-	#ifdef DEBUG
+#ifdef DEBUG
 	cerr<<__FILE__<<"("<<__LINE__<<"): ------------ SAVING DONE"<<endl;	
-	#endif
+#endif // DEBUG
 }
 
 //-------- getRcFilename -------------
 // Returns filename of resource file
 //------------------------------------
-char *Fluxbox::getRcFilename() {
-	char *dbfile=0;
+string Fluxbox::getRcFilename() {
  
-	if (!rc_file) {		
-		string str(getenv("HOME")+string("/.")+RC_PATH+string("/")+RC_INIT_FILE);
-		return StringUtil::strdup(str.c_str());
+	if (rc_file.size() == 0) { // set default filename
+		string defaultfile(getenv("HOME")+string("/.")+RC_PATH+string("/")+RC_INIT_FILE);
+		return defaultfile;
+	}
 
-	} else
-		dbfile = StringUtil::strdup(rc_file);
- 
-	return dbfile;
+	return rc_file;
+}
+
+//-------- getDefaultDataFilename -------------
+// Provides default filename of data file
+//---------------------------------------------
+void Fluxbox::getDefaultDataFilename(char *name, string &filename) {
+	filename = string(getenv("HOME")+string("/.")+RC_PATH+string("/")+name);
 }
 
 void Fluxbox::load_rc(void) {
 	XrmDatabaseHelper database;
 	
 	//get resource filename
-	auto_ptr<char> dbfile(getRcFilename());
-	if (dbfile.get()) {
-		if (!m_resourcemanager.load(dbfile.get())) {
-			cerr<<"Faild to load database:"<<dbfile.get()<<endl;
+	string dbfile(getRcFilename());
+
+	if (dbfile.size() != 0) {
+		if (!m_resourcemanager.load(dbfile.c_str())) {
+			cerr<<"Faild to load database:"<<dbfile<<endl;
 			cerr<<"Trying with: "<<DEFAULT_INITFILE<<endl;
 			if (!m_resourcemanager.load(DEFAULT_INITFILE))
 				cerr<<"Faild to load database: "<<DEFAULT_INITFILE<<endl;
@@ -1915,31 +2020,34 @@ void Fluxbox::load_rc(void) {
 	char *value_type;
 
 	if (m_rc_menufile->size()) {
-		char *tmpvar =StringUtil::expandFilename(m_rc_menufile->c_str());
-		*m_rc_menufile = (tmpvar==0 ? "" : tmpvar);
+		*m_rc_menufile = StringUtil::expandFilename(*m_rc_menufile);
 		if (!m_rc_menufile->size())
 			m_rc_menufile.setDefaultValue();
-
-		delete tmpvar;
 	} else
 		m_rc_menufile.setDefaultValue();
+ 
+ 	if (m_rc_slitlistfile->size() != 0) {
+ 		*m_rc_slitlistfile = StringUtil::expandFilename(*m_rc_slitlistfile);
+ 	} else {
+ 		string filename;
+ 		getDefaultDataFilename("slitlist", filename);
+ 		m_rc_slitlistfile.setFromString(filename.c_str());
+ 	}
 
 	if (*m_rc_colors_per_channel < 2)
 		*m_rc_colors_per_channel = 2;
 	else if (*m_rc_colors_per_channel > 6)
 		*m_rc_colors_per_channel = 6;
 
-	if (*m_rc_stylefile=="")
+	if (m_rc_stylefile->size() == 0)
 		*m_rc_stylefile = DEFAULTSTYLE;
-	else {
-		auto_ptr<char> tmpvar(StringUtil::expandFilename(m_rc_stylefile->c_str()));
-		*m_rc_stylefile = (tmpvar.get()==0 ? "" : tmpvar.get());
-	}
+	else // expand tilde
+		*m_rc_stylefile = StringUtil::expandFilename(*m_rc_stylefile);
 
 	//load file
-	database = XrmGetFileDatabase(dbfile.get());
+	database = XrmGetFileDatabase(dbfile.c_str());
 	if (database==0) {
-		cerr<<"Fluxbox: Cant open "<<dbfile.get()<<" !"<<endl;
+		cerr<<"Fluxbox: Cant open "<<dbfile<<" !"<<endl;
 		cerr<<"Using: "<<DEFAULT_INITFILE<<endl;
 		database = XrmGetFileDatabase(DEFAULT_INITFILE);
 	}
@@ -1963,14 +2071,23 @@ void Fluxbox::load_rc(void) {
 		(resource.auto_raise_delay.tv_sec * 1000);
 	resource.auto_raise_delay.tv_usec *= 1000;
 
+	// expand tilde
+	*m_rc_groupfile = StringUtil::expandFilename(*m_rc_groupfile);
+
+#ifdef DEBUG
+	cerr<<__FILE__<<": Loading groups ("<<*m_rc_groupfile<<")"<<endl;
+#endif // DEBUG
+	if (!Workspace::loadGroups(*m_rc_groupfile)) {
+		cerr<<"Faild to load groupfile: "<<*m_rc_groupfile<<endl;
+	}
 }
 
 void Fluxbox::load_rc(BScreen *screen) {
 	//get resource filename
-	auto_ptr<char> dbfile(getRcFilename());
-	if (dbfile.get()) {
-		if (!m_screen_rm.load(dbfile.get())) {
-			cerr<<"Faild to load database:"<<dbfile.get()<<endl;
+	string dbfile(getRcFilename());
+	if (dbfile.size() != 0) {
+		if (!m_screen_rm.load(dbfile.c_str())) {
+			cerr<<"Faild to load database:"<<dbfile<<endl;
 			cerr<<"Trying with: "<<DEFAULT_INITFILE<<endl;
 			if (!m_screen_rm.load(DEFAULT_INITFILE))
 				cerr<<"Faild to load database: "<<DEFAULT_INITFILE<<endl;
@@ -1982,7 +2099,7 @@ void Fluxbox::load_rc(BScreen *screen) {
 	
 	XrmDatabaseHelper database;
 
-	database = XrmGetFileDatabase(dbfile.get());
+	database = XrmGetFileDatabase(dbfile.c_str());
 	if (database==0)
 		database = XrmGetFileDatabase(DEFAULT_INITFILE);
 		
@@ -2018,9 +2135,9 @@ void Fluxbox::load_rc(BScreen *screen) {
 	sprintf(class_lookup, "Session.Screen%d.WorkspaceNames", screen_number);
 	if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
 			&value)) {
-		#ifdef DEBUG
+#ifdef DEBUG
 		cerr<<__FILE__<<"("<<__LINE__<<"): Workspaces="<<screen->getNumberOfWorkspaces()<<endl;
-		#endif
+#endif // DEBUG
 		char *search = StringUtil::strdup(value.addr);
 
 		int i;
@@ -2028,7 +2145,7 @@ void Fluxbox::load_rc(BScreen *screen) {
 			char *nn;
 
 			if (! i) nn = strtok(search, ",");
-			else nn = strtok(NULL, ",");
+			else nn = strtok(0, ",");
 
 			if (nn)
 				screen->addWorkspaceName(nn);	
@@ -2044,54 +2161,54 @@ void Fluxbox::load_rc(BScreen *screen) {
 	if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
 			&value)) {
 		if (! strncasecmp(value.addr, "clicktofocus", value.size)) {
-			screen->saveAutoRaise(False);
-			screen->saveSloppyFocus(False);
-			screen->saveSemiSloppyFocus(False);
+			screen->saveAutoRaise(false);
+			screen->saveSloppyFocus(false);
+			screen->saveSemiSloppyFocus(false);
 
 		} else if (! strncasecmp(value.addr, "autoraisesloppyfocus", value.size)) {
-			screen->saveSemiSloppyFocus(False);
-			screen->saveSloppyFocus(True);
-			screen->saveAutoRaise(True);
+			screen->saveSemiSloppyFocus(false);
+			screen->saveSloppyFocus(true);
+			screen->saveAutoRaise(true);
 		} else if (! strncasecmp(value.addr, "autoraisesemisloppyfocus", value.size)) {
-			screen->saveSloppyFocus(False);
-			screen->saveSemiSloppyFocus(True);
-			screen->saveAutoRaise(True);
+			screen->saveSloppyFocus(false);
+			screen->saveSemiSloppyFocus(true);
+			screen->saveAutoRaise(true);
 
 		} else if (! strncasecmp(value.addr, "semisloppyfocus", value.size)) {
-			screen->saveSloppyFocus(False);
-			screen->saveSemiSloppyFocus(True);
-			screen->saveAutoRaise(False);
+			screen->saveSloppyFocus(false);
+			screen->saveSemiSloppyFocus(true);
+			screen->saveAutoRaise(false);
 
 		} else {
 
-			screen->saveSemiSloppyFocus(False);	
-			screen->saveSloppyFocus(True);
-			screen->saveAutoRaise(False);
+			screen->saveSemiSloppyFocus(false);	
+			screen->saveSloppyFocus(true);
+			screen->saveAutoRaise(false);
 		}
 	} else {
-		screen->saveSemiSloppyFocus(False);
-		screen->saveSloppyFocus(True); //TODO: fluxgen, shouldn't this be false?
-		screen->saveAutoRaise(False); //as click should be default, or?
+		screen->saveSemiSloppyFocus(false);
+		screen->saveSloppyFocus(true); 
+		screen->saveAutoRaise(false);
 	}
 
 	sprintf(name_lookup, "session.screen%d.windowPlacement", screen_number);
 	sprintf(class_lookup, "Session.Screen%d.WindowPlacement", screen_number);
 	if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
-			&value))
+			&value)) {
 		if (! strncasecmp(value.addr, "RowSmartPlacement", value.size))
 			screen->savePlacementPolicy(BScreen::ROWSMARTPLACEMENT);
 		else if (! strncasecmp(value.addr, "ColSmartPlacement", value.size))
 			screen->savePlacementPolicy(BScreen::COLSMARTPLACEMENT);
 		else
 			screen->savePlacementPolicy(BScreen::CASCADEPLACEMENT);
-	else
+	} else
 		screen->savePlacementPolicy(BScreen::ROWSMARTPLACEMENT);
 
 #ifdef SLIT
 	sprintf(name_lookup, "session.screen%d.slit.placement", screen_number);
 	sprintf(class_lookup, "Session.Screen%d.Slit.Placement", screen_number);
 	if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
-			&value))
+			&value)) {
 		if (! strncasecmp(value.addr, "TopLeft", value.size))
 			screen->saveSlitPlacement(Slit::TOPLEFT);
 		else if (! strncasecmp(value.addr, "CenterLeft", value.size))
@@ -2108,40 +2225,40 @@ void Fluxbox::load_rc(BScreen *screen) {
 			screen->saveSlitPlacement(Slit::BOTTOMRIGHT);
 		else
 			screen->saveSlitPlacement(Slit::CENTERRIGHT);
-	else
+	} else
 		screen->saveSlitPlacement(Slit::CENTERRIGHT);
 
 	sprintf(name_lookup, "session.screen%d.slit.direction", screen_number);
 	sprintf(class_lookup, "Session.Screen%d.Slit.Direction", screen_number);
 	if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
-			&value))
+			&value)) {
 		if (! strncasecmp(value.addr, "Horizontal", value.size))
 			screen->saveSlitDirection(Slit::HORIZONTAL);
 		else
 			screen->saveSlitDirection(Slit::VERTICAL);
-	else
+	} else
 		screen->saveSlitDirection(Slit::VERTICAL);
 
 	sprintf(name_lookup, "session.screen%d.slit.onTop", screen_number);
 	sprintf(class_lookup, "Session.Screen%d.Slit.OnTop", screen_number);
 	if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
-			&value))
+			&value)) {
 		if (! strncasecmp(value.addr, "True", value.size))
 			screen->saveSlitOnTop(True);
 		else
 			screen->saveSlitOnTop(False);
-	else
+	} else
 		screen->saveSlitOnTop(False);
 
 	sprintf(name_lookup, "session.screen%d.slit.autoHide", screen_number);
 	sprintf(class_lookup, "Session.Screen%d.Slit.AutoHide", screen_number);
 	if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
-			&value))
+			&value)) {
 		if (! strncasecmp(value.addr, "True", value.size))
 			screen->saveSlitAutoHide(True);
 		else
 			screen->saveSlitAutoHide(False);
-	else
+	} else
 		screen->saveSlitAutoHide(False);
 
 	#ifdef XINERAMA
@@ -2185,9 +2302,12 @@ void Fluxbox::load_rc(BScreen *screen) {
 	if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
 			&value)) {
 		int clock;
-		if (sscanf(value.addr, "%d", &clock) != 1) screen->saveClock24Hour(False);
-		else if (clock == 24) screen->saveClock24Hour(True);
-		else screen->saveClock24Hour(False);
+		if (sscanf(value.addr, "%d", &clock) != 1)
+			screen->saveClock24Hour(False);
+		else if (clock == 24) 
+			screen->saveClock24Hour(True);
+		else 
+			screen->saveClock24Hour(False);
 	} else
 		screen->saveClock24Hour(False);
 #endif // HAVE_STRFTIME
@@ -2211,9 +2331,9 @@ void Fluxbox::load_rc(BScreen *screen) {
 void Fluxbox::loadRootCommand(BScreen *screen)	{
 	XrmDatabase database = (XrmDatabase) 0;
  
-	auto_ptr<char> dbfile(getRcFilename());
+	string dbfile(getRcFilename());
 
-	database = XrmGetFileDatabase(dbfile.get());
+	database = XrmGetFileDatabase(dbfile.c_str());
 	if (!database) 
 		database = XrmGetFileDatabase(DEFAULT_INITFILE);
 
@@ -2229,30 +2349,29 @@ void Fluxbox::loadRootCommand(BScreen *screen)	{
 	
 }
 
-void Fluxbox::reload_rc(void) {
+void Fluxbox::reload_rc() {
 	load_rc();
 	reconfigure();
 }
 
 
-void Fluxbox::reconfigure(void) {
+void Fluxbox::reconfigure() {
 	reconfigure_wait = true;
 
-	if (! timer.isTiming()) timer.start();
+	if (! timer.isTiming()) 
+		timer.start();
 }
 
 
-void Fluxbox::real_reconfigure(void) {
-	BaseDisplay::GrabGuard gg(*this);
-	grab();
+void Fluxbox::real_reconfigure() {
 
 	XrmDatabase new_blackboxrc = (XrmDatabase) 0;
 
-	auto_ptr<char> dbfile(getRcFilename());
-	XrmDatabase old_blackboxrc = XrmGetFileDatabase(dbfile.get());
+	string dbfile(getRcFilename());
+	XrmDatabase old_blackboxrc = XrmGetFileDatabase(dbfile.c_str());
 
 	XrmMergeDatabases(new_blackboxrc, &old_blackboxrc);
-	XrmPutFileDatabase(old_blackboxrc, dbfile.get());
+	XrmPutFileDatabase(old_blackboxrc, dbfile.c_str());
 	
 	if (old_blackboxrc)
 		XrmDestroyDatabase(old_blackboxrc);
@@ -2271,21 +2390,17 @@ void Fluxbox::real_reconfigure(void) {
 	}
 	menuTimestamps.erase(menuTimestamps.begin(), menuTimestamps.end());
 
-	std::list<BScreen *>::iterator sit = screenList.begin();
-	std::list<BScreen *>::iterator sit_end = screenList.end();
-	for (; sit != sit_end; ++sit) {
+	ScreenList::iterator sit = screenList.begin();
+	ScreenList::iterator sit_end = screenList.end();
+	for (; sit != sit_end; ++sit)
 		(*sit)->reconfigure();
-	}
 	
 	//reconfigure keys
-	char *keyfilename = StringUtil::expandFilename(m_rc_keyfile->c_str());	
-	key->reconfigure(keyfilename);
-	delete keyfilename;
+	key->reconfigure(StringUtil::expandFilename(*m_rc_keyfile).c_str());
 
 	//reconfigure tabs
 	reconfigureTabs();
 
-	ungrab();
 }
 
 //------------- reconfigureTabs ----------
@@ -2293,8 +2408,8 @@ void Fluxbox::real_reconfigure(void) {
 // ---------------------------------------
 void Fluxbox::reconfigureTabs(void) {
 	//tab reconfiguring
-	std::map<Window, Tab *>::iterator it = tabSearch.begin();
-	std::map<Window, Tab *>::iterator it_end = tabSearch.end();
+	TabList::iterator it = tabSearch.begin();
+	TabList::iterator it_end = tabSearch.end();
 	//setting all to unconfigured
 	for (; it != it_end; ++it) {
 		it->second->setConfigured(false);
@@ -2393,15 +2508,14 @@ void Fluxbox::timeout(void) {
 	reconfigure_wait = reread_menu_wait = false;
 }
 
-
+// set focused window
 void Fluxbox::setFocusedWindow(FluxboxWindow *win) {
-	
 	BScreen *old_screen = 0, *screen = 0;
 	FluxboxWindow *old_win = 0;
 	Toolbar *old_tbar = 0, *tbar = 0;
 	Workspace *old_wkspc = 0, *wkspc = 0;
 
-	if (focused_window!=0) {
+	if (focused_window != 0) {
 		old_win = focused_window;
 		old_screen = old_win->getScreen();
 		
@@ -2414,20 +2528,27 @@ void Fluxbox::setFocusedWindow(FluxboxWindow *win) {
 	}
 
 	if (win && ! win->isIconic()) {
-		
-		screen = win->getScreen();
-		tbar = screen->getToolbar();
-		wkspc = screen->getWorkspace(win->getWorkspaceNumber());		
-		focused_window = win;
-		win->setFocusFlag(True);
-		wkspc->menu()->setItemSelected(win->getWindowNumber(), true);
-		
+		// make sure we have a valid win pointer with a valid screen
+		ScreenList::iterator winscreen = 
+			std::find(screenList.begin(), screenList.end(),
+				win->getScreen());
+		if (winscreen == screenList.end())
+			focused_window = 0; // the window pointer wasn't valid, mark no window focused
+		else {
+			screen = *winscreen;
+			tbar = screen->getToolbar();
+			wkspc = screen->getWorkspace(win->getWorkspaceNumber());		
+			focused_window = win;     // update focused window
+			win->setFocusFlag(True); // set focus flag
+			// select this window in workspace menu
+			wkspc->menu()->setItemSelected(win->getWindowNumber(), true);
+		}
 	} else
-		focused_window = (FluxboxWindow *) 0;
+		focused_window = 0;
 
-	if (tbar!=0)
+	if (tbar != 0)
 		tbar->redrawWindowLabel(True);
-	if (screen!=0)
+	if (screen != 0)
 		screen->updateNetizenWindowFocus();
 
 	if (old_tbar && old_tbar != tbar)
